<?php
namespace Litovchenko\AirTable\Controller;

/**
 * This file is part of the "air_table" Extension for TYPO3 CMS.
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 */
use TYPO3\CMS\Backend\Template\Components\ButtonBar;
use TYPO3\CMS\Backend\Utility\BackendUtility as BackendUtilityCore;
use TYPO3\CMS\Backend\View\BackendTemplateView;
use TYPO3\CMS\Core\FormProtection\FormProtectionFactory;
use TYPO3\CMS\Core\Imaging\Icon;
use TYPO3\CMS\Core\Imaging\IconFactory;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\HttpUtility;
use TYPO3\CMS\Core\Utility\MathUtility;
use TYPO3\CMS\Core\Utility\VersionNumberUtility;
use TYPO3\CMS\Extbase\Mvc\View\ViewInterface;
use TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder;
use Litovchenko\AirTable\Utility\BaseUtility;

/**
 * Administration controller
 * @AirTable:<BackendModule>
 * @AirTable\Parameter\Name:<Value>
 */
class ClassesController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    /**
     * TsConfig configuration
     *
     * @var array
     */
    protected $tsConfiguration = [];

    /**
     * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
     */
    protected $configurationManager;

    /**
     * Function will be called before every other action
     *
     */
    public function initializeAction()
    {
        parent::initializeAction();
    }

    /**
     * BackendTemplateContainer
     *
     * @var BackendTemplateView
     */
    protected $view;

    /**
     * @var IconFactory
     */
    protected $iconFactory;

    /**
     * Backend Template Container
     *
     * @var BackendTemplateView
     */
    protected $defaultViewObjectName = BackendTemplateView::class;

    /**
     * Set up the doc header properly here
     *
     * @param ViewInterface $view
     */
    protected function initializeView(ViewInterface $view)
    {
        $this->iconFactory = GeneralUtility::makeInstance(IconFactory::class);
        /** @var BackendTemplateView $view */
        parent::initializeView($view);
        $view->getModuleTemplate()->getDocHeaderComponent()->setMetaInformation([]);
        $pageRenderer = $this->view->getModuleTemplate()->getPageRenderer();
		$this->createMenu();
		$this->createButtons();
    }

    /**
     * Create menu
     *
     */
    protected function createMenu()
    {
    }

    /**
     * Create the panel of buttons
     *
     */
    protected function createButtons()
    {
        $buttonBar = $this->view->getModuleTemplate()->getDocHeaderComponent()->getButtonBar();

        // Refresh
        $path = VersionNumberUtility::convertVersionNumberToInteger(TYPO3_branch) >= VersionNumberUtility::convertVersionNumberToInteger('8.6') ? 'Resources/Private/Language/' : '';
        $refreshButton = $buttonBar->makeLinkButton()
            ->setHref(GeneralUtility::getIndpEnv('REQUEST_URI'))
            ->setTitle('Обновить')
            ->setIcon($this->iconFactory->getIcon('actions-refresh', Icon::SIZE_SMALL));
        $buttonBar->addButton($refreshButton, ButtonBar::BUTTON_POSITION_RIGHT);
    }


    /**
     * Main action for administration
     */
    public function indexAction()
    {
        $assignedValues = [
            'content_Table' => $this->getContent('Table'),
			'content_Table2' => $this->getContent('Table2'),
			'content_Field' => $this->getContent('Field'),
            'requestUri' => GeneralUtility::quoteJSvalue(rawurlencode(GeneralUtility::getIndpEnv('REQUEST_URI'))),
        ];
        $this->view->assignMultiple($assignedValues);
    }

    /**
     * Get content with Tree
     *
     * @param int $type
     * @return string
     */
    protected function getContent($type)
    {
		$content = '';
		switch($type){
			case 'Table':
				$models = BaseUtility::getLoaderClasses('Classes/Domain/');
				foreach($models as $class) {
					$class_parents = class_parents($class);
					if(in_array('Litovchenko\AirTable\Model\BaseCrud',$class_parents) && BaseUtility::getClassAnnotationValueNew($class,'AirTable') == 'Table'){
						$table = BaseUtility::getTableNameFromClass($class);
						// Получаем информацию о функция параметрах
						$class_methods_content = '';
						$class_methods = get_class_methods($class);
						foreach($class_methods as $kM => $vM){
							if(preg_match('/^parameterClass/is',$vM)){
								$class_methods_content .= '<code style="color: green;">@AirTable\Parameter\\'.str_replace('parameterClass','',$vM).':&lt;...&gt;</code><br />';
							}
						};
						$content .= '
							 <tr class="">
								<td>EXT:'.BaseUtility::getExtNameFromClassPath($class).'</td>
								<td style="vertical-align: top;">
									<div class="panel-group panel-group-rst" role="tablist" aria-multiselectable="true" style="margin-bottom: 0;">
										<div class="panel panel-default panel-version t3js-version-changes" data-version="9.5.x">
											<div class="panel-heading" role="tab" id="heading-0" style="padding: 8px 12px; background-color: #fafafa; color: #000;">
												<h2 class="panel-title">
													<a href="#version-'.md5($class).'" class="" data-toggle="collapse" aria-expanded="true" aria-controls="#version-'.md5($class).'">
														<code>\\'.$class.'</code>, <code>'.BaseUtility::getTableNameFromClass($class).'</code>
													</a>
												</h2>
											</div>
											<div class="panel-collapse collapse" id="version-'.md5($class).'" role="tabpanel" data-group-version="9.5.x" aria-expanded="true" style="">
												<div class="panel-body t3js-changelog-list" role="tablist" aria-multiselectable="false">
													<div style="overflow-x: auto; border: #cccccc 0px solid;">
														<div style="margin: 0px;">
														'.$class_methods_content.'
														<code style="color: gray;">public static function baseRTypes()</code><br />
														<code style="color: gray;">public static function baseTabs()</code><br />
														<code style="color: gray;">public static function postBuildConfiguration(&$configuration = [])</code>
														</div>
													</div>
												</div>
											</div>
										</div>
									</div>
								</td>
								<td>
									'.BaseUtility::getClassAnnotationValueNew($class,'AirTable\Parameter\Label').'<br />
									<span style="color:gray">'.BaseUtility::getClassAnnotationValueNew($class,'AirTable\Parameter\Description').'</span>
								</td>
							 </tr>
						';
					}
				}
			break;
			case 'Table2':
				$models = BaseUtility::getLoaderClasses('Classes/Domain/');
				foreach($models as $class) {
					$class_parents = class_parents($class);
					if(in_array('Litovchenko\AirTable\Model\BaseCrud2',$class_parents) && BaseUtility::getClassAnnotationValueNew($class,'AirTable') == 'Table'){
						$table = BaseUtility::getTableNameFromClass($class);
						// Получаем информацию о функция параметрах
						$class_methods_content = '';
						$class_methods = get_class_methods($class);
						foreach($class_methods as $kM => $vM){
							if(preg_match('/^parameterClass/is',$vM)){
								$class_methods_content .= '<code style="color: green;">@AirTable\Parameter\\'.str_replace('parameterClass','',$vM).':&lt;...&gt;</code><br />';
							}
						};
						$content .= '
							 <tr class="">
								<td>EXT:'.BaseUtility::getExtNameFromClassPath($class).'</td>
								<td style="vertical-align: top;">
									<div class="panel-group panel-group-rst" role="tablist" aria-multiselectable="true" style="margin-bottom: 0;">
										<div class="panel panel-default panel-version t3js-version-changes" data-version="9.5.x">
											<div class="panel-heading" role="tab" id="heading-0" style="padding: 8px 12px; background-color: #fafafa; color: #000;">
												<h2 class="panel-title">
													<a href="#version-'.md5($class).'" class="" data-toggle="collapse" aria-expanded="true" aria-controls="#version-'.md5($class).'">
														<code>\\'.$class.'</code>, <code>'.BaseUtility::getTableNameFromClass($class).'</code>
													</a>
												</h2>
											</div>
											<div class="panel-collapse collapse" id="version-'.md5($class).'" role="tabpanel" data-group-version="9.5.x" aria-expanded="true" style="">
												<div class="panel-body t3js-changelog-list" role="tablist" aria-multiselectable="false">
													<div style="overflow-x: auto; border: #cccccc 0px solid;">
														<div style="margin: 0px;">
														'.$class_methods_content.'
														<code style="color: gray;">public static function baseRTypes()</code><br />
														<code style="color: gray;">public static function baseTabs()</code><br />
														<code style="color: gray;">public static function postBuildConfiguration(&$configuration = [])</code>
														</div>
													</div>
												</div>
											</div>
										</div>
									</div>
								</td>
								<td>
									'.BaseUtility::getClassAnnotationValueNew($class,'AirTable\Parameter\Label').'<br />
									<span style="color:gray">'.BaseUtility::getClassAnnotationValueNew($class,'AirTable\Parameter\Description').'</span>
								</td>
							 </tr>
						';
					}
				}
			break;
			case 'Field':
				$fields = BaseUtility::getLoaderClasses('Classes/Form/Field/');
				foreach($fields as $class) {
					if(BaseUtility::getClassAnnotationValueNew($class,'AirTable') == 'Field'){
						$expimp = '';
						if($class::EXPORT == true){
							$expimp .= '<code class="label label-success">Экспорт</code> ';
						}
						if($class::IMPORT == true){
							$expimp .= '<code class="label label-danger">Импорт</code>';
						}
						// Получаем информацию о функция параметрах
						$class_methods_content = '';
						$class_methods = get_class_methods($class);
						foreach($class_methods as $kM => $vM){
							if(preg_match('/^parameter/is',$vM) && !preg_match('/^parameterClass/is',$vM)){
								if($vM == 'parameterDoNotCheck')
									continue;
								if($vM == 'parameterDoNotSqlAnalyze')
									continue;
								
								$class_methods_content .= '<code style="color: green;">@AirTable\Parameter\\'.str_replace('parameter','',$vM).':&lt;...&gt;</code><br />';
							}
						};
						$content .= '
						 <tr class="">
							<td>EXT:'.BaseUtility::getExtNameFromClassPath($class).'</td>
							<td style="vertical-align: top;">
								<div class="panel-group panel-group-rst" role="tablist" aria-multiselectable="true" style="margin-bottom: 0;">
									<div class="panel panel-default panel-version t3js-version-changes" data-version="9.5.x">
										<div class="panel-heading" role="tab" id="heading-0" style="padding: 8px 12px; background-color: #fafafa; color: #000;">
											<h2 class="panel-title">
												<a href="#version-'.md5($class).'" class="" data-toggle="collapse" aria-expanded="true" aria-controls="#version-'.md5($class).'">
													<code>\\'.$class.'</code>
												</a>
											</h2>
										</div>
										<div class="panel-collapse collapse" id="version-'.md5($class).'" role="tabpanel" data-group-version="9.5.x" aria-expanded="true" style="">
											<div class="panel-body t3js-changelog-list" role="tablist" aria-multiselectable="false">
												<div style="overflow-x: auto; border: #cccccc 0px solid;">
													<div style="margin: 0px;">'.$class_methods_content.'</div>
												</div>
											</div>
										</div>
									</div>
								</div>
							</td>
							<td><code>'.end(explode('\\',$class)).'</code><br />'.$expimp.'</td>
							<td>
								'.BaseUtility::getClassAnnotationValueNew($class,'AirTable\Parameter\Label').'<br />
								<span style="color:gray">'.BaseUtility::getClassAnnotationValueNew($class,'AirTable\Parameter\Description').'</span>
							</td>
						 </tr>
						 ';
					}
				}
			break;
		}
		return $content;
	}

}
