<?php
namespace Litovchenko\AirTable\Controller;

/**
 * This file is part of the "air_table" Extension for TYPO3 CMS.
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 */
use TYPO3\CMS\Backend\Template\Components\ButtonBar;
use TYPO3\CMS\Backend\Utility\BackendUtility as BackendUtilityCore;
use TYPO3\CMS\Backend\View\BackendTemplateView;
use TYPO3\CMS\Core\FormProtection\FormProtectionFactory;
use TYPO3\CMS\Core\Imaging\Icon;
use TYPO3\CMS\Core\Imaging\IconFactory;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\HttpUtility;
use TYPO3\CMS\Core\Utility\MathUtility;
use TYPO3\CMS\Core\Utility\VersionNumberUtility;
use TYPO3\CMS\Extbase\Mvc\View\ViewInterface;
use TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder;
use Litovchenko\AirTable\Utility\BaseUtility;

/**
 * @AirTable:<BackendModule>
 * @AirTable\Parameter\Name:<Value>
 */
class ExportXlsController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    /**
     * TsConfig configuration
     *
     * @var array
     */
    protected $tsConfiguration = [];

    /**
     * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
     */
    protected $configurationManager;

    /**
     * Function will be called before every other action
     *
     */
    public function initializeAction()
    {
		$extnameFromRoute = \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('route');
		$extnameFromRoute = explode('/',$extnameFromRoute);
		$extnameFromRoute = str_replace('AirTableList','',$extnameFromRoute[2]);
		
		$path_Typo3Ext = \TYPO3\CMS\Core\Core\Environment::getExtensionsPath().'/';
		$path_Typo3CoreExt = \TYPO3\CMS\Core\Core\Environment::getFrameworkBasePath().'/';
		$extPathList = array_merge(
			glob($path_Typo3Ext.'*/'), // typo3conf/ext/
			['--'],
			glob($path_Typo3CoreExt.'*/') // typo3/sysext/
		);
		
		foreach($extPathList as $k => $extPath){
			if($k == $extnameFromRoute){
				$this->extNameDir = basename($extPath);
				break;
			}
		}
        
		parent::initializeAction();
    }

    /**
     * BackendTemplateContainer
     *
     * @var BackendTemplateView
     */
    protected $view;

    /**
     * @var IconFactory
     */
    protected $iconFactory;

    /**
     * Backend Template Container
     *
     * @var BackendTemplateView
     */
    protected $defaultViewObjectName = BackendTemplateView::class;

    /**
     * Set up the doc header properly here
     *
     * @param ViewInterface $view
     */
    protected function initializeView(ViewInterface $view)
    {
        $this->iconFactory = GeneralUtility::makeInstance(IconFactory::class);
        /** @var BackendTemplateView $view */
        parent::initializeView($view);
        $view->getModuleTemplate()->getDocHeaderComponent()->setMetaInformation([]);

        $pageRenderer = $this->view->getModuleTemplate()->getPageRenderer();
		$pageRenderer->addJsFile('EXT:air_table/Resources/Public/Js/List.js');
		$pageRenderer->addCssFile('EXT:air_table/Resources/Public/Css/List.css');
		
		$this->createMenu();
		$this->createButtons();
    }

    /**
     * Create menu
     *
     */
    protected function createMenu()
    {
    }

    /**
     * Create the panel of buttons
     *
     */
    protected function createButtons()
    {
        $buttonBar = $this->view->getModuleTemplate()->getDocHeaderComponent()->getButtonBar();

        // (+) Кнопка корзина
		if($this->actionMethodName == 'indexAction'){
			if(BaseUtility::_GETuc('BackendDeleteScope',0,'AllController')){
				$trashCssClass = 'btn btn-success btn-sm';
				$getArgs = [
					'paramExt' => BaseUtility::_GETuc('paramExt',$this->extNameDir),
					'paramSubDomain' => BaseUtility::_GETuc('paramSubDomain','Root'),
					'paramClass' => BaseUtility::_GETuc('paramClass',''),
					'BackendDeleteScope' => 0
				];
			} else {
				$trashCssClass = 'btn btn-default btn-sm';
				$getArgs = [
					'paramExt' => BaseUtility::_GETuc('paramExt',$this->extNameDir),
					'paramSubDomain' => BaseUtility::_GETuc('paramSubDomain','Root'),
					'paramClass' => BaseUtility::_GETuc('paramClass',''),
					'BackendDeleteScope' => 1
				];
			}
			$uriBuilder = $this->controllerContext->getUriBuilder();
			$url = $uriBuilder->setArguments($getArgs)->uriFor(str_replace('Action','',$this->actionMethodName), [], $this->request->getControllerName());
			$viewButton = $buttonBar->makeLinkButton()
				->setShowLabelText(true)
				->setClasses($trashCssClass)
				->setHref($url)
				->setDataAttributes([
					'toggle' => 'tooltip',
					'placement' => 'bottom',
					'title' => '-'
				])
				->setTitle('Корзина записей')
				->setIcon($this->iconFactory->getIcon('actions-edit-delete', Icon::SIZE_SMALL));
			$buttonBar->addButton($viewButton, ButtonBar::BUTTON_POSITION_LEFT, 1);
		}
		
        // (+) Кнопка обновить
        $path = VersionNumberUtility::convertVersionNumberToInteger(TYPO3_branch) >= VersionNumberUtility::convertVersionNumberToInteger('8.6') ? 'Resources/Private/Language/' : '';
        $refreshButton = $buttonBar->makeLinkButton()
            ->setHref(GeneralUtility::getIndpEnv('REQUEST_URI'))
            ->setTitle('Обновить')
            ->setIcon($this->iconFactory->getIcon('actions-refresh', Icon::SIZE_SMALL));
        $buttonBar->addButton($refreshButton, ButtonBar::BUTTON_POSITION_RIGHT);
    }


    /**
     * Main action for administration
     */
    public function indexAction()
    {		
		$varContent = '';
		$paramExt = BaseUtility::_GETuc('paramExt','Root');
		$paramSubDomain = BaseUtility::_GETuc('paramSubDomain','Root');
		
		if($paramExt == 'Root'){
			$path_Typo3Ext = \TYPO3\CMS\Core\Core\Environment::getExtensionsPath().'/';
			$path_Typo3CoreExt = \TYPO3\CMS\Core\Core\Environment::getFrameworkBasePath().'/';
			$extPathList = array_merge(
				glob($path_Typo3Ext.'*/'), // typo3conf/ext/
				['--'],
				glob($path_Typo3CoreExt.'*/') // typo3/sysext/
			);
			foreach($extPathList as $k => $extPath){
				if($extPath == '--'){
					continue;
				}
				// Классы (до 4 уровней)
				// $extPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath($_EXTKEY);
				if(\TYPO3\CMS\Core\Utility\ExtensionManagementUtility::isLoaded(basename($extPath))){
					// if(is_file($extPath."Classes/Domain/_.txt") && file_exists($extPath."Classes/Domain/_.txt")){
						if(is_dir($extPath."Classes/Domain/Model/")){
							$extRelPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath(basename($extPath));
							if(file_exists($extRelPath.'Classes/Domain/_.txt')){
								$descriptionExt = file_get_contents($extRelPath.'Classes/Domain/_.txt');
							} else {
								$descriptionExt = '';
							}
							$getArgs = [
								'paramExt' => basename($extPath),
								'paramSubDomain' => 'Root'
							];
							$uriBuilder = $this->controllerContext->getUriBuilder();
							$url = $uriBuilder->setArguments($getArgs)->uriFor('index', [], $this->request->getControllerName());
							$varContent .= '
							<tr>
								<td colspan="3"><a href="'.$url.'">'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_extension', Icon::SIZE_SMALL).' '.
									'EXT:'.basename($extPath).'</a>
									<!--<br /><i style="color:gray">'.$descriptionExt.'</i>-->
								</td>
							</tr>';
						}
					// }
				}
			}
		} else {
			if($paramSubDomain == 'Root'){
				$getArgs = [
					'paramExt' => 'Root',
					'paramSubDomain' => 'Root'
				];
				$uriBuilder = $this->controllerContext->getUriBuilder();
				$url = $uriBuilder->setArguments($getArgs)->uriFor('index', [], $this->request->getControllerName());
				$varContent .= '
					<tr>
						<td colspan="3"><a href="'.$url.'">'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_subdomain', Icon::SIZE_SMALL).' '.
							'..</a>
						</td>
					</tr>';
			}
		
			$extRelPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath($paramExt);
			if(file_exists($extRelPath.'Classes/Domain/_.txt')){
				$titleExt = 'EXT:'.$paramExt . ' <br /> ' . file_get_contents($extRelPath.'Classes/Domain/_.txt');
			} else {
				$titleExt = 'EXT:'.$paramExt;
			}
			
			// 1
			// $varTabLabel = $this->iconFactory->getIcon('extensions-air_table-icon_storage_extension', Icon::SIZE_SMALL).' ';
			// $varTabLabel .= $titleExt;
			
			// 2
			$classList = [];
			$models = BaseUtility::getLoaderClasses('Classes/Domain/');
			foreach($models as $class) {
				$class_parents = class_parents($class);
				// Регистрация новых пользовательских моделей
				if(in_array('Litovchenko\AirTable\Model\BaseCrud',$class_parents) && BaseUtility::getClassAnnotationValueNew($class,'AirTable') == 'Table'){
					$extName = BaseUtility::getExtNameFromClassPath($class);
					$subDomain = BaseUtility::getSubDomainNameFromClassPath($class);
					$classList[$extName][$subDomain][] = $class;
				}
				// Регистрация моделей на основе существующей таблицы TYPO3
				if(in_array('Litovchenko\AirTable\Model\BaseCrud2',$class_parents) && BaseUtility::getClassAnnotationValueNew($class,'AirTable') == 'Table'){
					$extName = BaseUtility::getExtNameFromClassPath($class);
					$subDomain = BaseUtility::getSubDomainNameFromClassPath($class);
					$classList[$extName][$subDomain][] = $class;
				}
			}
			
			// Сортировка
			foreach($classList as $extName => $subDomain){
				ksort($classList[$extName]);
				// sort($classList[$extName][$subDomain]);
			}

			foreach($classList[$paramExt] as $subDomain => $classes){
				if($subDomain != $paramSubDomain && $paramSubDomain == 'Root'){
					$extRelPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath($paramExt);
					if(file_exists($extRelPath.'Classes/Domain/Model/'.$subDomain.'/_.txt')){
						$titleSubDomain = file_get_contents($extRelPath.'Classes/Domain/Model/'.$subDomain.'/_.txt');
					} else {
						$titleSubDomain = $DomainModelKey;
					}
					$getArgs = [
						'paramExt' => $paramExt,
						'paramSubDomain' => $subDomain
					];
					$uriBuilder = $this->controllerContext->getUriBuilder();
					$url = $uriBuilder->setArguments($getArgs)->uriFor('index', [], $this->request->getControllerName());
					$varContent .= '
					<tr>
						<td colspan="3"><a href="'.$url.'">'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_subdomain', Icon::SIZE_SMALL).' '.
							$titleSubDomain.'</a>
						</td>
					</tr>';
				} elseif($paramSubDomain != 'Root') {
					$getArgs = [
						'paramExt' => $paramExt,
						'paramSubDomain' => 'Root'
					];
					$uriBuilder = $this->controllerContext->getUriBuilder();
					$url = $uriBuilder->setArguments($getArgs)->uriFor('index', [], $this->request->getControllerName());
					$varContent .= '
					<tr>
						<td colspan="3"><a href="'.$url.'">'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_subdomain', Icon::SIZE_SMALL).' '.
							'..</a>
						</td>
					</tr>';
					break;
				}
			}
			
			$countJ = 0;
			foreach($classList[$paramExt] as $subDomain => $classes){
				if($subDomain == $paramSubDomain){
					foreach($classes as $k => $v){
						if(BaseUtility::isModelCategoryForAnotherModel($v)){
							continue;
						}
						$dbCount = $v::count();
						$getArgs = [
							'paramExt' => $paramExt,
							'paramSubDomain' => $paramSubDomain,
							'paramClass' => $v, // table
						];
						$uriBuilder = $this->controllerContext->getUriBuilder();
						$url = $uriBuilder->setArguments($getArgs)->uriFor('step2', [], $this->request->getControllerName());
						if(BaseUtility::isModelSupportOneCategory($v) || BaseUtility::isModelSupportManyCategories($v)){
							$iconCat = $this->iconFactory->getIcon('actions-view-go-forward', Icon::SIZE_SMALL).' ';
							$modelCat = $v.'Category';
							$countCat = $modelCat::count();
							$getArgsCat = [
								'paramExt' => $paramExt,
								'paramSubDomain' => $paramSubDomain,
								'paramClass' => $v.'Category', // table
							];
							$uriBuilderCat = $this->controllerContext->getUriBuilder();
							$urlCat = $uriBuilderCat->setArguments($getArgsCat)->uriFor('step2', [], $this->request->getControllerName());
							
							// Access (Cat)
							$accessOverlay = 'default';
							if(BaseUtility::BeUserAccessTableSelect(BaseUtility::getTableNameFromClass($v.'Category')) == false){
								$urlCat = '#';
								$accessOverlay = 'danger';
							}
							
							$linkCat = '<a href="'.$urlCat.'" class="label label-'.$accessOverlay.'">Категории ('.$countCat.')</a>';
							// $linkCat .= '&nbsp;'.$this->iconFactory->getIcon('actions-view-go-forward', Icon::SIZE_SMALL);
						} else {
							$linkCat = '';
						}
						// Access
						$accessOverlay = '';
						if(BaseUtility::BeUserAccessTableSelect(BaseUtility::getTableNameFromClass($v)) == false){
							$url = '#';
							$accessOverlay = 'overlay-locked';
						}
						$varContent .= '
						<tr>
							<td width="98%">
								<a href="'.$url.'">'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_tablerecord', Icon::SIZE_SMALL, $accessOverlay).' '.
									BaseUtility::getClassAnnotationValueNew($v,'AirTable\Parameter\Label').'
								</a>
								'.$linkCat.'
								<!--<br /><i style="color:gray">'.BaseUtility::getClassAnnotationValueNew($v,'AirTable\Parameter\Description').'</i>-->
							</td>
							<td width="*" nowrap>
								<code>\\'.$v.'</code><br />
								<!--<code>'.BaseUtility::getTableNameFromClass($v).'</code>-->
							</td>
							<td width="*" nowrap>
								Записей: '.intval($dbCount).' шт.
							</td>
						</tr>';
						$countJ++;
					}
				}
			}
		}
		
		// Если нет записей
		if($countJ == 0 && $paramExt != 'Root'){
			$varContent .= '
			<tr>
				<td>
					'.$this->iconFactory->getIcon('install-check-extables', Icon::SIZE_SMALL).'
					Таблицы не найдены
				</td>
			</tr>';
		}
		
        $assignedValues = [
            'content' => $varContent
        ];
        $this->view->assignMultiple($assignedValues);
    }

    /**
     * Main action for administration
     */
    public function step2Action()
    {
		$content = '';
		$paramExt = BaseUtility::_GETuc('paramExt','Root');
		$paramSubDomain = BaseUtility::_GETuc('paramSubDomain','Root');
		$paramClass = BaseUtility::_GETuc('paramClass','');
		
		// Создаем модель
		// $sqlBuilderSelect = $paramClass::getModel();
		$sqlBuilderCount = $paramClass::getModel();
		$table = BaseUtility::getTableNameFromClass($paramClass);
		
		// Колонки
		// tab 3 (columns)
		$columns = $GLOBALS['TCA'][$table]['columns'];
		foreach($columns as $k => $v){
			// Pid Пропускаем, если записи всегда в корне
			if($GLOBALS['TCA'][$table]['ctrl']['rootLevel'] == 1 && $k == 'pid'){
				continue;
			}
			
			$checked = '';
			$disabled = '';
			if($k == 'uid' || $k == 'pid' || $k == 'RType'){
				$checked = 'checked';
				$disabled = 'disabled';
			}
			$AirTableFieldClass = $v['AirTable.Class'];
			if(method_exists($AirTableFieldClass, 'exportControllerRow') && $AirTableFieldClass::EXPORT == false){
				$disabled = 'disabled';
			}
			if(in_array($k,BaseUtility::_POSTuc('form3Apply','form3Columns'))){ // || BaseUtility::_POSTuc('form3Apply','form3Columns','all') == 'all'
				$checked = 'checked';
			}
			$noConfig = '';
			if(!$v['AirTable.Class']){
				$disabled = 'disabled';
				$noConfig = 'color: red';
			}
			$debugStr = '';
			if($GLOBALS['TYPO3_CONF_VARS']['BE']['debug'] == true){
				$debugStr = '<br /><code style="color: #b4b2b2;">'.$v['AirTable.Class'].'</code>';
			}
			$content .= "
			<tr class=''>
				<td width='40%' align='right'>
					<b style='".$noConfig."'>
						".htmlspecialchars($GLOBALS['LANG']->sL($v['label']))."
						".$debugStr."
					</b>
				</td>
				<td width='20%'>
					<label class='btn btn-default btn-sm' style='margin: 0; margin-bottom: 0px; padding: 0 7px 0 7px; width: 100%; text-align: left; cursor: pointer;'>
						<input type='checkbox' name='columns[]' class='btn btn-default btn-sm' style='margin-top: 0px;' value='".$k."' ".$disabled." ".$checked."> 
						Включить в список
					</label>
				</label>
				</td>
				<td width='40%'>
					<b style='".$noConfig."'>".$k."</b>
				</td>
			</tr>
			 ";
		}
		
		// Делаем выгрузку файла
		if($this->request->getControllerName() == 'ExportXls'){
			$format = '.xlsx';
		
		} elseif($this->request->getControllerName() == 'ExportCsv'){
			$format = '.csv';
		}
		
        $assignedValues = [
			'model' => $paramClass,
			'table' => $table,
			'tableCount' => intval($sqlBuilderCount->count()),
            'content' => $content,
			'format' => $format
        ];
        $this->view->assignMultiple($assignedValues);
    }
	
    /**
     * Main action for administration
     */
    public function step2ProcessAction()
    {
		// Параметры _GET
		$paramExt = BaseUtility::_GETuc('paramExt',$this->extNameDir);
		$paramSubDomain = BaseUtility::_GETuc('paramSubDomain','Root');
		$paramClass = BaseUtility::_GETuc('paramClass','');
		
		// Параметры _POST
		$_POST = \TYPO3\CMS\Core\Utility\GeneralUtility::_POST();
		
		// Создаем модель
		$sqlBuilderSelect = $_POST['model']::getModel();
		$sqlBuilderCount = $_POST['model']::getModel();
		$table = BaseUtility::getTableNameFromClass($_POST['model']);
		
		// 1 // select
		$columns = $GLOBALS['TCA'][$table]['columns'];
		foreach($columns as $k => $v){
		
			// Оставляем только выбранные колонки
			#if((in_array($k, BaseUtility::_POSTuc('form3Apply','tab3_columns','all')) && BaseUtility::_POSTuc('form3Apply','tab3_columns','all') != 'all') 
			#	|| ($k == 'uid' && BaseUtility::_POSTuc('form3Apply','tab3_columns','all') != 'all')){
			#		$sqlBuilderSelect = $sqlBuilderSelect->addSelect($table.'.'.$k);
			#}
			
			$AirTableFieldClass = $v['AirTable.Class'];
			if(in_array('filter',$_POST['params'])){
				if(method_exists($AirTableFieldClass, 'listControllerSqlBuilder')){ // Фильтр
					$sqlBuilderSelect = $AirTableFieldClass::listControllerSqlBuilder($this, $sqlBuilderSelect, $table, $k, $v);
				}
			}
			if(method_exists($AirTableFieldClass, 'listControllerSqlBuilderWith')){ // Присоединение связей
				$sqlBuilderSelect = $AirTableFieldClass::listControllerSqlBuilderWith($this, $sqlBuilderSelect, $table, $k, $v);
			}
		}
		
		// 3 // order
		if(in_array('order',$_POST['params'])){
			$paramOrder = BaseUtility::_POSTuc('form2Apply','form2Order');
			foreach($paramOrder as $k => $v){
				if($v != null){
					$sqlBuilderSelect = $sqlBuilderSelect->orderBy($table.'.'.$k,$v);
				}
			}
		}
		
		// 4 // limit
		if(in_array('limit',$_POST['params'])){
			$page = BaseUtility::_POSTuc('form0Apply','form0Page',1);
			$lim = BaseUtility::_POSTuc('form3Apply','form3Limit',30); // 30
			if($lim>=0){
				$rowsCount = $sqlBuilderSelect->count(); // Кол-во строк всего в данном запросе
				if($page == 1){
					$sqlBuilderSelect = $sqlBuilderSelect->limit($lim)->offset(0);
					$SqlPageGo = 1;
				} else {
					$sqlBuilderSelect = $sqlBuilderSelect->limit($lim)->offset($lim*($page-1));
					$SqlPageGo = 1;
				}
			}
		}
		
		$rows = $sqlBuilderSelect->get()->toArray();
		$rowsCountDisplay = count($rows); // Кол-во выбранных строк 
		#$rowsCountAllInDb = $sqlBuilderCount->count(); // Кол-во записей в базе всего
		
		// Делаем выгрузку файла
		if($this->request->getControllerName() == 'ExportXls'){
			$pathFolder = '/fileadmin/_xlsx-export_/';
			$fileExportName = $table.'-'.date("d.m.Y-H.i.s").'.xlsx';
		
		} elseif($this->request->getControllerName() == 'ExportCsv'){
			$pathFolder = '/fileadmin/_csv-export_/';
			$fileExportName = $table.'-'.date("d.m.Y-H.i.s").'.csv';
		}
		
		$pathSite = \TYPO3\CMS\Core\Core\Environment::getPublicPath();
		\TYPO3\CMS\Core\Utility\GeneralUtility::mkdir($pathSite.$pathFolder);
		
		(new \Rap2hpoutre\FastExcel\FastExcel($rows))->export($pathSite.$pathFolder.$fileExportName, function ($line) {
			$_POST = \TYPO3\CMS\Core\Utility\GeneralUtility::_POST();
			$table = BaseUtility::getTableNameFromClass($_POST['model']);
			$rowData = [];
			
			$columns = $GLOBALS['TCA'][$table]['columns'];
			foreach($columns as $k => $v){
				// Pid Пропускаем, если записи всегда в корне
				if($GLOBALS['TCA'][$table]['ctrl']['rootLevel'] == 1 && $k == 'pid'){
					continue;
				}
			
				if(in_array($k,$_POST['columns']) || $k == 'uid' || $k == 'pid' || $k == 'RType'){
					$AirTableFieldClass = $v['AirTable.Class'];
					if(method_exists($AirTableFieldClass, 'exportControllerRow') && $AirTableFieldClass::EXPORT == true){
						$AirTableFieldClass::exportControllerRow($rowData, $_POST['model'], $table, $k, $v, $line);
					}
				}
			}
			
			return $rowData;
		});
		
		$getArgs = [
			'pathFolder' => $pathFolder,
			'file' => $fileExportName,
			'count' => count($rows),
		];
		$uriBuilder = $this->controllerContext->getUriBuilder();
		$redirectUrl = $uriBuilder->setArguments($getArgs)->uriFor('step3', [], $this->request->getControllerName());
		\TYPO3\CMS\Core\Utility\HttpUtility::redirect($redirectUrl);
	}

    /**
     * Main action for administration
     */
    public function step3Action()
    {
		$_GET = \TYPO3\CMS\Core\Utility\GeneralUtility::_GET();
		$pathSite = \TYPO3\CMS\Core\Core\Environment::getPublicPath();
		
		$fileContent = '';
		if($this->request->getControllerName() == 'ExportCsv'){
			$fContent = file_get_contents($pathSite.$_GET['pathFolder'].$_GET['file']);
		}
		
        $assignedValues = [
			'fileContent' => $fContent,
			'pathFolder' => $_GET['pathFolder'],
			'file' => $_GET['file'],
			'count' => intval($_GET['count'])
        ];
        $this->view->assignMultiple($assignedValues);
	}

}
