<?php
namespace Litovchenko\AirTable\Controller;

/**
 * This file is part of the "air_table" Extension for TYPO3 CMS.
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 */
use TYPO3\CMS\Backend\Template\Components\ButtonBar;
use TYPO3\CMS\Backend\Utility\BackendUtility as BackendUtilityCore;
use TYPO3\CMS\Backend\View\BackendTemplateView;
use TYPO3\CMS\Core\FormProtection\FormProtectionFactory;
use TYPO3\CMS\Core\Imaging\Icon;
use TYPO3\CMS\Core\Imaging\IconFactory;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\HttpUtility;
use TYPO3\CMS\Core\Utility\MathUtility;
use TYPO3\CMS\Core\Utility\VersionNumberUtility;
use TYPO3\CMS\Extbase\Mvc\View\ViewInterface;
use TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder;
use Illuminate\Database\Capsule\Manager as DB;
use Litovchenko\AirTable\Utility\BaseUtility;

/**
 * @AirTable:<BackendModule>
 * @AirTable\Parameter\Name:<Value>
 */
class ImportXlsController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    /**
     * TsConfig configuration
     *
     * @var array
     */
    protected $tsConfiguration = [];

    /**
     * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
     */
    protected $configurationManager;

    /**
     * Function will be called before every other action
     *
     */
    public function initializeAction()
    {
		$extnameFromRoute = \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('route');
		$extnameFromRoute = explode('/',$extnameFromRoute);
		$extnameFromRoute = str_replace('AirTableList','',$extnameFromRoute[2]);
		
		$path_Typo3Ext = \TYPO3\CMS\Core\Core\Environment::getExtensionsPath().'/';
		$path_Typo3CoreExt = \TYPO3\CMS\Core\Core\Environment::getFrameworkBasePath().'/';
		$extPathList = array_merge(
			glob($path_Typo3Ext.'*/'), // typo3conf/ext/
			['--'],
			glob($path_Typo3CoreExt.'*/') // typo3/sysext/
		);
		
		foreach($extPathList as $k => $extPath){
			if($k == $extnameFromRoute){
				$this->extNameDir = basename($extPath);
				break;
			}
		}
        
		parent::initializeAction();
    }

    /**
     * BackendTemplateContainer
     *
     * @var BackendTemplateView
     */
    protected $view;

    /**
     * @var IconFactory
     */
    protected $iconFactory;

    /**
     * Backend Template Container
     *
     * @var BackendTemplateView
     */
    protected $defaultViewObjectName = BackendTemplateView::class;

    /**
     * Set up the doc header properly here
     *
     * @param ViewInterface $view
     */
    protected function initializeView(ViewInterface $view)
    {
        $this->iconFactory = GeneralUtility::makeInstance(IconFactory::class);
        /** @var BackendTemplateView $view */
        parent::initializeView($view);
        $view->getModuleTemplate()->getDocHeaderComponent()->setMetaInformation([]);

        $pageRenderer = $this->view->getModuleTemplate()->getPageRenderer();
		$pageRenderer->addJsFile('EXT:air_table/Resources/Public/Js/List.js');
		$pageRenderer->addCssFile('EXT:air_table/Resources/Public/Css/List.css');
		
		$this->createMenu();
		$this->createButtons();
    }

    /**
     * Create menu
     *
     */
    protected function createMenu()
    {
    }

    /**
     * Create the panel of buttons
     *
     */
    protected function createButtons()
    {
        $buttonBar = $this->view->getModuleTemplate()->getDocHeaderComponent()->getButtonBar();
		
        // (+) Кнопка корзина
		if($this->actionMethodName == 'indexAction'){
			if(BaseUtility::_GETuc('BackendDeleteScope',0,'AllController')){
				$trashCssClass = 'btn btn-success btn-sm';
				$getArgs = [
					'paramExt' => BaseUtility::_GETuc('paramExt',$this->extNameDir),
					'paramSubDomain' => BaseUtility::_GETuc('paramSubDomain','Root'),
					'paramClass' => BaseUtility::_GETuc('paramClass',''),
					'BackendDeleteScope' => 0
				];
			} else {
				$trashCssClass = 'btn btn-default btn-sm';
				$getArgs = [
					'paramExt' => BaseUtility::_GETuc('paramExt',$this->extNameDir),
					'paramSubDomain' => BaseUtility::_GETuc('paramSubDomain','Root'),
					'paramClass' => BaseUtility::_GETuc('paramClass',''),
					'BackendDeleteScope' => 1
				];
			}
			$uriBuilder = $this->controllerContext->getUriBuilder();
			$url = $uriBuilder->setArguments($getArgs)->uriFor(str_replace('Action','',$this->actionMethodName), [], $this->request->getControllerName());
			$viewButton = $buttonBar->makeLinkButton()
				->setShowLabelText(true)
				->setClasses($trashCssClass)
				->setHref($url)
				->setDataAttributes([
					'toggle' => 'tooltip',
					'placement' => 'bottom',
					'title' => '-'
				])
				->setTitle('Корзина записей')
				->setIcon($this->iconFactory->getIcon('actions-edit-delete', Icon::SIZE_SMALL));
			$buttonBar->addButton($viewButton, ButtonBar::BUTTON_POSITION_LEFT, 1);
		}
		
        // (+) Кнопка обновить
        $path = VersionNumberUtility::convertVersionNumberToInteger(TYPO3_branch) >= VersionNumberUtility::convertVersionNumberToInteger('8.6') ? 'Resources/Private/Language/' : '';
        $refreshButton = $buttonBar->makeLinkButton()
            ->setHref(GeneralUtility::getIndpEnv('REQUEST_URI'))
            ->setTitle('Обновить')
            ->setIcon($this->iconFactory->getIcon('actions-refresh', Icon::SIZE_SMALL));
        $buttonBar->addButton($refreshButton, ButtonBar::BUTTON_POSITION_RIGHT);
    }


    /**
     * Main action for administration
     */
    public function indexAction()
    {
		$varContent = '';
		$paramExt = BaseUtility::_GETuc('paramExt','Root');
		$paramSubDomain = BaseUtility::_GETuc('paramSubDomain','Root');
		
		if($paramExt == 'Root'){
			$path_Typo3Ext = \TYPO3\CMS\Core\Core\Environment::getExtensionsPath().'/';
			$path_Typo3CoreExt = \TYPO3\CMS\Core\Core\Environment::getFrameworkBasePath().'/';
			$extPathList = array_merge(
				glob($path_Typo3Ext.'*/'), // typo3conf/ext/
				['--'],
				glob($path_Typo3CoreExt.'*/') // typo3/sysext/
			);
			foreach($extPathList as $k => $extPath){
				if($extPath == '--'){
					continue;
				}
				// Классы (до 4 уровней)
				// $extPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath($_EXTKEY);
				if(\TYPO3\CMS\Core\Utility\ExtensionManagementUtility::isLoaded(basename($extPath))){
					// if(is_file($extPath."Classes/Domain/_.txt") && file_exists($extPath."Classes/Domain/_.txt")){
						if(is_dir($extPath."Classes/Domain/Model/")){
							$extRelPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath(basename($extPath));
							if(file_exists($extRelPath.'Classes/Domain/_.txt')){
								$descriptionExt = file_get_contents($extRelPath.'Classes/Domain/_.txt');
							} else {
								$descriptionExt = '';
							}
							$getArgs = [
								'paramExt' => basename($extPath),
								'paramSubDomain' => 'Root'
							];
							$uriBuilder = $this->controllerContext->getUriBuilder();
							$url = $uriBuilder->setArguments($getArgs)->uriFor('index', [], $this->request->getControllerName());
							$varContent .= '
							<tr>
								<td colspan="3"><a href="'.$url.'">'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_extension', Icon::SIZE_SMALL).' '.
									'EXT:'.basename($extPath).'</a>
									<!--<br /><i style="color:gray">'.$descriptionExt.'</i>-->
								</td>
							</tr>';
						}
					// }
				}
			}
		} else {
			if($paramSubDomain == 'Root'){
				$getArgs = [
					'paramExt' => 'Root',
					'paramSubDomain' => 'Root'
				];
				$uriBuilder = $this->controllerContext->getUriBuilder();
				$url = $uriBuilder->setArguments($getArgs)->uriFor('index', [], $this->request->getControllerName());
				$varContent .= '
					<tr>
						<td colspan="3"><a href="'.$url.'">'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_subdomain', Icon::SIZE_SMALL).' '.
							'..</a>
						</td>
					</tr>';
			}
			
			$extRelPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath($paramExt);
			if(file_exists($extRelPath.'Classes/Domain/_.txt')){
				$titleExt = 'EXT:'.$paramExt . ' <br /> ' . file_get_contents($extRelPath.'Classes/Domain/_.txt');
			} else {
				$titleExt = 'EXT:'.$paramExt;
			}
			
			// 1
			// $varTabLabel = $this->iconFactory->getIcon('extensions-air_table-icon_storage_extension', Icon::SIZE_SMALL).' ';
			// $varTabLabel .= $titleExt;
			
			// 2
			$classList = [];
			$models = BaseUtility::getLoaderClasses('Classes/Domain/');
			foreach($models as $class) {
				$class_parents = class_parents($class);
				// Регистрация новых пользовательских моделей
				if(in_array('Litovchenko\AirTable\Model\BaseCrud',$class_parents) && BaseUtility::getClassAnnotationValueNew($class,'AirTable') == 'Table'){
					$extName = BaseUtility::getExtNameFromClassPath($class);
					$subDomain = BaseUtility::getSubDomainNameFromClassPath($class);
					$classList[$extName][$subDomain][] = $class;
				}
				// Регистрация моделей на основе существующей таблицы TYPO3
				if(in_array('Litovchenko\AirTable\Model\BaseCrud2',$class_parents) && BaseUtility::getClassAnnotationValueNew($class,'AirTable') == 'Table'){
					$extName = BaseUtility::getExtNameFromClassPath($class);
					$subDomain = BaseUtility::getSubDomainNameFromClassPath($class);
					$classList[$extName][$subDomain][] = $class;
				}
			}
			
			// Сортировка
			foreach($classList as $extName => $subDomain){
				ksort($classList[$extName]);
				// sort($classList[$extName][$subDomain]);
			}

			foreach($classList[$paramExt] as $subDomain => $classes){
				if($subDomain != $paramSubDomain && $paramSubDomain == 'Root'){
					$extRelPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath($paramExt);
					if(file_exists($extRelPath.'Classes/Domain/Model/'.$subDomain.'/_.txt')){
						$titleSubDomain = file_get_contents($extRelPath.'Classes/Domain/Model/'.$subDomain.'/_.txt');
					} else {
						$titleSubDomain = $DomainModelKey;
					}
					$getArgs = [
						'paramExt' => $paramExt,
						'paramSubDomain' => $subDomain
					];
					$uriBuilder = $this->controllerContext->getUriBuilder();
					$url = $uriBuilder->setArguments($getArgs)->uriFor('index', [], $this->request->getControllerName());
					$varContent .= '
					<tr>
						<td colspan="3"><a href="'.$url.'">'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_subdomain', Icon::SIZE_SMALL).' '.
							$titleSubDomain.'</a>
						</td>
					</tr>';
				} elseif($paramSubDomain != 'Root') {
					$getArgs = [
						'paramExt' => $paramExt,
						'paramSubDomain' => 'Root'
					];
					$uriBuilder = $this->controllerContext->getUriBuilder();
					$url = $uriBuilder->setArguments($getArgs)->uriFor('index', [], $this->request->getControllerName());
					$varContent .= '
					<tr>
						<td colspan="3"><a href="'.$url.'">'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_subdomain', Icon::SIZE_SMALL).' '.
							'..</a>
						</td>
					</tr>';
					break;
				}
			}
			
			$countJ = 0;
			foreach($classList[$paramExt] as $subDomain => $classes){
				if($subDomain == $paramSubDomain){
					foreach($classes as $k => $v){
						if(BaseUtility::isModelCategoryForAnotherModel($v)){
							continue;
						}
						$dbCount = $v::count();
						$getArgs = [
							'paramExt' => $paramExt,
							'paramSubDomain' => $paramSubDomain,
							'paramClass' => $v, // table
						];
						$uriBuilder = $this->controllerContext->getUriBuilder();
						$url = $uriBuilder->setArguments($getArgs)->uriFor('step2', [], $this->request->getControllerName());
						if(BaseUtility::isModelSupportOneCategory($v) || BaseUtility::isModelSupportManyCategories($v)){
							$iconCat = $this->iconFactory->getIcon('actions-view-go-forward', Icon::SIZE_SMALL).' ';
							$modelCat = $v.'Category';
							$countCat = $modelCat::count();
							$getArgsCat = [
								'paramExt' => $paramExt,
								'paramSubDomain' => $paramSubDomain,
								'paramClass' => $v.'Category', // table
							];
							$uriBuilderCat = $this->controllerContext->getUriBuilder();
							$urlCat = $uriBuilderCat->setArguments($getArgsCat)->uriFor('step2', [], $this->request->getControllerName());
							
							// Access (Cat)
							$accessOverlay = 'default';
							if(BaseUtility::BeUserAccessTableSelect(BaseUtility::getTableNameFromClass($v.'Category')) == false){
								$urlCat = '#';
								$accessOverlay = 'danger';
							}
							
							$linkCat = '<a href="'.$urlCat.'" class="label label-'.$accessOverlay.'">Категории ('.$countCat.')</a>';
							// $linkCat .= '&nbsp;'.$this->iconFactory->getIcon('actions-view-go-forward', Icon::SIZE_SMALL);
						} else {
							$linkCat = '';
						}
						// Access
						$accessOverlay = '';
						if(BaseUtility::BeUserAccessTableSelect(BaseUtility::getTableNameFromClass($v)) == false){
							$url = '#';
							$accessOverlay = 'overlay-locked';
						}
						$varContent .= '
						<tr>
							<td width="98%">
								<a href="'.$url.'">'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_tablerecord', Icon::SIZE_SMALL, $accessOverlay).' '.
									BaseUtility::getClassAnnotationValueNew($v,'AirTable\Parameter\Label').'
								</a>
								'.$linkCat.'
								<!--<br /><i style="color:gray">'.BaseUtility::getClassAnnotationValueNew($v,'AirTable\Parameter\Description').'</i>-->
							</td>
							<td width="*" nowrap>
								<code>\\'.$v.'</code><br />
								<!--<code>'.BaseUtility::getTableNameFromClass($v).'</code>-->
							</td>
							<td width="*" nowrap>
								Записей: '.intval($dbCount).' шт.
							</td>
						</tr>';
						$countJ++;
					}
				}
			}
		}
		
		// Если нет записей
		if($countJ == 0 && $paramExt != 'Root'){
			$varContent .= '
			<tr>
				<td>
					'.$this->iconFactory->getIcon('install-check-extables', Icon::SIZE_SMALL).'
					Таблицы не найдены
				</td>
			</tr>';
		}
		
        $assignedValues = [
            'content' => $varContent
        ];
        $this->view->assignMultiple($assignedValues);
    }

    /**
     * Main action for administration
     */
    public function step2Action()
    {
		$paramExt = BaseUtility::_GETuc('paramExt','Root');
		$paramSubDomain = BaseUtility::_GETuc('paramSubDomain','Root');
		$paramClass = BaseUtility::_GETuc('paramClass','');
		$table = BaseUtility::getTableNameFromClass($paramClass);
		
		// Параметры  _POST
		$_POST = \TYPO3\CMS\Core\Utility\GeneralUtility::_POST();

		// Проверяем загружен ли файл
		$contentError = '';
		if($this->request->getControllerName() == 'ImportXls'){
			$format = '.xls';
			if(is_uploaded_file($GLOBALS['_FILES']["filename"]["tmp_name"])){
				if(end(explode('.',$GLOBALS['_FILES']['filename']['name']))!='xlsx'){
					$contentError = 'К загрузке допускаются файлы только с расширением ".xlsx".';
					
				} else {
					// Если файл загружен успешно, перемещаем его
					// из временной директории в конечную
					$fileImportName = $table.'-'.date("d.m.Y-H.i.s").'.xlsx';
					$pathSite = \TYPO3\CMS\Core\Core\Environment::getPublicPath();
					$pathFolder = '/fileadmin/_xlsx-import_/';
					\TYPO3\CMS\Core\Utility\GeneralUtility::mkdir($pathSite.$pathFolder);
					
					if(move_uploaded_file($GLOBALS['_FILES']["filename"]["tmp_name"], $pathSite.$pathFolder.$fileImportName)){
						$getArgs = [
							'model' => $_POST['model'],
							'table' => $_POST['table'],
							'file' => $fileImportName
						];
						$uriBuilder = $this->controllerContext->getUriBuilder();
						$redirectUrl = $uriBuilder->setArguments($getArgs)->uriFor('step3', [], $this->request->getControllerName());
						\TYPO3\CMS\Core\Utility\HttpUtility::redirect($redirectUrl);
					 }else {
						 $contentError = 'Не удалось загрузить файл.';
					 }
				}
			} elseif(!empty($GLOBALS['_POST'])){
				 $contentError = 'Не выбран файл для загрузки на сервер.';
			}
		
		} elseif($this->request->getControllerName() == 'ImportCsv'){
			$format = '.csv';
			if(!empty($_POST)){
				if(empty($_POST['csv_content'])){
					$contentError = 'Поле с данными для импорта не заполнено.';
					
				}elseif(!empty($_POST['csv_content'])){
					$fileImportName = $table.'-'.date("d.m.Y-H.i.s").'.csv';
					$pathSite = \TYPO3\CMS\Core\Core\Environment::getPublicPath();
					$pathFolder = '/fileadmin/_csv-import_/';
					\TYPO3\CMS\Core\Utility\GeneralUtility::mkdir($pathSite.$pathFolder);
					BaseUtility::fileWrite($pathSite.$pathFolder.$fileImportName,$_POST['csv_content']);
					$getArgs = [
						'model' => $_POST['model'],
						'table' => $_POST['table'],
						'file' => $fileImportName
					];
					$uriBuilder = $this->controllerContext->getUriBuilder();
					$redirectUrl = $uriBuilder->setArguments($getArgs)->uriFor('step3', [], $this->request->getControllerName());
					\TYPO3\CMS\Core\Utility\HttpUtility::redirect($redirectUrl);
				}
			}
		}
		
        $assignedValues = [
			'model' => $paramClass,
			'table' => $table,
            'contentError' => $contentError,
			'format' => $format
        ];
        $this->view->assignMultiple($assignedValues);
    }
	
    /**
     * Main action for administration
     */
    public function step3Action()
    {	
		#$paramExt = BaseUtility::_GETuc('paramExt','Root');
		#$paramSubDomain = BaseUtility::_GETuc('paramSubDomain','Root');
		#$paramClass = BaseUtility::_GETuc('paramClass','');
		
		// Параметры _GET, _POST
		$_GET = \TYPO3\CMS\Core\Utility\GeneralUtility::_GET();
		$_POST = \TYPO3\CMS\Core\Utility\GeneralUtility::_POST();
		
		$fileImportName = $_GET['file'];
		$model = $_GET['model'];
		$table = $_GET['table'];
		
		// Список дублирующих ошибок
		$contentError = [];
		
		// Работу с таблицами которых нет в TCA пишем как ошибка
		if(!isset($GLOBALS['TCA'][$table])){
			$contentError[] = "Конфигурация таблицы не определена";
		
		// Не надена таблица
		#} elseif(!\Illuminate\Support\Facades\Schema::hasTable($table)){
		#	$contentError[] = "Таблица не существует в БД";
		
		} else {
			
			$pathSite = \TYPO3\CMS\Core\Core\Environment::getPublicPath();
			if($this->request->getControllerName() == 'ImportXls'){
				$pathFolder = '/fileadmin/_xlsx-import_/';
				
			} elseif($this->request->getControllerName() == 'ImportCsv'){
				$pathFolder = '/fileadmin/_csv-import_/';
			}
			
			// Не наден файл
			if(@!file_exists($pathSite.$pathFolder.$fileImportName)){
				$contentError[] = "На сервере не найден загруженный файл 
				<code>".$pathFolder.$fileImportName."</code> с данными для импорта";
			
			} else {
				
				// Получаем данные импорта
				$collection = (new \Rap2hpoutre\FastExcel\FastExcel)->import($pathSite.$pathFolder.$fileImportName);
				
				// Отстутсвуют данные для импорта!
				#if($collection)==0){
				#	$contentError[] = 'В файле отсутствуют данные для импорта';
				
				#} else {
					
					// Не найден столбец "uid"
					if(!isset($collection[0]['uid'])){
						$contentError[] = 'В файле не найден столбец "uid", необходимый для импорта данных';
					
					// Не найден столбец "pid"
					// Pid Пропускаем, если записи всегда в корне
					} elseif(!isset($collection[0]['pid']) && $GLOBALS['TCA'][$table]['ctrl']['rootLevel'] != 1){
						$contentError[] = 'В файле не найден столбец "pid", необходимый для импорта данных';
					
					// Разрешено сопоставление полей
					} else {
						
						// Колонки
						// tab 3 (columns)
						$columns = $GLOBALS['TCA'][$table]['columns'];
						foreach($columns as $k => $v){
							// Pid Пропускаем, если записи всегда в корне
							if($GLOBALS['TCA'][$table]['ctrl']['rootLevel'] == 1 && $k == 'pid'){
								continue;
							}
							
							$status = '';
							$checked = '';
							$disabled = '';
							$AirTableFieldClass = $v['AirTable.Class'];
							if(method_exists($AirTableFieldClass, 'importControllerRow') && $AirTableFieldClass::IMPORT == false){
								$status = 'warning';
								$checked = '';
								$disabled = 'disabled';
							} else {
								if(isset($collection[0][$k])){
									$status = 'success';
									if($k == 'uid' || $k == 'pid'){
										$checked = 'checked';
										$disabled = 'disabled';
									} else {
										$checked = '';
										$disabled = '';
									}
								} else {
									$disabled = 'disabled';
									$status = 'warning';
								}
							}
							$noConfig = '';
							if(!$v['AirTable.Class']){
								$disabled = 'disabled';
								$noConfig = 'color: red';
							}
							$debugStr = '';
							if($GLOBALS['TYPO3_CONF_VARS']['BE']['debug'] == true){
								$debugStr = '<br /><code style="color: #b4b2b2;">'.$v['AirTable.Class'].'</code>';
							}
							$content .= "
							<tr class='".$status."'>
								<td width='40%' align='right'>
									<b style='".$noConfig."'>
										".htmlspecialchars($GLOBALS['LANG']->sL($v['label']))."
										".$debugStr."
									</b>
								</td>
								<td width='20%'>
									<label class='btn btn-default btn-sm' style='margin: 0; margin-bottom: 0px; padding: 0 7px 0 7px; width: 100%; text-align: left; cursor: pointer;'>
										<input type='checkbox' name='columns[]' class='btn btn-default btn-sm' style='margin-top: 0px;' value='".$k."' ".$disabled." ".$checked."> 
										Включить в обработку
									</label>
								</label>
								</td>
								<td width='40%'>
									<b style='".$noConfig."'>".$k."</b>
								</td>
							</tr>
							 ";
						}
						
						// Колонки которые не существуют в БД
						foreach($collection[0] as $kFile => $vFile){
							if(!isset($GLOBALS['TCA'][$table]['columns'][$kFile])){
								$content .= "
								<tr class='danger'>
									<td width='40%' align='right'>
										<b>Поле отсутствует в базе</b>
									</td>
									<td width='20%'>
										<label class='btn btn-default btn-sm' style='margin: 0; margin-bottom: 0px; padding: 0 7px 0 7px; width: 100%; text-align: left; cursor: pointer;'>
											<input type='checkbox' name='columns[]' class='btn btn-default btn-sm' style='margin-top: 0px;' disabled> 
											Не поддерживается
										</label>
									</label>
									</td>
									<td width='40%'>
										<b style='".$noConfig."'>".$kFile."</b>
									</td>
								</tr>
								 ";
							}
						}
						
					}
					
				#}
				
				# print "<Pre>";
				# print_r($collection);
				# $collection[0];
			}
			
		}
		
        $assignedValues = [
            'model' => $model,
            'table' => $table,
            'file' => $fileImportName,
            'contentError' => implode('<br />',$contentError),
            'countError' => count($contentError),
			'content' => $content,
			'countRows' => intval(count($collection))
        ];
        $this->view->assignMultiple($assignedValues);
	}
	
    /**
     * Main action for administration
     */
    public function step3ProcessAction()
    {
		// Параметры
		#$paramExt = BaseUtility::_GETuc('paramExt',$this->extNameDir);
		#$paramSubDomain = BaseUtility::_GETuc('paramSubDomain','Root');
		#$paramClass = BaseUtility::_GETuc('paramClass','');
		
		// Параметры _GET,_POST
		$_GET = \TYPO3\CMS\Core\Utility\GeneralUtility::_GET();
		$_POST = \TYPO3\CMS\Core\Utility\GeneralUtility::_POST();
		$model = $_GET['model'];
		$table = $_GET['table'];
		$fileImportName = $_GET['file'];
		
		// Сбрасываем поле importprocess
		// Поле понадобилось пока что бы происходило обновление записи производилось каждый раз
		DB::table($table)->update(['importprocess'=>0]);
		
		// Получаем данные импорта
		$pathSite = \TYPO3\CMS\Core\Core\Environment::getPublicPath();
		if($this->request->getControllerName() == 'ImportXls'){
			$pathFolder = '/fileadmin/_xlsx-import_/';
			
		} elseif($this->request->getControllerName() == 'ImportCsv'){
			$pathFolder = '/fileadmin/_csv-import_/';
		}
		$collection = (new \Rap2hpoutre\FastExcel\FastExcel)->import($pathSite.$pathFolder.$fileImportName);

		// Кол-во выполненных операций
		$processLineResult = [];
		$countProcessLine = 0;
		$countEmpty = 0;
		$countInsert = 0;
		$countUpdate = 0;
		$countNoUpdate = 0; // не известный Uid
		$countErrorTransaction = 0;
		
		// Читаем строки
		$columns = $GLOBALS['TCA'][$table]['columns'];
		foreach($collection as $k => $line){
			
			// Нет данных для обновления
			if(count($line) == 0){
				$processLineResult[$k] = 4;
				$countEmpty++;
				$countProcessLine++;
				
			} else {
				DB::beginTransaction();
				try 
				{
					// Удаление записи
					if(preg_match('/^D(.*)/is',$line['uid'],$match)){
						$result = $model::where('uid','=',$match[1])->delete();
						if($result > 0){
							$processLineResult[$k] = 6;
							$countDelete++;
							$countProcessLine++;
						} else {
							$processLineResult[$k] = 3;
							$countNoDelete++;
							$countProcessLine++;
						}
					
					// Создание записи
					} elseif($line['uid'] < 1){
						// Сначала создаем (из-за связей, иначе не сделаем связь, например Mto1)
						// т.к. ID-пока еще не знаем! После обновляем
						$id = $model::insertGetId(['importprocess' => 1]);
						$rowData = $this->rowData($model, $table, $line, $id);
						$result = $model::where('uid','=',$id)->update($rowData);
						
						$processLineResult[$k] = 1;
						$countInsert++;
						$countProcessLine++;
					
					// Обновление записи
					} elseif($line['uid'] > 0){
						$rowData = $this->rowData($model, $table, $line, $line['uid']);
						$result = $model::where('uid','=',$line['uid'])->update($rowData);
						if($result > 0){
							$processLineResult[$k] = 2;
							$countUpdate++;
							$countProcessLine++;
						} else {
							$processLineResult[$k] = 3;
							$countNoUpdate++;
							$countProcessLine++;
						}
					}
					DB::commit();
				} catch (\Exception $e) {
					DB::rollback();
					$processLineResult[$k] = 5;
					$countErrorTransaction++;
					$countProcessLine++;
				}
			}
		}
		
		$getArgs = [
			'model' => $model,
			'table' => $table,
			'file' => $fileImportName,
			'countEmpty' => $countEmpty,
			'countInsert' => $countInsert, // Insert
			'countUpdate' => $countUpdate, // Update
			'countNoUpdate' => $countNoUpdate, // Update (no)
			'countDelete' => $countDelete, // Delete
			'countNoDelete' => $countNoDelete, // Delete (no)
			'countErrorTransaction' => $countErrorTransaction,
			'countProcessLine' => $countProcessLine,
			'processLineResult' => $processLineResult
		];
		$uriBuilder = $this->controllerContext->getUriBuilder();
		$redirectUrl = $uriBuilder->setArguments($getArgs)->uriFor('step4', [], $this->request->getControllerName());
		\TYPO3\CMS\Core\Utility\HttpUtility::redirect($redirectUrl);
	}
	
	// Собираем выдает обработанные данные
	public function rowData($model, $table, $line, $recordId){
		$_POST = \TYPO3\CMS\Core\Utility\GeneralUtility::_POST();
		$line['uid'] = $recordId; // Устанавливаем Id
		$rowData = [];
		foreach($line as $kFileField => $vFileField){
			if((isset($GLOBALS['TCA'][$table]['columns'][$kFileField]) && $kFileField != 'uid') || $kFileField == 'pid' || $kFileField == 'RType'){
				if(in_array($kFileField,$_POST['columns'])){ // При условии, что мы выбрали колонку
					$AirTableFieldClass = $GLOBALS['TCA'][$table]['columns'][$kFileField]['AirTable.Class'];
					if(method_exists($AirTableFieldClass, 'importControllerRow') && $AirTableFieldClass::IMPORT == true){
						$AirTableFieldClass::importControllerRow($rowData, $model, $table, $kFileField, $GLOBALS['TCA'][$table]['columns'][$kFileField], $line);
					}
				}
			}
		}
		$rowData['importprocess'] = 1;
		unset($rowData['uid']); // Удаляем, Id
		
		return $rowData;
	}
	
    /**
     * Main action for administration
     */
    public function step4Action()
    {
		// Параметры
		#$paramExt = BaseUtility::_GETuc('paramExt',$this->extNameDir);
		#$paramSubDomain = BaseUtility::_GETuc('paramSubDomain','Root');
		#$paramClass = BaseUtility::_GETuc('paramClass','');
		
		$model = $_GET['model'];
		$table = $_GET['table'];
		$fileImportName = $_GET['file'];
		
		$countEmpty = $_GET['countEmpty'];
		$countInsert = $_GET['countInsert'];
		$countUpdate = $_GET['countUpdate'];
		$countNoUpdate = $_GET['countNoUpdate'];
		$countDelete = $_GET['countDelete'];
		$countNoDelete = $_GET['countNoDelete'];
		$countErrorTransaction = $_GET['countErrorTransaction'];
		$countProcessLine = $_GET['countProcessLine'];
		$processLineResult = $_GET['processLineResult'];
		
		// Получаем данные импорта
		$pathSite = \TYPO3\CMS\Core\Core\Environment::getPublicPath();
		if($this->request->getControllerName() == 'ImportXls'){
			$pathFolder = '/fileadmin/_xlsx-import_/';
			
		} elseif($this->request->getControllerName() == 'ImportCsv'){
			$pathFolder = '/fileadmin/_csv-import_/';
		}
		$collection = (new \Rap2hpoutre\FastExcel\FastExcel)->import($pathSite.$pathFolder.$fileImportName);
		
		$content = '';
		foreach($collection as $k => $line){
			switch($processLineResult[$k]){
				case 5: $comment = 'Транзакция - откат изменений'; break;
				case 4: $comment = 'Нет данных для обновления'; break;
				case 1: $comment = 'Выполнено - создание записи'; break;
				case 2: $comment = 'Выполнено - обновление записи'; break;
				case 6: $comment = 'Выполнено - удаление записи'; break;
				case 3: $comment = 'Проигнорировано ("uid" не найден в базе данных)'; break;
			}
			
			$content .= "<tr>
				<td width='10' style='white-space: nowrap;'>№ строки ".($k+1)."</td>
				<td width='10' style='white-space: nowrap;'>[ID: ".$line['uid']."]</td>
				<td nowrap>".$comment."</td>
			</tr>";
		}
		
        $assignedValues = [
            'table' => $table,
            'strCount' => intval($countProcessLine),
			'strEmpty' => intval($countEmpty),
            'strInsert' => intval($countInsert),
			'strUpdate' => intval($countUpdate),
			'strNoUpdate' => intval($countNoUpdate),
			'strDelete' => intval($countDelete),
			'strNoDelete' => intval($countNoDelete),
			'strErrorTransaction' => intval($countErrorTransaction),
			'content' => $content
        ];
        $this->view->assignMultiple($assignedValues);
	}

}
