<?php
namespace Litovchenko\AirTable\Controller;

/**
 * This file is part of the "air_table" Extension for TYPO3 CMS.
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 */
use TYPO3\CMS\Backend\Template\Components\ButtonBar;
use TYPO3\CMS\Backend\Utility\BackendUtility as BackendUtilityCore;
use TYPO3\CMS\Backend\View\BackendTemplateView;
use TYPO3\CMS\Core\FormProtection\FormProtectionFactory;
use TYPO3\CMS\Core\Imaging\Icon;
use TYPO3\CMS\Core\Imaging\IconFactory;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\HttpUtility;
use TYPO3\CMS\Core\Utility\MathUtility;
use TYPO3\CMS\Core\Utility\VersionNumberUtility;
use TYPO3\CMS\Extbase\Mvc\View\ViewInterface;
use TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder;
use Illuminate\Database\Capsule\Manager as DB;
use Litovchenko\AirTable\Utility\BaseUtility;

define("LAST_COLUMN", 'width: auto; border-top: 0; padding: 0; margin: 0;');
define("HIDE_CHECKBOX", 'display: none;');
define("HIDE_CHECKBOX_TD_NEXT", 'border-left: 0;');
define("STYLE_EMPTY_FIELD", 'background: #d8d8d8; color: #fff;');

/**
 * @AirTable:<BackendModule>
 * @AirTable\Parameter\Name:<Value>
 */
class ListController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    /**
     * Page uid
     *
     * @var int
     */
    protected $extNameDir = ''; // $pageExtUid = 0;

    /**
     * TsConfig configuration
     *
     * @var array
     */
    protected $tsConfiguration = [];

    /**
     * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
     */
    protected $configurationManager;

    /**
     * Function will be called before every other action
     *
     */
    public function initializeAction()
    {
		$extnameFromRoute = \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('route');
		$extnameFromRoute = explode('/',$extnameFromRoute);
		$extnameFromRoute = str_replace('AirTableList','',$extnameFromRoute[2]);
		
		$path_Typo3Ext = \TYPO3\CMS\Core\Core\Environment::getExtensionsPath().'/';
		$path_Typo3CoreExt = \TYPO3\CMS\Core\Core\Environment::getFrameworkBasePath().'/';
		$extPathList = array_merge(
			glob($path_Typo3Ext.'*/'), // typo3conf/ext/
			['--'],
			glob($path_Typo3CoreExt.'*/') // typo3/sysext/
		);
		
		foreach($extPathList as $k => $extPath){
			if($k == $extnameFromRoute){
				$this->extNameDir = basename($extPath);
				break;
			}
		}

        parent::initializeAction();
    }

    /**
     * BackendTemplateContainer
     *
     * @var BackendTemplateView
     */
    protected $view;

    /**
     * @var IconFactory
     */
    public $iconFactory;

    /**
     * Backend Template Container
     *
     * @var BackendTemplateView
     */
    protected $defaultViewObjectName = BackendTemplateView::class;

    /**
     * Set up the doc header properly here
     *
     * @param ViewInterface $view
     */
    protected function initializeView(ViewInterface $view)
    {
        $this->iconFactory = GeneralUtility::makeInstance(IconFactory::class);
		
        /** @var BackendTemplateView $view */
        parent::initializeView($view);
        $view->getModuleTemplate()->getDocHeaderComponent()->setMetaInformation([]);

        $pageRenderer = $this->view->getModuleTemplate()->getPageRenderer();
		$pageRenderer->addJsFile('EXT:air_table/Resources/Public/Js/List.js');
		$pageRenderer->addCssFile('EXT:air_table/Resources/Public/Css/List.css');
		
		// Выводим кнопки только при условии, что это не выбор записей
		if(\TYPO3\CMS\Core\Utility\GeneralUtility::_GET('recordSelection') == 0){
			if($this->actionMethodName == 'listAction'){
				$this->createMenu();
			}
		}
		$this->createButtons();
    }

    /**
     * Create menu
     *
     */
    protected function createMenu()
    {
		$getArgs = [
			'paramExt' => BaseUtility::_GETuc('paramExt',$this->extNameDir),
			'paramSubDomain' => BaseUtility::_GETuc('paramSubDomain','Root'),
			'paramClass' => BaseUtility::_GETuc('paramClass',''),
			'recordSelection' => \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('recordSelection'),
			'recordSelectionFieldname' => \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('recordSelectionFieldname'),
		];
		$listOptions[1] = 'Кол-во новых записей: 1';
		$listOptions[3] = 'Кол-во новых записей: 3';
		$listOptions[5] = 'Кол-во новых записей: 5';
		$listOptions[10] = 'Кол-во новых записей: 10';
        $menu = $this->view->getModuleTemplate()->getDocHeaderComponent()->getMenuRegistry()->makeMenu();
        $menu->setIdentifier('CountNewRecords');
        foreach ($listOptions as $k => $title) {
			$getArgs['countNewRecords'] = $k;
            $uriBuilder = $this->controllerContext->getUriBuilder();
			$url = $uriBuilder->setArguments($getArgs)->uriFor('list', [], 'List');
			$item = $menu
                ->makeMenuItem()
                ->setHref($url)
                ->setTitle($title);
            if (\TYPO3\CMS\Core\Utility\GeneralUtility::_GET('countNewRecords') == $k) {
                $item->setActive(true);
            }
            $menu->addMenuItem($item);
        }
        $this->view->getModuleTemplate()->getDocHeaderComponent()->getMenuRegistry()->addMenu($menu);
    }

    /**
     * Create the panel of buttons
     *
     */
    protected function createButtons()
    {
		$paramExt = BaseUtility::_GETuc('paramExt',$this->extNameDir);
		$paramSubDomain = BaseUtility::_GETuc('paramSubDomain','Root');
		$paramClass = BaseUtility::_GETuc('paramClass','');
		$table = BaseUtility::getTableNameFromClass($paramClass);
	
		$buttonBar = $this->view->getModuleTemplate()->getDocHeaderComponent()->getButtonBar();
		$buttons = array();
		
		// (+) Кнопки "Создать запись". Находимся в категории
		if(BaseUtility::isModelCategoryForAnotherModel($paramClass) && BaseUtility::BeUserAccessTableModify($table) == true) {
			if($this->actionMethodName == 'listAction'){
				$buttons[] = array(
					'title' => 'Новая категория',
					'table' => $table,
					'icon' => $GLOBALS['TCA'][$table]['ctrl']['typeicon_classes']['default'],
					'columnsOnly' => 'uid,RType,title,parent_id,hidden',
					'defVals' => [
						$table => [
							// Fix Typo3 bug (про создании записей для "renderType->selectTree" не работает defVals)
							'parent_id' => current(explode(',',BaseUtility::_POSTuc('form1Apply','form1FieldValue_parent_id','')))
						]
					]
				);
				#$buttons[] = array(
				#	'title' => 'Создать новый элемент',
				#	'table' => preg_replace('/_category$/is','',$table),
				#	'icon' => $GLOBALS['TCA'][preg_replace('/_category$/is','',$table)]['ctrl']['typeicon_classes']['default'],
				#	'columnsOnly' => 'uid,RType,title,category_id,category_many,parent_id,hidden',
				#);
			}
			
		// (+) Кнопки "Создать запись". Находимся в элементе категории
		} elseif(BaseUtility::isModelSupportOneCategory($paramClass) || BaseUtility::isModelSupportManyCategories($paramClass)) {
			if($this->actionMethodName == 'listAction'){
				if(BaseUtility::isModelSupportOneCategory($paramClass) && BaseUtility::BeUserAccessTableModify($table.'_category') == true){
					$buttons[] = array(
						'title' => 'Новая категория',
						'table' => $table.'_category',
						'icon' => $GLOBALS['TCA'][$table.'_category']['ctrl']['typeicon_classes']['default'],
						'columnsOnly' => 'uid,RType,title,parent_id,hidden',
						'defVals' => [
							$table.'_category' => [
								// Fix Typo3 bug (про создании записей для "renderType->selectTree" не работает defVals)
								'parent_id' => current(explode(',',BaseUtility::_POSTuc('form1Apply','form1FieldValue_category_id','')))
							]
						]
					);
				}elseif(BaseUtility::isModelSupportManyCategories($paramClass) && BaseUtility::BeUserAccessTableModify($table.'_category') == true){
				$buttons[] = array(
						'title' => 'Новая категория',
						'table' => $table.'_category',
						'icon' => $GLOBALS['TCA'][$table.'_category']['ctrl']['typeicon_classes']['default'],
						'columnsOnly' => 'uid,RType,title,parent_id,hidden',
						'defVals' => [
							$table.'_category' => [
								// Fix Typo3 bug (про создании записей для "renderType->selectTree" не работает defVals)
								'parent_id' => current(explode(',',BaseUtility::_POSTuc('form1Apply','form1FieldValue_category_many','')))
							]
						]
					);
				}
				if(BaseUtility::BeUserAccessTableModify($table) == true){
					$buttons[] = array(
						'title' => 'Новый элемент',
						'table' => $table,
						'icon' => $GLOBALS['TCA'][$table]['ctrl']['typeicon_classes']['default'],
						'columnsOnly' => 'uid,RType,title,category_id,category_many,parent_id,hidden',
						'defVals' => [
							$table => [
								// Fix Typo3 bug (про создании записей для "renderType->selectTree" не работает defVals)
								'category_id' => current(explode(',',BaseUtility::_POSTuc('form1Apply','form1FieldValue_category_id',''))),
								'category_many' => BaseUtility::_POSTuc('form1Apply','form1FieldValue_category_many',''),
								'parent_id' => current(explode(',',BaseUtility::_POSTuc('form1Apply','form1FieldValue_parent_id',''))),
							]
						]
					);
				}
			}
			
		// (+) Кнопки "Создать запись". Обычная запись
		} else {
			if($this->actionMethodName == 'listAction' && BaseUtility::BeUserAccessTableModify($table) == true){
				$fieldLabel = $GLOBALS['TCA'][$table]['ctrl']['label'];
				$fieldDisabled = $GLOBALS['TCA'][$table]['ctrl']['enablecolumns']['disabled'];
				$buttons[] = array(
					'title' => 'Новая запись',
					'table' => $table,
					'icon' => $GLOBALS['TCA'][$table]['ctrl']['typeicon_classes']['default'],
					'columnsOnly' => 'uid,RType,'.$fieldLabel.',parent_id,'.$fieldDisabled,
					'defVals' => [
						$table => [
							// Fix Typo3 bug (про создании записей для "renderType->selectTree" не работает defVals)
							'parent_id' => current(explode(',',BaseUtility::_POSTuc('form1Apply','form1FieldValue_parent_id','')))
						]
					]
				);
			}
		}

		// Выводим кнопки только при условии, что это не выбор записей
		if(\TYPO3\CMS\Core\Utility\GeneralUtility::_GET('recordSelection') == 0){
			foreach ($buttons as $key => $tableConfiguration) {
					$backendLinkConf = [
						'edit['.$tableConfiguration['table'].'][0]' => 'new',
						'columnsOnly' => $tableConfiguration['columnsOnly'],
						'defVals' => $tableConfiguration['defVals'],
						'returnUrl' => \TYPO3\CMS\Core\Utility\GeneralUtility::getIndpEnv('REQUEST_URI')
					];
					
					// Кол-во новых записей
					switch(\TYPO3\CMS\Core\Utility\GeneralUtility::_GET('countNewRecords')){
						default: // +1
						break;
						case 3: // +3
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0b]']='new';
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0c]']='new';
						break;
						case 5: // +5
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0b]']='new';
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0c]']='new';
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0d]']='new';
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0e]']='new';
						break;
						case 10: // +10
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0b]']='new';
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0c]']='new';
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0d]']='new';
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0e]']='new';
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0f]']='new';
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0j]']='new';
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0h]']='new';
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0i]']='new';
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0j]']='new';
							$backendLinkConf['edit['.$tableConfiguration['table'].'][0k]']='new';
						break;
					}
					
					$backendLink = \TYPO3\CMS\Backend\Utility\BackendUtility::getModuleUrl(
						'record_edit',
						$backendLinkConf
					);
					$viewButton = $buttonBar->makeLinkButton()
						->setShowLabelText(true)
						->setClasses('') // btn btn-default btn-sm
						->setHref($backendLink)
						->setDataAttributes([
							'toggle' => 'tooltip',
							'placement' => 'bottom',
							'title' => $tableConfiguration['title']
						])
						->setTitle($tableConfiguration['title'])
						->setIcon($this->iconFactory->getIcon($tableConfiguration['icon'], Icon::SIZE_SMALL, 'overlay-new'));
					$buttonBar->addButton($viewButton, ButtonBar::BUTTON_POSITION_LEFT, 1);
			}
		}

        // Tree
		#if(\TYPO3\CMS\Core\Utility\GeneralUtility::_GET('recordSelection') == 0){
		#	if(@in_array("parent_id", $GLOBALS['TCA'][$storage_table]['ctrl']['fieldsDefault'])
		#	|| @in_array("category_id", $GLOBALS['TCA'][$storage_table]['ctrl']['fieldsDefault']) 
		#	|| @in_array("category_many", $GLOBALS['TCA'][$storage_table]['ctrl']['fieldsDefault']))
		#	{
		#		if($GLOBALS['BE_USER']->uc['air_table']['ListControllerTreeSwitcher'][$pid] == 1){
		#			$treeCssClass = 'btn btn-success btn-sm';
		#		} else {
		#			$treeCssClass = 'btn btn-primary btn-sm';
		#		}
		#		$getArgs = [
		#			'pid' => \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('pid'),
		#			'tab_active' => \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('tab_active'),
		#			'tree' => 'show_or_hide',
		#		];
		#		$uriBuilder = $this->controllerContext->getUriBuilder();
		#		$url = $uriBuilder->setArguments($getArgs)->uriFor('list', [], 'List');
		#		$viewButton = $buttonBar->makeLinkButton()
		#			->setShowLabelText(true)
		#			->setClasses($treeCssClass) // btn btn-default btn-sm
		#			->setHref($url)
		#			->setDataAttributes([
		#				'toggle' => 'tooltip',
		#				'placement' => 'bottom',
		#				'title' => '-'
		#			])
		#			->setTitle('Дерево записей')
		#			->setIcon($this->iconFactory->getIcon('actions-pagetree', Icon::SIZE_SMALL));
		#		$buttonBar->addButton($viewButton, ButtonBar::BUTTON_POSITION_LEFT, 1);
		#	}
		#}
		
        // (+) Кнопка корзина
		// if(BaseUtility::hasClassTrait($paramClass,'Litovchenko\AirTable\Field\Traits\Deleted') == true) {
			if($this->actionMethodName == 'indexAction' || $this->actionMethodName == 'listAction'){
				if(BaseUtility::_GETuc('BackendDeleteScope',0,'AllController')){
					$trashCssClass = 'btn btn-success btn-sm';
					$getArgs = [
						'paramExt' => BaseUtility::_GETuc('paramExt',$this->extNameDir),
						'paramSubDomain' => BaseUtility::_GETuc('paramSubDomain','Root'),
						'paramClass' => BaseUtility::_GETuc('paramClass',''),
						'recordSelection' => \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('recordSelection'),
						'recordSelectionFieldname' => \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('recordSelectionFieldname'),
						'BackendDeleteScope' => 0
					];
				} else {
					$trashCssClass = 'btn btn-default btn-sm';
					$getArgs = [
						'paramExt' => BaseUtility::_GETuc('paramExt',$this->extNameDir),
						'paramSubDomain' => BaseUtility::_GETuc('paramSubDomain','Root'),
						'paramClass' => BaseUtility::_GETuc('paramClass',''),
						'recordSelection' => \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('recordSelection'),
						'recordSelectionFieldname' => \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('recordSelectionFieldname'),
						'BackendDeleteScope' => 1
					];
				}
				$uriBuilder = $this->controllerContext->getUriBuilder();
				$url = $uriBuilder->setArguments($getArgs)->uriFor(str_replace('Action','',$this->actionMethodName), [], 'List');
				$viewButton = $buttonBar->makeLinkButton()
					->setShowLabelText(true)
					->setClasses($trashCssClass)
					->setHref($url)
					->setDataAttributes([
						'toggle' => 'tooltip',
						'placement' => 'bottom',
						'title' => '-'
					])
					->setTitle('Корзина записей')
					->setIcon($this->iconFactory->getIcon('actions-edit-delete', Icon::SIZE_SMALL));
				$buttonBar->addButton($viewButton, ButtonBar::BUTTON_POSITION_LEFT, 1);
			}
		// }
		
        
		if($this->actionMethodName == 'listAction'){
			// (+) Экспорт
			if(BaseUtility::BeUserAccessModule('AirTableExportxls')){
				$path = VersionNumberUtility::convertVersionNumberToInteger(TYPO3_branch) >= VersionNumberUtility::convertVersionNumberToInteger('8.6') ? 'Resources/Private/Language/' : '';
				$getArgs = [
					'paramExt' => $paramExt,
					'paramSubDomain' => $paramSubDomain,
					'paramClass' => $paramClass,
					'tx_airtable_airtabletxairtabletabservice_airtableexportxls' => [
						'controller' => 'ExportXls',
						'action' => 'step2'
					]
				];
				$routingUriBuilder = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\CMS\Backend\Routing\UriBuilder');
				$backendLinkExport = $routingUriBuilder->buildUriFromRoute('AirTableTxairtabletabservice_AirTableExportxls', $getArgs);
				$exportButton = $buttonBar->makeLinkButton()
					->setHref($backendLinkExport)
					->setShowLabelText(true)
					->setTitle('Экспорт записей .XLS')
					->setClasses('')
					->setOnclick('window.open(this.href,\'_blank\'); return false;')
					->setIcon($this->iconFactory->getIcon('actions-database-export', Icon::SIZE_SMALL));
				$buttonBar->addButton($exportButton, ButtonBar::BUTTON_POSITION_LEFT, 2);
			}
			
			// (+) Импорт
			if(BaseUtility::BeUserAccessModule('AirTableImportxls')){
				$path = VersionNumberUtility::convertVersionNumberToInteger(TYPO3_branch) >= VersionNumberUtility::convertVersionNumberToInteger('8.6') ? 'Resources/Private/Language/' : '';
				$getArgs = [
					'paramExt' => $paramExt,
					'paramSubDomain' => $paramSubDomain,
					'paramClass' => $paramClass,
					'tx_airtable_airtabletxairtabletabservice_airtableimportxls' => [
						'controller' => 'ImportXls',
						'action' => 'step2'
					]
				];
				$routingUriBuilder = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\CMS\Backend\Routing\UriBuilder');
				$backendLinkImport = $routingUriBuilder->buildUriFromRoute('AirTableTxairtabletabservice_AirTableImportxls', $getArgs);
				$importButton = $buttonBar->makeLinkButton()
					->setHref($backendLinkImport)
					->setShowLabelText(true)
					->setTitle('Импорт записей .XLS')
					->setClasses('')
					->setOnclick('window.open(this.href,\'_blank\'); return false;')
					->setIcon($this->iconFactory->getIcon('actions-database-import', Icon::SIZE_SMALL));
				$buttonBar->addButton($importButton, ButtonBar::BUTTON_POSITION_LEFT, 2);
			}
	
			// (+) Создать заметку для таблицы
			$path = VersionNumberUtility::convertVersionNumberToInteger(TYPO3_branch) >= VersionNumberUtility::convertVersionNumberToInteger('8.6') ? 'Resources/Private/Language/' : '';
			$backendLink = \TYPO3\CMS\Backend\Utility\BackendUtility::getModuleUrl(
				'record_edit',
				array(
					'edit[sys_note][0]' => 'new',
					'defVals' => [
						'sys_note' => [
							'ex_air_table_modelname' => $paramClass
						]
					],
					'returnUrl' => \TYPO3\CMS\Core\Utility\GeneralUtility::getIndpEnv('REQUEST_URI')
				)
			);
			$refreshButton = $buttonBar->makeLinkButton()
				->setHref($backendLink)
				->setShowLabelText(true)
				->setTitle('Заметка к таблице')
				->setIcon($this->iconFactory->getIcon('actions-localize', Icon::SIZE_SMALL));
			$buttonBar->addButton($refreshButton, ButtonBar::BUTTON_POSITION_LEFT, 2);
	
			// (+) Кнопка перехода между таблицами
			#$path = VersionNumberUtility::convertVersionNumberToInteger(TYPO3_branch) >= VersionNumberUtility::convertVersionNumberToInteger('8.6') ? 'Resources/Private/Language/' : '';
			#$refreshButton = $buttonBar->makeLinkButton()
			#	->setHref(GeneralUtility::getIndpEnv('REQUEST_URI'))
			#	->setTitle('Таблицы модели')
			#	->setShowLabelText(true)
			#	->setIcon($this->iconFactory->getIcon('actions-database', Icon::SIZE_SMALL));
			#$buttonBar->addButton($refreshButton, ButtonBar::BUTTON_POSITION_LEFT, 3);
		}
		
		// (+) Обратно, распечатать, в новом окне (из просмотра)	
		if($this->actionMethodName == 'showAction'){
			$path = VersionNumberUtility::convertVersionNumberToInteger(TYPO3_branch) >= VersionNumberUtility::convertVersionNumberToInteger('8.6') ? 'Resources/Private/Language/' : '';
			$getArgs = [
				'paramExt' => BaseUtility::_GETuc('paramExt',$this->extNameDir),
				'paramSubDomain' => BaseUtility::_GETuc('paramSubDomain','Root'),
				'paramClass' => BaseUtility::_GETuc('paramClass',''),
			];
			$uriBuilder = $this->controllerContext->getUriBuilder();
			$url = $uriBuilder->setArguments($getArgs)->uriFor('list', [], 'List');
			$backButton = $buttonBar->makeLinkButton()
				->setHref($url)
				->setTitle('Обратно')
				->setShowLabelText(true)
				->setIcon($this->iconFactory->getIcon('actions-view-go-back', Icon::SIZE_SMALL));
			$buttonBar->addButton($backButton, ButtonBar::BUTTON_POSITION_LEFT,1);
			$printButton = $buttonBar->makeLinkButton()
				->setHref(GeneralUtility::getIndpEnv('REQUEST_URI'))
				->setTitle('Открыть в новом окне')
				->setShowLabelText(true)
				->setOnclick('window.open(this.href,\'_blank\'); return false;')
				->setIcon($this->iconFactory->getIcon('actions-window-open', Icon::SIZE_SMALL));
			$buttonBar->addButton($printButton, ButtonBar::BUTTON_POSITION_LEFT,1);
		}
		
        // (+) Кнопка обновить
        $path = VersionNumberUtility::convertVersionNumberToInteger(TYPO3_branch) >= VersionNumberUtility::convertVersionNumberToInteger('8.6') ? 'Resources/Private/Language/' : '';
        $refreshButton = $buttonBar->makeLinkButton()
            ->setHref(GeneralUtility::getIndpEnv('REQUEST_URI'))
            ->setTitle('Обновить')
            ->setIcon($this->iconFactory->getIcon('actions-refresh', Icon::SIZE_SMALL));
        $buttonBar->addButton($refreshButton, ButtonBar::BUTTON_POSITION_RIGHT);
    }


    /**
     * Main action for administration
     */
    public function indexAction()
    {
		$paramExt = BaseUtility::_GETuc('paramExt',$this->extNameDir);
		$paramSubDomain = BaseUtility::_GETuc('paramSubDomain','Root');
		
		$extRelPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath($paramExt);
		#if(file_exists($extRelPath.'Classes/Domain/_.txt')){
		#	$titleExt = 'EXT:'.$paramExt . ' <br /> <i style="color: gray"> ' . file_get_contents($extRelPath.'Classes/Domain/_.txt') . ' </i> ';
		#} else {
			$titleExt = 'EXT:'.$paramExt;
		#}
		
		// 1
		$varTabLabel = $this->iconFactory->getIcon('extensions-air_table-icon_storage_extension', Icon::SIZE_SMALL).' ';
		$varTabLabel .= $titleExt;
		
		// 2
		$classList = [];
		$models = BaseUtility::getLoaderClasses('Classes/Domain/');
		foreach($models as $class) {
			$class_parents = class_parents($class);
			// Регистрация новых пользовательских моделей
			if(in_array('Litovchenko\AirTable\Model\BaseCrud',$class_parents) && BaseUtility::getClassAnnotationValueNew($class,'AirTable') == 'Table'){
				$extName = BaseUtility::getExtNameFromClassPath($class);
				$subDomain = BaseUtility::getSubDomainNameFromClassPath($class);
				$classList[$extName][$subDomain][] = $class;
			}
			// Регистрация моделей на основе существующей таблицы TYPO3
			if(in_array('Litovchenko\AirTable\Model\BaseCrud2',$class_parents) && BaseUtility::getClassAnnotationValueNew($class,'AirTable') == 'Table'){
				$extName = BaseUtility::getExtNameFromClassPath($class);
				$subDomain = BaseUtility::getSubDomainNameFromClassPath($class);
				$classList[$extName][$subDomain][] = $class;
			}
			// Регистрация моделей на основе существующей таблицы TYPO3
			#if(in_array('TYPO3\CMS\Extbase\DomainObject\AbstractEntity',$class_parents)){
			#	$r = new \ReflectionClass($class);
			#	if(!$r->isAbstract()){
			#		$extName = BaseUtility::getExtNameFromClassPath($class);
			#		$subDomain = BaseUtility::getSubDomainNameFromClassPath($class);
			#		$classList[$extName][$subDomain][] = $class;
			#	}
			#}
		}
		
		// Сортировка
		foreach($classList as $extName => $subDomain){
			ksort($classList[$extName]);
			// sort($classList[$extName][$subDomain]);
		}
		
		$countJ = 0;
		$varContent = '';
		foreach($classList[$paramExt] as $subDomain => $classes){
			if($subDomain != $paramSubDomain && $paramSubDomain == 'Root'){
				$extRelPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath($paramExt);
				if(file_exists($extRelPath.'Classes/Domain/Model/'.$subDomain.'/_.txt')){
					$titleSubDomain = file_get_contents($extRelPath.'Classes/Domain/Model/'.$subDomain.'/_.txt');
				} else {
					$titleSubDomain = $DomainModelKey;
				}
				$getArgs = [
					'paramExt' => $paramExt,
					'paramSubDomain' => $subDomain
				];
				$uriBuilder = $this->controllerContext->getUriBuilder();
				$url = $uriBuilder->setArguments($getArgs)->uriFor('index', [], 'List');
				$varContent .= '
				<tr>
					<td colspan="3"><a href="'.$url.'">'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_subdomain', Icon::SIZE_SMALL).' '.
						$titleSubDomain.'</a>
					</td>
				</tr>';
				$countJ++;
			} elseif($paramSubDomain != 'Root') {
				$getArgs = [
					'paramExt' => $paramExt,
					'paramSubDomain' => 'Root'
				];
				$uriBuilder = $this->controllerContext->getUriBuilder();
				$url = $uriBuilder->setArguments($getArgs)->uriFor('index', [], 'List');
				$varContent .= '
				<tr>
					<td colspan="3"><a href="'.$url.'">'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_subdomain', Icon::SIZE_SMALL).' '.
						'..</a>
					</td>
				</tr>';
				$countJ++;
				break;
			}
		}
		foreach($classList[$paramExt] as $subDomain => $classes){
			if($subDomain == $paramSubDomain){
				foreach($classes as $k => $v){
					if(BaseUtility::isModelCategoryForAnotherModel($v)){
						continue;
					}
					$dbCount = $v::count();
					$getArgs = [
						'paramExt' => $paramExt,
						'paramSubDomain' => $paramSubDomain,
						'paramClass' => $v, // table
					];
					$uriBuilder = $this->controllerContext->getUriBuilder();
					$url = $uriBuilder->setArguments($getArgs)->uriFor('list', [], 'List');
					if(BaseUtility::isModelSupportOneCategory($v) || BaseUtility::isModelSupportManyCategories($v)){
						$iconCat = $this->iconFactory->getIcon('actions-view-go-forward', Icon::SIZE_SMALL).' ';
						$modelCat = $v.'Category';
						$countCat = $modelCat::count();
						$getArgsCat = [
							'paramExt' => $paramExt,
							'paramSubDomain' => $paramSubDomain,
							'paramClass' => $v.'Category', // table
						];
						$uriBuilderCat = $this->controllerContext->getUriBuilder();
						$urlCat = $uriBuilderCat->setArguments($getArgsCat)->uriFor('list', [], 'List');
						
						// Access (Cat)
						$accessOverlay = 'default';
						if(BaseUtility::BeUserAccessTableSelect(BaseUtility::getTableNameFromClass($v.'Category')) == false){
							$urlCat = '#';
							$accessOverlay = 'danger';
						}
						
						$linkCat = '<a href="'.$urlCat.'" class="label label-'.$accessOverlay.'">Категории ('.$countCat.')</a>';
						// $linkCat .= '&nbsp;'.$this->iconFactory->getIcon('actions-view-go-forward', Icon::SIZE_SMALL);
					} else {
						$linkCat = '';
					}
					$livesearch = '';
					foreach($GLOBALS['TYPO3_CONF_VARS']['SYS']['livesearch'] as $kL => $vL){
						if($vL == BaseUtility::getTableNameFromClass($v)){
							$livesearch = '<i style="color: gray">#'.$kL.'</i>';
							break;
						}
					}
					// Access
					$accessOverlay = '';
					if(BaseUtility::BeUserAccessTableSelect(BaseUtility::getTableNameFromClass($v)) == false){
						$url = '#';
						$accessOverlay = 'overlay-locked';
					}
					$varContent .= '
					<tr>
						<td width="98%">
							<a href="'.$url.'">'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_tablerecord', Icon::SIZE_SMALL, $accessOverlay).' '.
								BaseUtility::getClassAnnotationValueNew($v,'AirTable\Parameter\Label').'
							</a>
							'.$linkCat.'
							<!--<br /><i style="color:gray">'.BaseUtility::getClassAnnotationValueNew($v,'AirTable\Parameter\Description').'</i>-->
						</td>
						<td width="*" nowrap>
							<code>\\'.$v.''.$livesearch.'</code>
							<!--<br />-->
							<!--<code>'.BaseUtility::getTableNameFromClass($v).'</code>-->
						</td>
						<td width="*" nowrap>
							Записей: '.intval($dbCount).' шт.
						</td>
					</tr>';
					$countJ++;
				}
			}
		}
		
		// Если нет записей
		if($countJ == 0){
			$varContent .= '
			<tr>
				<td>
					'.$this->iconFactory->getIcon('install-check-extables', Icon::SIZE_SMALL).'
					Таблицы не найдены
				</td>
			</tr>';
		}
		
        $assignedValues = [
			'tabLabel' => $varTabLabel,
            'content' => $varContent,
            'umlCode' => $this->umlCode($classList[$paramExt], $paramSubDomain)
        ];
        $this->view->assignMultiple($assignedValues);
    }
	
    /**
     * Generate Uml code (http://www.plantuml.com/plantuml/uml/)
     */
    public function umlCode($classList, $paramSubDomain)
    {
		$umlCodeContent = '';
		$umlCodeContentFields = [];
		$umlCodeContentRelation = '';
		foreach($classList as $subDomain => $classes){
			if($subDomain == $paramSubDomain){
				foreach($classes as $k => $v){
					$umlCodeContentFields[$v][] = ''; // Что бы создавались пустые таблицы
					$table = BaseUtility::getTableNameFromClass($v);
					$columns = $GLOBALS['TCA'][$table]['columns'];
					foreach($columns as $kColumn => $vColumn){
						$annotationRelationType = BaseUtility::getClassFieldAnnotationValueNew($v,$kColumn,'AirTable\Field');
						$annotationForeignModel = BaseUtility::getClassFieldAnnotationValueNew($v,$kColumn,'AirTable\Field\Parameter\ForeignModel');
						$annotationForeignKey = BaseUtility::getClassFieldAnnotationValueNew($v,$kColumn,'AirTable\Field\Parameter\ForeignKey');
						if(strstr($vColumn['AirTable.Class'],'Rel_') && !strstr($vColumn['AirTable.Class'],'_Revers')){
							// $this->umlCode .= "+ ".$kColumn." &#8601; \n"; // <!--int(10)--> -- // -- ".htmlspecialchars($GLOBALS['LANG']->sL($vColumn['label']))
							// $this->umlCode .= "+ ".$kColumn." &#8599; \n"; // <!--int(10)--> -- // -- ".htmlspecialchars($GLOBALS['LANG']->sL($vColumn['label']))
							#+ В рамках модели
							## Внешние
							#~ На себя
							#";
							$relName = end(explode('\\',$vColumn['AirTable.Class']));
							$relName = str_replace('Rel_','',$relName);
							$relName = explode('to',$relName);
							
							// <!--int(10)--> -- // -- ".htmlspecialchars($GLOBALS['LANG']->sL($vColumn['label']))
							$umlCodeContentFields[$v][$kColumn] = "+ ".$kColumn." &#8599;(исхд.)\n";
							$umlCodeContentRelation .= end(explode('\\',$v));
							$umlCodeContentRelation .= ' "'.$relName[0].'" --o "'.$relName[1].'" ';
							$umlCodeContentRelation .= end(explode('\\',$annotationForeignModel)) . " ";
							$umlCodeContentRelation .= ":\"".str_replace(['Rel_','to'],['',':'],end(explode('\\',$vColumn['AirTable.Class'])))."\\n".$kColumn."\\n".$annotationForeignKey."\"\n";
							if($annotationForeignKey != ''){
								$umlCodeContentFields[$annotationForeignModel][$annotationForeignKey] = "+ ".$annotationForeignKey." &#8601;(вхд.)\n";
							} else {
								$umlCodeContentFields[$annotationForeignModel][] = '';
							}
						}else{
							if($kColumn == 'uid'){
								// <!--int(10)--> -- // -- ".htmlspecialchars($GLOBALS['LANG']->sL($vColumn['label']))
								// $umlCodeContentFields[$v][$kColumn] .= "- ".$kColumn;
							}
						}
					}
				}
			}
		}
		foreach($umlCodeContentFields as $class => $columns){
			if(!class_exists($class)){
				continue;
			}
			$table = BaseUtility::getTableNameFromClass($class);
			
				// RType
				$RTypeStr = '';
				$RType = [];
				if(BaseUtility::hasClassTrait($class,'Litovchenko\AirTable\Field\Traits\RType')){
					foreach($GLOBALS['TCA'][$table]['columns']['RType']['config']['items'] as $k => $v){
						$RType[] = 'RType: ' . $v[1] . ' // ' . $v[0];
					}
					if(count($RType)>0){
						$RTypeStr = "\n".implode("\n",$RType);
					}
				}
			
			$umlCodeContent .= "class ".end(explode('\\',$class))." {
".BaseUtility::getClassAnnotationValueNew($class,'AirTable\Parameter\Label')."\n".$table.$RTypeStr."
==
- uid
- pid\n".implode("",$columns)."}
\n";
		}
		return '@startuml
'.htmlspecialchars($umlCodeContent.$umlCodeContentRelation).'@enduml';
	}
	
    /**
     * Main action for administration
     */
    public function listAction()
    {
		// Параметры _GET
		$paramExt = BaseUtility::_GETuc('paramExt',$this->extNameDir);
		$paramSubDomain = BaseUtility::_GETuc('paramSubDomain','Root');
		$paramClass = BaseUtility::_GETuc('paramClass','');
	
		// Создаем модель
		$sqlBuilderSelect = $paramClass::getModel();
		$sqlBuilderCount = $paramClass::getModel();
		$table = BaseUtility::getTableNameFromClass($paramClass);
		
		// Если идет выполнить с записью
		$this->perform($table);
		
		// Дебаг
		$debugInController = '<br /><span style="color: green">'. BaseUtility::debugInController().'</span>';
		
		// Если отправлены формы фильтра, сортировки или настроек, ставим страницу №1
		#$_POST = \TYPO3\CMS\Core\Utility\GeneralUtility::_POST();
		#if($_POST['form1Apply'] != 0 or $_POST['form2Apply'] != 0 or $_POST['form3Apply'] != 0 ){
		#	$firstKey = \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('route').'_POST';
		#	$GLOBALS['BE_USER']->uc['air_table']['_POSTuc'][$firstKey]['form0Apply']['form0Page'] = 1;
		#	$GLOBALS['BE_USER']->writeUC();
		#}
		
		// 1 // select
		$columns = $GLOBALS['TCA'][$table]['columns'];
		foreach($columns as $k => $v){
		
			// Оставляем только выбранные колонки
			#if((in_array($k, BaseUtility::_POSTuc('form3Apply','tab3_columns','all')) && BaseUtility::_POSTuc('form3Apply','tab3_columns','all') != 'all') 
			#	|| ($k == 'uid' && BaseUtility::_POSTuc('form3Apply','tab3_columns','all') != 'all')){
			#		$sqlBuilderSelect = $sqlBuilderSelect->addSelect($table.'.'.$k);
			#}
			$AirTableFieldClass = $v['AirTable.Class'];
			if(method_exists($AirTableFieldClass, 'listControllerSqlBuilder')){ // Фильтр
				$sqlBuilderSelect = $AirTableFieldClass::listControllerSqlBuilder($this, $sqlBuilderSelect, $table, $k, $v);
			}
			if(method_exists($AirTableFieldClass, 'listControllerSqlBuilderWith')){ // Присоединение связей
				$sqlBuilderSelect = $AirTableFieldClass::listControllerSqlBuilderWith($this, $sqlBuilderSelect, $table, $k, $v);
			}
		}
		
		// 3 // order
		$paramOrder = BaseUtility::_POSTuc('form2Apply','form2Order');
		foreach($paramOrder as $k => $v){
			if($v != null){
				$sqlBuilderSelect = $sqlBuilderSelect->orderBy($table.'.'.$k,$v);
			}
		}
		
		// 4 // limit
		$page = BaseUtility::_POSTuc('form0Apply','form0Page',1);
		$lim = BaseUtility::_POSTuc('form3Apply','form3Limit',30); // 30
		if($lim>=0){
			$rowsCount = $sqlBuilderSelect->count(); // Кол-во строк всего в данном запросе
			if($page == 1){
				$sqlBuilderSelect = $sqlBuilderSelect->limit($lim)->offset(0);
				$SqlPageGo = 1;
			} else {
				$sqlBuilderSelect = $sqlBuilderSelect->limit($lim)->offset($lim*($page-1));
				$SqlPageGo = 1;
			}
		}
		
		$sql = $sqlBuilderSelect->toSql();
		$rows = $sqlBuilderSelect->get()->toArray();
		$rowsCountDisplay = count($rows); // Кол-во выбранных строк 
		$rowsCountAllInDb = $sqlBuilderCount->count(); // Кол-во записей в базе всего
		
		$SqlPagePrev = $page-1;
		if($page < ceil($rowsCount/$lim)){
			$SqlPageNext = $page+1;
		} else {
			$SqlPageNext = 0;
		}
		
		#if($GLOBALS['BE_USER']->uc['air_table']['ListControllerTreeSwitcher'][$pid] == 1){
		#	$content = $this->getContent($pid);
		#	$scenario = 1;
		#} else {
			$content = '
			<style>
			table.remove-border { }
			table.remove-border > tbody > tr > th:first-child { border-left: none !important; }
			table.remove-border > tbody > tr > th:nth-last-child(-n+2) { border-right: none; }
			/* table.remove-border > tbody > tr > th:last-child { border-left: none; border-right: none; } */
			table.remove-border > tbody > tr > td:first-child { border-left: none !important; }
			table.remove-border > tbody > tr > td:nth-last-child(-n+2) { border-right: none; }
			/* table.remove-border > tbody > tr > td:last-child { border-left: none; border-right: none; } */
			</style>
			<div style="overflow-x: auto; border-left: #ccc 1px solid; border-right: #ccc 1px solid;">
			<table class="table table-striped table-bordered table-hover remove-border" style="width: 100%; margin-bottom: 0; border-left: none; border-right: none;">
			<tbody>
				'.$this->rowContent($rows, $table).'
			</tbody>
			</table>
			</div>';
			$scenario = 0;
		#}
		
		// Наличие полей корзины, вкл/выкл
		$fieldDelete = 0;
		if(isset($GLOBALS['TCA'][$table]['ctrl']['delete'])) {
			$fieldDelete = 1;
		}
		$fieldDisabled = 0;
		if(isset($GLOBALS['TCA'][$table]['ctrl']['enablecolumns']['disabled'])) {
			$fieldDisabled = 1;
		}
		
		// Экспорт/импорт (ccs
		/* Ссылки перенес на верх */
		
		// Litovchenko\AirTable\Model\SysNote
		$rowsNote = \Litovchenko\AirTable\Model\SysNote::getModel()
			->where('ex_air_table_modelname','=',$paramClass)
			->with('getDefaultRelation_cruser_Method')
			->orderBy('crdate','desc')
			->get()
			->toArray();
			
		// Добавляем ссылку на редактирование
		foreach($rowsNote as $kNote => $vNote){
			$backendLink = \TYPO3\CMS\Backend\Utility\BackendUtility::getModuleUrl(
				'record_edit',
				array(
					'edit[sys_note]['.$vNote['uid'].']' => 'edit',
					'returnUrl' => \TYPO3\CMS\Core\Utility\GeneralUtility::getIndpEnv('REQUEST_URI')
				)
			);
			$rowsNote[$kNote]['backendLink'] = $backendLink;
		}
			
		// Если пусто
		$recordContentNoteEmpty = '';
		if(count($rowsNote) == 0){
			$icon = $this->iconFactory->getIcon('mimetypes-x-sys_note', Icon::SIZE_DEFAULT);
			$recordContentNoteEmpty = '<div style="margin: 50px;"><h3>'.$icon.' Без заметок!</h3></div>';
		}
		
		// Если есть категории
		if(BaseUtility::isModelSupportOneCategory($paramClass) || BaseUtility::isModelSupportManyCategories($paramClass)){
			if(BaseUtility::isModelSupportOneCategory($paramClass)&&BaseUtility::isModelSupportManyCategories($paramClass)){
				$showTreeInfobox = 'В модели используется два поля для выбора категорий ("Категория" и "Категории (возможно несколько))';
				$showTreeInfoboxState = 2;
			}elseif(BaseUtility::isModelSupportManyCategories($paramClass)){
				$showTreeInfobox = 'Один элемент может состоять в нескольких категориях';
				$showTreeInfoboxState = -1;
				$contentCategory = $this->rowContentCategory();
			} else {
				$showTreeInfobox = 'Один элемент может состоять только в 1 категории';
				$showTreeInfoboxState = -1;
				$contentCategory = $this->rowContentCategory();
			}
			$showTree = 1;
		
		// Если есть родитель
		} elseif(BaseUtility::hasClassTrait($paramClass,'Litovchenko\AirTable\Field\Traits\ParentId')){
			$contentCategory = $this->rowContentElement();
			$showTreeInfobox = 'Построение дерева на базе одной таблицы по полю "Родителя"';
			$showTreeInfoboxState = -1;
			$showTree = 1;
			
		} else {
			$showTree = 0;
			#$showTreeReset = 0;
		}
		
		// Список колонок на редактирование
		#$columnsOnly = [];
		#$columnsOnlyAll = [];
		#$columns = $GLOBALS['TCA'][$table]['columns'];
		#foreach($columns as $k => $v){
		#	$columnsOnlyAll[] = $k;
		#	if(in_array($k,BaseUtility::_POSTuc('form3Apply','form3Columns'))){
		#		$columnsOnly[] = $k;
		#	}
		#}
		
        $assignedValues = [
			'h1Content' => BaseUtility::getClassAnnotationValueNew($paramClass,'AirTable\Parameter\Label'),
			'icon' => $this->iconFactory->getIcon('extensions-air_table-icon_storage_tablerecord', Icon::SIZE_SMALL),
			'iconCategory' => $this->iconFactory->getIcon('extensions-air_table-icon_storage_tablerecord', Icon::SIZE_SMALL),
			
			#'pre' => print_r($GLOBALS['_GET'],true), // Служебное
			#'pre' => print_r($GLOBALS['_POST'],true), // Служебное
			#'pre' => print_r($GLOBALS['BE_USER']->uc['air_table']['_POSTuc'],true), // Служебное
			
			'recordContentTable' => $this->rowContentTable(),
			'recordContent' => $content,
			'recordContentCategory' => $contentCategory,
			'recordContentNote' => $rowsNote,
			'recordContentNoteEmpty' => $recordContentNoteEmpty,
			'recordContentNoteCount' => count($rowsNote),
			'scenario' => 0,
			'showTree' => $showTree,
			'showTreeInfobox' => $showTreeInfobox,
			'showTreeInfoboxState' => $showTreeInfoboxState,
			
			'fieldDelete' => $fieldDelete,
			'fieldDisabled' => $fieldDisabled,
			'columnsOnly' => count($columnsOnly)>0?implode(',',$columnsOnly):implode(',',$columnsOnlyAll),
			'accessTableModify' => BaseUtility::BeUserAccessTableModify($table),
			
			'modelName' => $paramClass,
			'sql' => $sql . $debugInController,
			'debug' => $GLOBALS['TYPO3_CONF_VARS']['BE']['debug'],
			'tablesIcon' => $this->iconFactory->getIcon('actions-database', Icon::SIZE_SMALL),
			
			'rowsCount' => intval($rowsCount),
			'rowsCountDisplay' => intval($rowsCountDisplay),
			'rowsCountAllInDb' => intval($rowsCountAllInDb),

			'SqlPage' => $page,
			'SqlPageGo' => $SqlPageGo,
			'SqlPagePrev' => $SqlPagePrev,
			'SqlPageNext' => $SqlPageNext,
			
			'form' => $this->formHtmlContent($table, $rowsCount, $lim),
			'form0Apply' => BaseUtility::_POSTuc('form0Apply','form0Apply',0),
			'form1Apply' => BaseUtility::_POSTuc('form1Apply','form1Apply',0),
			'form2Apply' => BaseUtility::_POSTuc('form2Apply','form2Apply',0),
			'form3Apply' => BaseUtility::_POSTuc('form3Apply','form3Apply',0),
			#'form4Apply' => BaseUtility::_POSTuc('form4Apply','form4Apply',0),
			#'form5Apply' => BaseUtility::_POSTuc('form5Apply','form5Apply',0),
			
			#'backendLinkExport' => $backendLinkExport,
			#'backendLinkImport' => $backendLinkImport,
			
			// Если идет выбор записей
			'recordSelection' => \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('recordSelection'),
			'recordSelectionFieldname' => \TYPO3\CMS\Core\Utility\GeneralUtility::_GET('recordSelectionFieldname')
        ];
        $this->view->assignMultiple($assignedValues);
    }
	
	public function rowContentTable(){
		$paramExt = BaseUtility::_GETuc('paramExt',$this->extNameDir);
		$paramSubDomain = BaseUtility::_GETuc('paramSubDomain','Root');
		$paramClass = BaseUtility::_GETuc('paramClass','');
		
		// 2
		$classList = [];
		$models = BaseUtility::getLoaderClasses('Classes/Domain/');
		foreach($models as $class) {
			$class_parents = class_parents($class);
			// Регистрация новых пользовательских моделей
			if(in_array('Litovchenko\AirTable\Model\BaseCrud',$class_parents) && BaseUtility::getClassAnnotationValueNew($class,'AirTable') == 'Table'){
				$extName = BaseUtility::getExtNameFromClassPath($class);
				$subDomain = BaseUtility::getSubDomainNameFromClassPath($class);
				$classList[$extName][$subDomain][] = $class;
			}
			// Регистрация моделей на основе существующей таблицы TYPO3
			if(in_array('Litovchenko\AirTable\Model\BaseCrud2',$class_parents) && BaseUtility::getClassAnnotationValueNew($class,'AirTable') == 'Table'){
				$extName = BaseUtility::getExtNameFromClassPath($class);
				$subDomain = BaseUtility::getSubDomainNameFromClassPath($class);
				$classList[$extName][$subDomain][] = $class;
			}
		}
		
		// Сортировка
		foreach($classList as $extName => $subDomain){
			ksort($classList[$extName]);
			// sort($classList[$extName][$subDomain]);
		}
		
		// Контент
		$content = '';
		
		// Переход в корень
		$getArgs = [
			'paramExt' => $paramExt,
			'paramSubDomain' => 'Root'
		];
		$uriBuilder = $this->controllerContext->getUriBuilder();
		$url = $uriBuilder->setArguments($getArgs)->uriFor('index', [], 'List');
		
		$icon = $this->iconFactory->getIcon('extensions-air_table-icon_storage_subdomain', Icon::SIZE_SMALL);
		$content .= '
			<li>
				<span class="list-tree-group">
				<span class="list-tree-label" style="white-space: nowrap;">
					<a href="'.$url.'" class="'.$active.'" style="margin-bottom: 2px;">'.$icon.' ..</a>
				</span>
				</span>
			</li>
		';
		foreach($classList[$paramExt] as $k => $v){
			if($k != $paramSubDomain){
				continue;
			}
			foreach($v as $k2 => $v2){
				$getArgs = [
					'paramExt' => $paramExt,
					'paramSubDomain' => $paramSubDomain,
					'paramClass' => $v2, // table
				];
				$uriBuilder = $this->controllerContext->getUriBuilder();
				$url = $uriBuilder->setArguments($getArgs)->uriFor('list', [], 'List');
				$title = BaseUtility::getClassAnnotationValueNew($v2,'AirTable\Parameter\Label');
				
				// Access
				$accessOverlay = '';
				if(BaseUtility::BeUserAccessTableSelect(BaseUtility::getTableNameFromClass($v2)) == false){
					$url = '#';
					$accessOverlay = 'overlay-locked';
				}
				
				if($paramClass == $v2){
					$active = 'badge badge-success';
				} else {
					$active = '';
				}
				$content .= '
					<li>
						<span class="list-tree-group">
						<span class="list-tree-label" style="white-space: nowrap;">
							<a href="'.$url.'" class="'.$active.'" style="margin-bottom: 2px;">
								'.$this->iconFactory->getIcon('extensions-air_table-icon_storage_tablerecord', Icon::SIZE_SMALL, $accessOverlay).' 
								'.$title.'
							</a>
						</span>
						</span>
					</li>
				';
			}
		}
		
		return '<ul class="list-tree text-monospace">'.$content.'</ul>';
	}
	
	public function rowContentCategory($parentId = 0){
		$content = '';
		
		$paramClass = BaseUtility::_GETuc('paramClass','').'Category';
		$table = BaseUtility::getTableNameFromClass($paramClass);
		$sqlBuilderSelect = $paramClass::getModel();
		
		// Приоритет отдаеться категориям "Many",
		// если определено два трейта
		if(BaseUtility::isModelSupportManyCategories(BaseUtility::_GETuc('paramClass',''))){
			$field = 'category_many';
		} else {
			$field = 'category_id';
		}
		if(BaseUtility::hasClassTrait(BaseUtility::_GETuc('paramClass',''),'Litovchenko\AirTable\Field\Traits\ParentId')){
			$field2 = 'parent_id';
		} else {
			$field2 = ''; 
		}
		
		if(BaseUtility::hasClassTrait($paramClass,'Litovchenko\AirTable\Field\Traits\ParentId')){
			$rows = $sqlBuilderSelect
				->where('parent_id','=',$parentId)
				->orderBy($GLOBALS['TCA'][$table]['ctrl']['label'])
				->get()
				->toArray();
		} else {
			$rows = $sqlBuilderSelect
				->orderBy($GLOBALS['TCA'][$table]['ctrl']['label'])
				->get()
				->toArray();
		}
		
		foreach($rows as $k => $v){
			if($v[$GLOBALS['TCA'][$table]['ctrl']['delete']] == 1) {
				$icon = $this->getIconForRecord($table, $v, \TYPO3\CMS\Core\Imaging\Icon::SIZE_SMALL);
			} else {
				$icon = $this->iconFactory->getIconForRecord($table, $v, \TYPO3\CMS\Core\Imaging\Icon::SIZE_SMALL);
			}
			
			// Sub 1 (папки)
			$treeLink = '';
			$contentSubCategory = '';
			$contentSubCategoryElement = '';
			if(BaseUtility::hasClassTrait($paramClass,'Litovchenko\AirTable\Field\Traits\ParentId')){
				
				// Sub 2 (элементы)
				$rowsCount2 = 0;
				if(BaseUtility::hasClassTrait(BaseUtility::_GETuc('paramClass',''),'Litovchenko\AirTable\Field\Traits\ParentId')){
					$rowsCount2 = $this->rowContentCategoryElementCount(0,$v['uid']);
				}
				
				$rowsCount = $sqlBuilderSelect->where('parent_id','=',$v['uid'])->count(); // Кол-во подпапок
				if($rowsCount+$rowsCount2 > 0){
					// Переключатель веточек
					BaseUtility::_POSTuc('form4Apply_'.$v['uid'],'form4Apply_'.$v['uid'],0); // Активация формы
					if(BaseUtility::_POSTuc('form4Apply_'.$v['uid'],'form4Apply_'.$v['uid'].'_branch','') == 'open'){
						$switch_value = 'open';
						$js_value = 'closed';
						$contentSubCategory = $this->rowContentCategory($v['uid']);
						if($rowsCount2>0){
							$contentSubCategoryElement = $this->rowContentCategoryElement(0,$v['uid']);
						}
					} else {
						$switch_value = 'closed';
						$js_value = 'open';
						$contentSubCategory = '';
					}
					// Веточка
					$treeLink = '
					<a href="#" class="list-tree-control list-tree-control-'.$switch_value.'" 
						style="top: -2px;" onclick="branchSwitch(\'form4\','.$v['uid'].',\''.$js_value.'\'); return false;"
					>
						<i class="fa"></i>
					</a>';
				}
			}

			// Соединяем две ветки в 1 убирая между ними дублирование "</ul><ul class="list-tree text-monospace">"
			if(trim($contentSubCategory) != '' && trim($contentSubCategoryElement) != ''){
				$contentSubCategory = preg_replace('/<\/ul>$/is','',trim($contentSubCategory));
				$contentSubCategoryElement = preg_replace('/^<ul class="list-tree text-monospace">/is','',trim($contentSubCategoryElement));
			}
			
			// Активность выбранной категории
			$paramFilter = BaseUtility::_POSTuc('form1Apply','form1FieldValue_'.$field,'');
			if(in_array($v['uid'],explode(',',$paramFilter))){
				$active = 'badge badge-success';
			} else {
				$active = '';
			}
			
			// Название записи
			$title = '';
			if(BaseUtility::hasClassTrait($paramClass,'Litovchenko\AirTable\Field\Traits\RType')){
				$title .= '['.BaseUtility::getTcaFieldItem($table,'RType',$v['RType'])[0].'] ';
			}
			if(BaseUtility::hasClassTrait($paramClass,'Litovchenko\AirTable\Field\Traits\Title')){
				$title .= $v['title'];
			} else {
				$title .= $v['uid'];
			}
			
			$content .= '
				<li>
					<span class="list-tree-group">
					<span class="list-tree-label" style="white-space: nowrap;">
						'.$treeLink.'
						<a href="#" class="'.$active.'" style="margin-bottom: 2px;" onClick="filterCategory(\''.$field.'\',\''.$field2.'\','.$v['uid'].',\'\');">'.$icon.' '.$title.'</a>
						&nbsp;
						'.$contentSubCategory.'
						'.$contentSubCategoryElement.'
					</span>
					</span>
				</li>
			';
		}
		
		if(count($rows)!=0){
			if($parentId == 0){
				$icon = $this->iconFactory->getIcon('apps-filetree-root', Icon::SIZE_SMALL);
				// Активность выбранной категории
				$paramFilter = BaseUtility::_POSTuc('form1Apply','form1FieldValue_'.$field,'');
				if($paramFilter == ''){
					$active = 'badge badge-default';
				} else {
					$active = '';
				}
				return '
				<div class="text-monospace" style="line-height: 21px;">
					<a href="#" class="'.$active.'" style="margin-bottom: 2px;" onClick="filterCategoryReset(\''.$field.'\',\''.$field2.'\');">'.$icon.' Все категории</a>
				</div>
				<ul class="list-tree text-monospace">
					'.$content.'
				</ul>';
			} else {
				return '<ul class="list-tree text-monospace">'.$content.'</ul>';
			}
		} else {
			if(count($rows) == 0 && $parentId == 0){
				$icon = $this->iconFactory->getIcon('apps-filetree-folder-default', Icon::SIZE_DEFAULT, 'overlay-missing');
				return '<div style="margin: 25px;"><h3>'.$icon.' Категории не найдены!</h3></div>';
			} else {
				return '';
			}
		}
	}
	
	public function rowContentCategoryElementCount($parentId = 0, $categoryId = 0){
		// Кол-во подпапок
		$paramClass = BaseUtility::_GETuc('paramClass','');
		$sqlBuilderSelect = $paramClass::getModel();
		
		if(BaseUtility::isModelSupportOneCategory($paramClass)){
			$rowsCount = $sqlBuilderSelect->where('parent_id','=',$parentId)->where('category_id','=',$categoryId)->count();
		} elseif(BaseUtility::isModelSupportManyCategories($paramClass)){
			$rowsCount = $sqlBuilderSelect->where('parent_id','=',$parentId)->whereHas('getDefaultRelation_category_many_Method', function($q) use ($categoryId) {
				$q->whereIn('uid_foreign', [$categoryId]);
			})->count();
		}
		return $rowsCount;
	}
	
	public function rowContentCategoryElement($parentId = 0, $categoryId = 0){
		$content = '';
		
		$paramClass = BaseUtility::_GETuc('paramClass','');
		$table = BaseUtility::getTableNameFromClass($paramClass);
		$sqlBuilderSelect = $paramClass::getModel();
		
		$field = 'parent_id';
		
		if(BaseUtility::isModelSupportOneCategory($paramClass)){
			$field2 = 'category_id';
			$rows = $sqlBuilderSelect
				->where('parent_id','=',$parentId)
				->where('category_id','=',$categoryId)
				->orderBy($GLOBALS['TCA'][$table]['ctrl']['label'])
				->get()
				->toArray();
			
		} elseif(BaseUtility::isModelSupportManyCategories($paramClass)){
			$field2 = 'category_many';
			$rows = $sqlBuilderSelect
				->where('parent_id','=',$parentId)
				->whereHas('getDefaultRelation_category_many_Method', function($q) use ($categoryId) {
					$q->whereIn('uid_foreign', [$categoryId]);
				})
				->orderBy($GLOBALS['TCA'][$table]['ctrl']['label'])
				->get()
				->toArray();
		}
		
		foreach($rows as $k => $v){
			if($v[$GLOBALS['TCA'][$table]['ctrl']['delete']] == 1) {
				$icon = $this->getIconForRecord($table, $v, \TYPO3\CMS\Core\Imaging\Icon::SIZE_SMALL);
			} else {
				$icon = $this->iconFactory->getIconForRecord($table, $v, \TYPO3\CMS\Core\Imaging\Icon::SIZE_SMALL);
			}
			$treeLink = '';
			$contentSubCategory = '';
			#if(BaseUtility::hasClassTrait($paramClass,'Litovchenko\AirTable\Field\Traits\ParentId')){
				$rowsCount = $this->rowContentCategoryElementCount($v['uid'], $categoryId);
				if($rowsCount > 0){
					// Переключатель веточек
					BaseUtility::_POSTuc('form5Apply_'.$v['uid'],'form5Apply_'.$v['uid'],0); // Активация формы
					if(BaseUtility::_POSTuc('form5Apply_'.$v['uid'],'form5Apply_'.$v['uid'].'_branch','') == 'open'){
						$switch_value = 'open';
						$js_value = 'closed';
						$contentSubCategory = $this->rowContentCategoryElement($v['uid'], $categoryId);
					} else {
						$switch_value = 'closed';
						$js_value = 'open';
						$contentSubCategory = '';
					}
					// Веточка
					$treeLink = '
					<a href="#" class="list-tree-control list-tree-control-'.$switch_value.'" 
						style="top: -2px;" onclick="branchSwitch(\'form5\','.$v['uid'].',\''.$js_value.'\'); return false;"
					>
						<i class="fa"></i>
					</a>';
				}
			#}
			
			// Активность выбранной категории
			$paramFilter = BaseUtility::_POSTuc('form1Apply','form1FieldValue_'.$field,'');
			if(in_array($v['uid'],explode(',',$paramFilter))){
				$active = 'badge badge-info';
			} else {
				$active = '';
			}
			
			// Название записи
			$title = '';
			if(BaseUtility::hasClassTrait($paramClass,'Litovchenko\AirTable\Field\Traits\RType')){
				$title .= '['.BaseUtility::getTcaFieldItem($table,'RType',$v['RType'])[0].'] ';
			}
			if(BaseUtility::hasClassTrait($paramClass,'Litovchenko\AirTable\Field\Traits\Title')){
				$title .= $v['title'];
			} else {
				$title .= $v['uid'];
			}
			
			$content .= '
				<li>
					<span class="list-tree-group">
					<span class="list-tree-label" style="white-space: nowrap;">
						'.$treeLink.'
						<a href="#" class="'.$active.'" style="margin-bottom: 2px;" onClick="filterCategory(\''.$field.'\',\''.$field2.'\','.$v['uid'].',\''.$categoryId.'\');">'.$icon.' '.$title.'</a>
						&nbsp;
						'.$contentSubCategory.'
					</span>
					</span>
				</li>
			';
		}
		
		if(count($rows)!=0){
			if($parentId == 0){
				$icon = $this->iconFactory->getIcon('apps-filetree-root', Icon::SIZE_SMALL);
				// Активность выбранной категории
				$paramFilter = BaseUtility::_POSTuc('form1Apply','form1FieldValue_'.$field,'');
				if($paramFilter == ''){
					$active = 'badge badge-default';
				} else {
					$active = '';
				}
				return '
				<!--
				<div class="text-monospace" style="line-height: 21px;">
					<a href="#" class="'.$active.'" style="margin-bottom: 2px;" onClick="filterCategoryReset(\''.$field.'\');">'.$icon.' Все записи</a>
				</div>
				-->
				<ul class="list-tree text-monospace">
					'.$content.'
				</ul>';
			} else {
				return '<ul class="list-tree text-monospace">'.$content.'</ul>';
			}
		} else {
			return '';
		}
	}
	
	public function rowContentElement($parentId = 0){
		$content = '';
		
		$paramClass = BaseUtility::_GETuc('paramClass','');
		$table = BaseUtility::getTableNameFromClass($paramClass);
		$sqlBuilderSelect = $paramClass::getModel();
		$field = 'parent_id';
		
		$rows = $sqlBuilderSelect
				->where('parent_id','=',$parentId)
				->orderBy($GLOBALS['TCA'][$table]['ctrl']['label'])
				->get()
				->toArray();
		
		foreach($rows as $k => $v){
			if($v[$GLOBALS['TCA'][$table]['ctrl']['delete']] == 1) {
				$icon = $this->getIconForRecord($table, $v, \TYPO3\CMS\Core\Imaging\Icon::SIZE_SMALL);
			} else {
				$icon = $this->iconFactory->getIconForRecord($table, $v, \TYPO3\CMS\Core\Imaging\Icon::SIZE_SMALL);
			}
			$treeLink = '';
			$contentSubCategory = '';
			if(BaseUtility::hasClassTrait($paramClass,'Litovchenko\AirTable\Field\Traits\ParentId')){
				$rowsCount = $sqlBuilderSelect->where('parent_id','=',$v['uid'])->count(); // Кол-во подпапок
				if($rowsCount > 0){
					// Переключатель веточек
					BaseUtility::_POSTuc('form5Apply_'.$v['uid'],'form5Apply_'.$v['uid'],0); // Активация формы
					if(BaseUtility::_POSTuc('form5Apply_'.$v['uid'],'form5Apply_'.$v['uid'].'_branch','') == 'open'){
						$switch_value = 'open';
						$js_value = 'closed';
						$contentSubCategory = $this->rowContentElement($v['uid']);
					} else {
						$switch_value = 'closed';
						$js_value = 'open';
						$contentSubCategory = '';
					}
					// Веточка
					$treeLink = '
					<a href="#" class="list-tree-control list-tree-control-'.$switch_value.'" 
						style="top: -2px;" onclick="branchSwitch(\'form5\','.$v['uid'].',\''.$js_value.'\'); return false;"
					>
						<i class="fa"></i>
					</a>';
				}
			}
			
			// Активность выбранной категории
			$paramFilter = BaseUtility::_POSTuc('form1Apply','form1FieldValue_'.$field,'');
			if(in_array($v['uid'],explode(',',$paramFilter))){
				$active = 'badge badge-success';
			} else {
				$active = '';
			}
			
			// Название записи
			$title = '';
			if(BaseUtility::hasClassTrait($paramClass,'Litovchenko\AirTable\Field\Traits\RType')){
				$title .= '['.BaseUtility::getTcaFieldItem($table,'RType',$v['RType'])[0].'] ';
			}
			if(BaseUtility::hasClassTrait($paramClass,'Litovchenko\AirTable\Field\Traits\Title')){
				$title .= $v['title'];
			} else {
				$title .= $v['uid'];
			}
			
			$content .= '
				<li>
					<span class="list-tree-group">
					<span class="list-tree-label" style="white-space: nowrap;">
						'.$treeLink.'					
						<a href="#" class="'.$active.'" style="margin-bottom: 2px;" onClick="filterCategory(\''.$field.'\',\'\','.$v['uid'].',\'\');">'.$icon.' '.$title.'</a>
						&nbsp;
						'.$contentSubCategory.'
					</span>
					</span>
				</li>
			';
		}
		
		if(count($rows)!=0){
			if($parentId == 0){
				$iconAll = $this->iconFactory->getIcon('apps-filetree-root', Icon::SIZE_SMALL);
				$iconRoot = $this->iconFactory->getIcon('apps-pagetree-folder-root', Icon::SIZE_SMALL);
					
				// Активность 1
				$paramFilter = BaseUtility::_POSTuc('form1Apply','form1FieldValue_'.$field,'');
				if($paramFilter == ''){
					$activeAll = 'badge badge-default';
				} else {
					$activeAll = '';
				}
				
				// Активность 2
				#$paramFilter = BaseUtility::_POSTuc('form1Apply','form1FieldValue_'.$field,'');
				#if($paramFilter === 0){
				#	$activeRoot = 'badge badge-default';
				#} else {
				#	$activeRoot = '';
				#}
				
				return '
				<div class="text-monospace" style="line-height: 21px;">
					<a href="#" class="'.$activeAll.'" style="margin-bottom: 2px;" onClick="filterCategoryReset(\''.$field.'\');">'.$iconAll.' Все записи</a>
				</div>
				<!--
				<div class="text-monospace" style="line-height: 21px;">
					<a href="#" class="'.$activeRoot.'" style="margin-bottom: 2px;" onClick="filterCategory(\''.$field.'\',\'\',\'0\',\'\');">'.$iconRoot.' Записи в корне</a>
				</div>
				-->
				<ul class="list-tree text-monospace">
					'.$content.'
				</ul>
				';
			} else {
				return '<ul class="list-tree text-monospace">'.$content.'</ul>';
			}
		} else {
			if(count($rows) == 0 && $parentId == 0){
				$icon = $this->iconFactory->getIcon('apps-pagetree-page-default', Icon::SIZE_DEFAULT, 'overlay-missing');
				return '<div style="margin: 25px;"><h3>'.$icon.' Записи не найдены!</h3></div>';
			} else {
				return '';
			}
		}
	}
	
	public function perform($table = ''){
		$_POST = \TYPO3\CMS\Core\Utility\GeneralUtility::_POST();
		$perform = $_POST['perform'];
		$idList = $_POST['idList'];
		if(!empty($idList)){
			switch($perform){
				case 'edit':
					$backendLink = \TYPO3\CMS\Backend\Utility\BackendUtility::getModuleUrl(
						'record_edit', [
							'edit['.$table.']['.$idList.']' => 'edit',
							'returnUrl' => \TYPO3\CMS\Core\Utility\GeneralUtility::getIndpEnv('REQUEST_URI')
						]
					);
					header('Location: ' . $backendLink);
				break;
				case 'edit_columnsOnly':
					$columnsOnly = $_POST['columnsList'];
					$backendLink = \TYPO3\CMS\Backend\Utility\BackendUtility::getModuleUrl(
						'record_edit', [
							'columnsOnly' => $columnsOnly,
							'edit['.$table.']['.$idList.']' => 'edit',
							'returnUrl' => \TYPO3\CMS\Core\Utility\GeneralUtility::getIndpEnv('REQUEST_URI')
						]
					);
					header('Location: ' . $backendLink);
				break;
				case 'hide':
					#$data = [];
					#$data[$table][$v] = [$fieldDisabled => 1];
					#$dataHandler = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Core\DataHandling\DataHandler::class);
					#$dataHandler->start($data, []);
					#$dataHandler->process_datamap();
					$fieldDisabled = $GLOBALS['TCA'][$table]['ctrl']['enablecolumns']['disabled'];
					foreach(explode(',',$idList) as $k => $v){
						DB::table($table)->where('uid', $v)->update([$fieldDisabled => 1]);
					}
				break;
				case 'unhide':
					#$data = [];
					#$data[$table][$v] = [$fieldDisabled => 0];
					#$dataHandler = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Core\DataHandling\DataHandler::class);
					#$dataHandler->start($data, []);
					#$dataHandler->process_datamap();
					$fieldDisabled = $GLOBALS['TCA'][$table]['ctrl']['enablecolumns']['disabled'];
					foreach(explode(',',$idList) as $k => $v){
						DB::table($table)->where('uid', $v)->update([$fieldDisabled => 0]);
					}
				break;
				case 'trash':
					#$data = [];
					#$data[$table][$v] = [$fieldDelete => 1];
					#$dataHandler = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Core\DataHandling\DataHandler::class);
					#$dataHandler->start($data, []);
					#$dataHandler->process_datamap();
					$fieldDelete = $GLOBALS['TCA'][$table]['ctrl']['delete'];
					foreach(explode(',',$idList) as $k => $v){
						DB::table($table)->where('uid', $v)->update([$fieldDelete => 1]);
					}
				break;
				case 'untrash':
					#$cmd = [];
					#$cmd[$table][$v]['undelete'] = 1;
					#$dataHandler = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Core\DataHandling\DataHandler::class);
					#$dataHandler->start([], $cmd);
					#$dataHandler->process_cmdmap();
					$fieldDelete = $GLOBALS['TCA'][$table]['ctrl']['delete'];
					foreach(explode(',',$idList) as $k => $v){
						DB::table($table)->where('uid', $v)->update([$fieldDelete => 0]);
					}
				break;
				case 'read':
					$getArgs = [
						'paramExt' => BaseUtility::_GETuc('paramExt',$this->extNameDir),
						'paramSubDomain' => BaseUtility::_GETuc('paramSubDomain','Root'),
						'paramClass' => BaseUtility::_GETuc('paramClass',''),
						'idList' => $idList
					];
					$uriBuilder = $this->controllerContext->getUriBuilder();
					$url = $uriBuilder->setArguments($getArgs)->uriFor('show', [], 'List');
					header('Location: ' . $url);
				break;
				#case 'read_columnsOnly':
				#	$getArgs = [
				#		'paramExt' => BaseUtility::_GETuc('paramExt',$this->extNameDir),
				#		'paramSubDomain' => BaseUtility::_GETuc('paramSubDomain','Root'),
				#		'paramClass' => BaseUtility::_GETuc('paramClass',''),
				#		'idList' => $idList,
				#		'columnsOnly' => $_POST['columnsList'],
				#	];
				#	$uriBuilder = $this->controllerContext->getUriBuilder();
				#	$url = $uriBuilder->setArguments($getArgs)->uriFor('show', [], 'List');
				#	header('Location: ' . $url);
				#break;
			}
		}
	}
	
	public function formHtmlContent($table='', $rowsCount, $lim){
		$ar = [];
		
		$_POST = \TYPO3\CMS\Core\Utility\GeneralUtility::_POST();
		#print "<pre>";
		#print_r($_POST);
		#print "</pre>";
		
		// tab 0 (list)
		$pageListAr = [];
		$pageList = ceil($rowsCount/$lim);
		for($i = 1; $i <= $pageList; $i++){
			$selected = '';
			if($i == BaseUtility::_POSTuc('form0Apply','form0Page',1)){
				$selected = 'selected';
			}
			$ar['tab0']['page'] .= '<option value="'.$i.'" '.$selected.'>Страница '.$i.'</option>';
			$pageListAr[] = $i;
		}
		if(!in_array(BaseUtility::_POSTuc('form0Apply','form0Page',1),$pageListAr)){
			$ar['tab0']['page'] .= '<option selected style="background: red; color: white;">Страница не существует</option>';
			$ar['tab0']['pageBrokenSelectStyle'] = 'border: red 1px solid;';
		}

		// (+) tab 1 (filter)
		$columns = $GLOBALS['TCA'][$table]['columns'];
		foreach($columns as $k => $v){
			// Pid Пропускаем, если записи всегда в корне
			if($GLOBALS['TCA'][$table]['ctrl']['rootLevel'] == 1 && $k == 'pid'){
				continue;
			}
			
			$filterContent = '';
			$AirTableFieldClass = $v['AirTable.Class'];
			if(method_exists($AirTableFieldClass, 'listControllerHtmlFilter')){
				$uriBuilder = $this->controllerContext->getUriBuilder();
				$filterContent = $AirTableFieldClass::listControllerHtmlFilter($this, $table, $k, $v, $uriBuilder);
			} else {
				$filterContent = '<input class="form-control btn-sm" name="" value="" disabled>';
			}
			$noConfig = '';
			if(!$v['AirTable.Class']){
				$noConfig = 'color: red';
				continue; // Не выводим их - New!!!
			}
			$paramFilterTemp = BaseUtility::_POSTuc('form1Apply','form1Field_'.$k,'');
			$debugStr = '';
			if($GLOBALS['TYPO3_CONF_VARS']['BE']['debug'] == true){
				$debugStr = '<br /><code style="color: #b4b2b2;">'.$v['AirTable.Class'].'.'.$k.'</code>';
			}
			$ar['tab1']['columns'] .= '
			<tr class="'.(!empty($paramFilterTemp)?'success':'').'">
				<td align="right">
					<b style="'.$noConfig.'">
						'.htmlspecialchars($GLOBALS['LANG']->sL($v['label'])).'
						'.$debugStr.'
					</b>
				</td>
				<td>'.$filterContent.'</td>
			</tr>';
		}
		
		// (+) tab 2 (order)
		$paramOrder = BaseUtility::_POSTuc('form2Apply','form2Order');
		$columns = $GLOBALS['TCA'][$table]['columns'];
		foreach($columns as $k => $v){
			// Pid Пропускаем, если записи всегда в корне
			if($GLOBALS['TCA'][$table]['ctrl']['rootLevel'] == 1 && $k == 'pid'){
				continue;
			}
			
			$disabled = '';
			$AirTableFieldClass = $v['AirTable.Class'];
			if(method_exists($AirTableFieldClass, 'listControllerSqlBuilderOrder')){
				if($AirTableFieldClass::listControllerSqlBuilderOrder($this, $table, $k, $v)!=true){
					$disabled = 'disabled';
				}
			} else {
				$disabled = 'disabled';
			}
			$noConfig = '';
			if(!$v['AirTable.Class']){
				$noConfig = 'color: red';
				continue; // Не выводим их - New!!!
			}
			$debugStr = '';
			if($GLOBALS['TYPO3_CONF_VARS']['BE']['debug'] == true){
				$debugStr = '<br /><code style="color: #b4b2b2;">'.$v['AirTable.Class'].'.'.$k.'</code>';
			}
			$ar['tab2']['columns'] .= '
			<tr class="'.($paramOrder[$k] == 'ASC' || $paramOrder[$k] == 'DESC'?'success':'').'">
			<td align="right">
				<b style="'.$noConfig.'">
					'.htmlspecialchars($GLOBALS['LANG']->sL($v['label'])).'
					'.$debugStr.'
				</b>
			</td>
			<td>
				<select name="form2Order['.$k.']" class="form-control btn-sm" style="" '.$disabled.'>
					<option value="">По умолчанию</option>
					<option value="ASC" '.($paramOrder[$k] == 'ASC'?'selected':'').'>ASC (по возрастанию 1,2,3)</option>
					<option value="DESC" '.($paramOrder[$k] == 'DESC'?'selected':'').'>DESC (по убыванию 3,2,1)</option>
				</select>
			</td>
			</tr>';
		}

		// (+) tab 3 (columns)
		$paramClass = BaseUtility::_GETuc('paramClass','');
		$columns = $GLOBALS['TCA'][$table]['columns'];
		foreach($columns as $k => $v){
			// Pid Пропускаем, если записи всегда в корне
			if($GLOBALS['TCA'][$table]['ctrl']['rootLevel'] == 1 && $k == 'pid'){
				continue;
			}
			
			$checked = '';
			$disabled = '';
			if($k == 'uid' || $k == 'pid' || $k == 'RType'){
				$checked = 'checked';
				$disabled = 'disabled';
			}
			if(in_array($k,BaseUtility::_POSTuc('form3Apply','form3Columns'))){ // || BaseUtility::_POSTuc('form3Apply','form3Columns','all') == 'all'
				$checked = 'checked';
			}
			
			// Если по умолчанию НЕ показывать!
			if(BaseUtility::_POSTuc('form3Apply','form3Columns','all') == 'all' 
			&& BaseUtility::getClassFieldAnnotationValueNew($paramClass,$k,'AirTable\Field\Parameter\Show') == 1){
				$checked = 'checked';
			}
			
			$noConfig = '';
			if(!$v['AirTable.Class']){
				$noConfig = 'color: red';
				// continue; // Не выводим их - New!!!
			}
			$debugStr = '';
			if($GLOBALS['TYPO3_CONF_VARS']['BE']['debug'] == true){
				$debugStr = '<br /><code style="color: #b4b2b2;">'.$v['AirTable.Class'].'.'.$k.'</code>';
			}
			$ar['tab3']['columns'] .= '
			<tr class="'.($checked == 'checked'?'success':'').'">
			<td align="right">
				<b style="'.$noConfig.'">
					'.htmlspecialchars($GLOBALS['LANG']->sL($v['label'])).'
					'.$debugStr.'
				</b>
			</td>
			<td>
				<label class="btn btn-default btn-sm" style="margin: 0; margin-bottom: 0px; padding: 0 7px 0 7px; width: 100%; text-align: left; cursor: pointer;">
					<input type="checkbox" name="form3Columns[]" class="btn btn-default btn-sm" style="margin-top: 0px;" value='.$k.' '.$disabled.' '.$checked.'> 
					Включить в список
				</label>
			</td>
			</tr>';
		}
		
		$limit = [1=>1,2=>2,3=>3,4=>4,5=>5,10=>10,15=>15,30=>30,100=>100,1000=>1000,-1=>'Все'];
		foreach($limit as $k => $v){
			$selected = '';
			if($k == BaseUtility::_POSTuc('form3Apply','form3Limit',30)){
				$selected = 'selected';
			}
			$ar['tab3']['limit'] .= '
				<option value="'.$k.'" '.$selected.'>'.$v.'</option>
			';
		}
		
		$th_duplicate = [0=>'Не дублировать',1,2,3,4,5,10,50,100];
		foreach($th_duplicate as $k => $v){
			$selected = '';
			if($k == BaseUtility::_POSTuc('form3Apply','form3ThDuplicate',0)){
				$selected = 'selected';
			}
			$ar['tab3']['th_duplicate'] .= '
				<option value="'.$k.'" '.$selected.'>'.$v.'</option>
			';
		}
		
		$rows_render = [0=>'Горизонтально',1=>'Вертикально'];
		foreach($rows_render as $k => $v){
			$selected = '';
			if($k == BaseUtility::_POSTuc('form3Apply','form3RowsRender',0)){
				$selected = 'selected';
			}
			$ar['tab3']['rows_render'] .= '
				<option value="'.$k.'" '.$selected.'>'.$v.'</option>
			';
		}
		
		$td_realvalue = [0=>'Нет',1=>'Да'];
		foreach($td_realvalue as $k => $v){
			$selected = '';
			if($k == BaseUtility::_POSTuc('form3Apply','form3TdRealvalue',0)){
				$selected = 'selected';
			}
			$ar['tab3']['td_realvalue'] .= '
				<option value="'.$k.'" '.$selected.'>'.$v.'</option>
			';
		}
		
		return $ar;
	}
	
	public function rowContent($rows, $table=''){
		// Собираем контент
		$jCount = 1;
		$content = '';
		$contentTh = [];
		$contentTd = [];
		
		$contentTh_0 = '<th nowrap style="width: 29px; border-top: none; vertical-align: top; padding: 0;">
			<div style="position: relative; width: 29px; height: 30px;">
			<label class="btn btn-default btn-sm" style="position: absolute; top: 2px; left: 2px; margin: 0; padding: 0 5px 0 5px; cursor: pointer;">
				<input type="checkbox" class="btn btn-default btn-sm" style="margin-top: 0px;" name="foo_uid" value="0" onClick="toggleCheckboxes(this,\'foo_uid\'); countCheckboxes(this,\'foo_uid\');" disabled_ZZZ>
			</label>
			</div>
		</th>';
		
		$contentTd_0 = '<td nowrap style="width: 29px; border-top: none; vertical-align: top; padding: 0;">
			<div style="position: relative; width: 29px; height: 30px;">
			<label class="btn btn-default btn-sm" style="position: absolute; top: 2px; left: 2px; margin: 0; padding: 0 5px 0 5px; cursor: pointer;">
				<input type="checkbox" class="btn btn-default btn-sm" style="margin-top: 0px;" name="foo_uid" onClick="countCheckboxes(this,\'foo_uid\');" value="###UID###" disabled_ZZZ>
			</label>
			</div>
		</td>';
		
		// Собираем название колонок (в зависимости от настройки дублируются)
		$paramClass = BaseUtility::_GETuc('paramClass','');
		$columns = $GLOBALS['TCA'][$table]['columns'];
		foreach($columns as $k => $v){
			
			// Оставляем только выбранные колонки
			if(!in_array($k, BaseUtility::_POSTuc('form3Apply','form3Columns','all'))
				&& BaseUtility::_POSTuc('form3Apply','form3Columns','all') != 'all'
				&& $k != 'uid' && $k != 'pid' && $k != 'RType'){
				continue;
			}
			
			// Pid Пропускаем, если записи всегда в корне
			if($GLOBALS['TCA'][$table]['ctrl']['rootLevel'] == 1 && $k == 'pid'){
				continue;
			}
			
			// Если по умолчанию НЕ показывать!
			if(BaseUtility::_POSTuc('form3Apply','form3Columns','all') == 'all'
			&& BaseUtility::getClassFieldAnnotationValueNew($paramClass,$k,'AirTable\Field\Parameter\Show') == 0)
			{
				continue;
			}
				
			$AirTableFieldClass = $v['AirTable.Class'];
			if(method_exists($AirTableFieldClass, 'listControllerHtmlTh')){
				$contentTh[$k] = $AirTableFieldClass::listControllerHtmlTh($this, $table, $k, $v);
				$jCount++;
			} else {
				// Название поля
				$debugStr = '';
				if($GLOBALS['TYPO3_CONF_VARS']['BE']['debug'] == true){
					$debugStr = '<br /><code style="color: red;">'.$k.'</code>';
				}
				$contentTh[$k] = '<th nowrap style="border-top: none;">
					<div style="position: relative; color: red;">
						<!--
						<label class="btn btn-default btn-sm" style="position: absolute; top: -4px; left: -30px; margin: 0; padding: 0 5px 0 5px; cursor: pointer;">
							<input type="checkbox" class="btn btn-default btn-sm" style="margin-top: 0px;" name="field_uid" value="'.$k.'" onClick="toggleFields(this,\'field_uid\');" disabled_ZZZ>
						</label>
						-->
						'.htmlspecialchars($GLOBALS['LANG']->sL($v['label'])).'
						'.$debugStr.'
					</div>
				</th>';
				$jCount++;
			}
		}
		
		// Собираем строки
		$paramClass = BaseUtility::_GETuc('paramClass','');
		foreach($rows as $k => $v){
			// $content .= '<tr>';
			$columns = $GLOBALS['TCA'][$table]['columns'];
			foreach($columns as $k2 => $v2){
				
				// Оставляем только выбранные колонки
				if(!in_array($k2, BaseUtility::_POSTuc('form3Apply','form3Columns')) 
					&& BaseUtility::_POSTuc('form3Apply','form3Columns','all') != 'all'
					&& $k2 != 'uid' && $k2 != 'pid' && $k2 != 'RType'){
					continue;
				}
			
				// Pid Пропускаем, если записи всегда в корне
				if($GLOBALS['TCA'][$table]['ctrl']['rootLevel'] == 1 && $k2 == 'pid'){
					continue;
				}

				// Если по умолчанию НЕ показывать!
				if(BaseUtility::_POSTuc('form3Apply','form3Columns','all') == 'all' 
				&& BaseUtility::getClassFieldAnnotationValueNew($paramClass,$k2,'AirTable\Field\Parameter\Show') == 0)
				{
					continue;
				}
			
				// [RType] ->->-> Проверяем есть ли эта колонка в структуре?
				$fieldsInType = BaseUtility::getAllColumnsFromType($table, $v[$GLOBALS['TCA'][$table]['ctrl']['type']]);
				if(isset($GLOBALS['TCA'][$table]['ctrl']['type']) 
						&& isset($GLOBALS['TCA'][$table]['columns'][$GLOBALS['TCA'][$table]['ctrl']['type']]) 
							&& !in_array($k2,$fieldsInType) 
								&& $k2 != 'uid' 
									&& $k2 != 'pid'
										&& $table != 'sys_file' // Исключение (TCA оформленна не полностью)
				){
					$contentTd[$v['uid']][$k2] = '<td style="background: #ededed; color: #b4b2b2; vertical-align: top;" nowrap><i>Не существует</i></td>';
				} else {
					$AirTableFieldClass = $v2['AirTable.Class'];
					if(method_exists($AirTableFieldClass, 'listControllerHtmlTd')){
						$editIcon = $this->iconFactory->getIcon('actions-page-open', Icon::SIZE_SMALL).' ';
						$backendLink = \TYPO3\CMS\Backend\Utility\BackendUtility::getModuleUrl(
							'record_edit', [
								'columnsOnly' => $k2,
								'edit['.$table.']['.$v['uid'].']' => 'edit',
								'returnUrl' => \TYPO3\CMS\Core\Utility\GeneralUtility::getIndpEnv('REQUEST_URI')
							]
						);
						if(BaseUtility::_POSTuc('form3Apply','form3TdRealvalue',0) == 1){
							$contentTd[$v['uid']][$k2] = '<td style="vertical-align: top; color: blue;" nowrap>'.nl2br(htmlspecialchars($v[$k2])).'</td>';
						} else {
							$contentTd[$v['uid']][$k2] = $AirTableFieldClass::listControllerHtmlTd($this, $table, $k2, $v2, $v, $backendLink, $editIcon);
						}
					} else {
						$contentTd[$v['uid']][$k2] = '<td style="vertical-align: top;" nowrap><span style="color: red;">'.nl2br(htmlspecialchars($v[$k2])).'</span></td>';
					}
				}
			}
			// $content .= '<td style="'.LAST_COLUMN.'"><!--для растягивания колонок когда их всего 2 шт.--></td>';
			// $content .= '</tr>';
		}
		
		// Если пусто
		if(count($rows) == 0){
			$icon = $this->iconFactory->getIcon('apps-pagetree-page-default', Icon::SIZE_DEFAULT, 'overlay-missing');
			$content .= '<tr class="active">';
			$content .= $contentTh_0;
			$content .= implode('',$contentTh);
			$content .= '</tr>';
			$content .= '<tr>';
			$content .= '<td style="border-top: none;" colspan="'.($jCount+10).'" align="left">
							<div style="margin: 50px;">
								<h3>'.$icon.' Записи не найдены!</h3>
							</div>
						</td>';
			$content .= '</tr>';
			
		// Вертикальный вывод
		} elseif(BaseUtility::_POSTuc('form3Apply','form3RowsRender',0)==1) {
			$content .= '<tr class="active">';
			$content .= $contentTh_0;
			$foreachCounter = 0;
			$th_duplicate = BaseUtility::_POSTuc('form3Apply','form3ThDuplicate',0);
			foreach($contentTd as $kTd => $vTd){
				if($foreachCounter >= 1 && is_int($foreachCounter/$th_duplicate)){
					$content .= str_replace('disabled_ZZZ','disabled',$contentTh_0);
				}
				$content .= str_replace('###UID###',$kTd,$contentTd_0);
				$foreachCounter++;
			}
			$content .= '</tr>';
			$columns = $GLOBALS['TCA'][$table]['columns'];
			foreach($columns as $k2 => $v2){
				// Оставляем только выбранные колонки
				if(!in_array($k2, BaseUtility::_POSTuc('form3Apply','form3Columns')) 
					&& BaseUtility::_POSTuc('form3Apply','form3Columns','all') != 'all'
					&& $k2 != 'uid' && $k2 != 'pid' && $k2 != 'RType'){
					continue;
				}
				$foreachCounter = 0;
				$content .= '<tr>';
				$content .= $contentTh[$k2];
				foreach($contentTd as $kTd => $vTd){
					foreach($vTd as $tdKey => $tdContent){
						if($tdKey != $k2){
							continue;
						}
						if($foreachCounter >= 1 && is_int($foreachCounter/$th_duplicate)){
							$content .= str_replace('disabled_ZZZ','disabled',$contentTh[$tdKey]);
						}
						$content .= $tdContent;
						$foreachCounter++;
					}
				}
				$content .= '</tr>';
			}
		
		// Горизонтальный вывод (по умолчанию)
		} else {
			$content .= '<tr class="active">';
			$content .= $contentTh_0;
			$content .= implode('',$contentTh);
			$content .= '</tr>';
			$foreachCounter = 0;
			$th_duplicate = BaseUtility::_POSTuc('form3Apply','form3ThDuplicate',0);
			foreach($contentTd as $kTd => $vTd){
				if($foreachCounter >= 1 && is_int($foreachCounter/$th_duplicate)){
					$content .= '<tr class="active">';
					$content .= str_replace('disabled_ZZZ','disabled',$contentTh_0);
					$content .= str_replace('disabled_ZZZ','disabled',implode('',$contentTh));
					$content .= '</tr>';
				}
				$content .= '<tr>';
				$content .= str_replace('###UID###',$kTd,$contentTd_0);
				$content .= implode('',$vTd);
				$content .= '</tr>';
				$foreachCounter++;
			}
		}
		
		// Нижняя часть ("thFooter")
		/*
		$content .= '<tr class="active">';
		$content .= '<th></th>';
		$content .= '<th></th>';
		
		$columns = [];
		$columns['uid']['label'] = 'ID';
		$columns['uid']['AirTableConfig'] = 'INPUT_NUMBER';
		if(@in_array("RType", $GLOBALS['TCA'][$table]['ctrl']['fieldsDefault'])){
			$columns['RType'] = $GLOBALS['TCA'][$table]['columns']['RType'];
			$columns['RType']['label'] = 'Тип записи';
			$columns['RType']['AirTableConfig'] = 'SWITCH';
		}
		$columns += $GLOBALS['TCA'][$table]['columns'];
		foreach($columns as $k => $v){
			if($k == 'uid'){ continue; }
			if($k != 'RType' && !in_array($k, $this->getParam($table,'tab3','columns','all')) && $this->getParam($table,'tab3','columns','all') != 'all'){
				continue;
			}
				
			$AirTableConfig = $v['AirTableConfig'];
			$class = 'ColumnLibraryConfiguration_'.$AirTableConfig;
			if(class_exists($class) && method_exists($class, 'thFooter')){
				$obj = new $class;
				$content .= $obj->thFooter($this, $table, $k, $v);
			} else {
				$content .= '<th></th>';
			}
		}
		$content .= '</tr>';
		*/
			
		return $content;
	}
	
    /**
     * Main action for administration
     */
    public function showAction()
    {
		// Параметры _GET
		$paramExt = BaseUtility::_GETuc('paramExt',$this->extNameDir);
		$paramSubDomain = BaseUtility::_GETuc('paramSubDomain','Root');
		$paramClass = BaseUtility::_GETuc('paramClass','');
	
		// Создаем модель
		$sqlBuilderSelect = $paramClass::getModel();
		$table = BaseUtility::getTableNameFromClass($paramClass);
		
		// 1 // select
		$columns = $GLOBALS['TCA'][$table]['columns'];
		foreach($columns as $k => $v){
		
			// Оставляем только выбранные колонки
			#if((in_array($k, BaseUtility::_POSTuc('form3Apply','tab3_columns','all')) && BaseUtility::_POSTuc('form3Apply','tab3_columns','all') != 'all') 
			#	|| ($k == 'uid' && BaseUtility::_POSTuc('form3Apply','tab3_columns','all') != 'all')){
			#		$sqlBuilderSelect = $sqlBuilderSelect->addSelect($table.'.'.$k);
			#}
			$AirTableFieldClass = $v['AirTable.Class'];
			if(method_exists($AirTableFieldClass, 'listControllerSqlBuilder')){
				$sqlBuilderSelect = $AirTableFieldClass::listControllerSqlBuilder($this, $sqlBuilderSelect, $table, $k, $v);
			}
		}
		
		// 3 // order
		$paramOrder = BaseUtility::_POSTuc('form2Apply','form2Order');
		foreach($paramOrder as $k => $v){
			if($v != null){
				$sqlBuilderSelect = $sqlBuilderSelect->orderBy($table.'.'.$k,$v);
			}
		}
		
		// 4 // limit
		$page = BaseUtility::_POSTuc('form0Apply','form0Page',1);
		$lim = BaseUtility::_POSTuc('form3Apply','form3Limit',30); // 30
		if($lim>=0){
			if($page == 1){
				$sqlBuilderSelect = $sqlBuilderSelect->limit($lim)->offset(0);
			} else {
				$sqlBuilderSelect = $sqlBuilderSelect->limit($lim)->offset($lim*($page-1));
			}
		}
		
		// Выполняем запрос
		$rows = $sqlBuilderSelect->get()->toArray();
		
		// Собираем контент
		$content = '
		<table class="table table-striped table-bordered table-hover" style="width: 100%;">
		<tbody>
			'.$this->rowContentShow($rows, $table).'
		</tbody>
		</table>';
		
		// H1-ссылка
		$getArgs = [
			'paramExt' => BaseUtility::_GETuc('paramExt',$this->extNameDir),
			'paramSubDomain' => BaseUtility::_GETuc('paramSubDomain','Root'),
			'paramClass' => BaseUtility::_GETuc('paramClass',''),
			#'idList' => $idList
		];
		$uriBuilder = $this->controllerContext->getUriBuilder();
		$url = $uriBuilder->setArguments($getArgs)->uriFor('list', [], 'List');
		$h1Content = "<a href='".$url."'>".BaseUtility::getClassAnnotationValueNew($paramClass,'AirTable\Parameter\Label')."</a>";
		
        $assignedValues = [
			'modelName' => $paramClass,
			'h1Content' => $h1Content,
			'recordContent' => $content
        ];
        $this->view->assignMultiple($assignedValues);
	}
	
	public function rowContentShow($rows, $table=''){
		// Собираем контент
		$content = '';
		$jCount = 1;
		
		// Выводим строки
		foreach($rows as $k => $v){
			
			// Если даннаый ID не выбран, пропускаем
			if(!in_array($v['uid'],explode(',',$GLOBALS['_GET']['idList']))){
				continue;
			}
		
			$content .= '<tr>';
			$content .= '<td colspan="2" align="center" style="background: #ededed;"><h4><b>Запись №'.$v['uid'].'</b></h4></td>';
			$content .= '</tr>';
			$columns = $GLOBALS['TCA'][$table]['columns'];
			foreach($columns as $k2 => $v2){
				$content .= '<tr>';
			
				// Оставляем только выбранные колонки
				if(!in_array($k2, BaseUtility::_POSTuc('form3Apply','form3Columns')) 
					&& BaseUtility::_POSTuc('form3Apply','form3Columns','all') != 'all'){
					continue;
				}
				
				$AirTableFieldClass = $v2['AirTable.Class'];
				if(method_exists($AirTableFieldClass, 'listControllerHtmlTh')){
					// Название поля
					$debugStr = '';
					if($GLOBALS['TYPO3_CONF_VARS']['BE']['debug'] == true){
						$debugStr = '<br /><code style="color: #b4b2b2;">'.BaseUtility::getClassNameWithoutNamespace($v2['AirTable.Class']).'.'.$k2.'</code>';
					}
					$content .= '
					<td width="50%" align="right">'
						.$GLOBALS['LANG']->sL($v2['label'])
						.$debugStr.'
					</td>';
					$jCount++;
				} else {
					// Название поля
					$debugStr = '';
					if($GLOBALS['TYPO3_CONF_VARS']['BE']['debug'] == true){
						$debugStr = '<br /><code style="color: red;">'.$k2.'</code>';
					}
					$content .= '
					<td width="50%" align="right" style="color: red;">'
						.htmlspecialchars($GLOBALS['LANG']->sL($v2['label']))
						.$debugStr.'
					</td>';
					$jCount++;
				}
				
				// [RType] ->->-> Проверяем есть ли эта колонка в структуре?
				$fieldsInType = BaseUtility::getAllColumnsFromType($table, $v[$GLOBALS['TCA'][$table]['ctrl']['type']]);
				if(isset($GLOBALS['TCA'][$table]['ctrl']['type']) 
							&& isset($GLOBALS['TCA'][$table]['columns'][$GLOBALS['TCA'][$table]['ctrl']['type']]) 
								&& !in_array($k2,$fieldsInType) 
									&& $k2 != 'uid' 
										&& $k2 != 'pid'
											&& $table != 'sys_file' // Исключение (TCA оформленна не полностью)
				){
					$content .= '<td style="background: #ededed; color: #b4b2b2; vertical-align: top;" nowrap><i>Не существует</i></td>';
				} else {
					$AirTableFieldClass = $v2['AirTable.Class'];
					if(method_exists($AirTableFieldClass, 'listControllerHtmlTd')){
						$editIcon = $this->iconFactory->getIcon('actions-page-open', Icon::SIZE_SMALL).' ';
						$backendLink = \TYPO3\CMS\Backend\Utility\BackendUtility::getModuleUrl(
							'record_edit', [
								'columnsOnly' => $k2,
								'edit['.$table.']['.$v['uid'].']' => 'edit',
								'returnUrl' => \TYPO3\CMS\Core\Utility\GeneralUtility::getIndpEnv('REQUEST_URI')
							]
						);
						$content .= $AirTableFieldClass::listControllerHtmlTd($this, $table, $k2, $v2, $v, $backendLink, $editIcon);
					} else {
						$content .= '<td style="vertical-align: top;" nowrap><span style="color: red;">'.nl2br(htmlspecialchars($v[$k2])).'</span></td>';
					}
				}
				$content .= '</tr>';
			}
		}
		
		// Если пусто
		if(count($rows) == 0){
			$icon = $this->iconFactory->getIcon('apps-pagetree-page-default', Icon::SIZE_DEFAULT, 'overlay-missing');
			$content .= '<tr>';
			$content .= '<td style="border-top: none;" colspan="'.($jCount+10).'" align="left">
							<div style="margin: 50px;">
								<h3>'.$icon.' Записи не найдены!</h3>
							</div>
						</td>';
			$content .= '</tr>';
		} else {
			$content = $content;
		}
		
		// Нижняя часть ("thFooter")
		/*
		$content .= '<tr class="active">';
		$content .= '<th></th>';
		$content .= '<th></th>';
		
		$columns = [];
		$columns['uid']['label'] = 'ID';
		$columns['uid']['AirTableConfig'] = 'INPUT_NUMBER';
		if(@in_array("RType", $GLOBALS['TCA'][$table]['ctrl']['fieldsDefault'])){
			$columns['RType'] = $GLOBALS['TCA'][$table]['columns']['RType'];
			$columns['RType']['label'] = 'Тип записи';
			$columns['RType']['AirTableConfig'] = 'SWITCH';
		}
		$columns += $GLOBALS['TCA'][$table]['columns'];
		foreach($columns as $k => $v){
			if($k == 'uid'){ continue; }
			if($k != 'RType' && !in_array($k, $this->getParam($table,'tab3','columns','all')) && $this->getParam($table,'tab3','columns','all') != 'all'){
				continue;
			}
				
			$AirTableConfig = $v['AirTableConfig'];
			$class = 'ColumnLibraryConfiguration_'.$AirTableConfig;
			if(class_exists($class) && method_exists($class, 'thFooter')){
				$obj = new $class;
				$content .= $obj->thFooter($this, $table, $k, $v);
			} else {
				$content .= '<th></th>';
			}
		}
		$content .= '</tr>';
		*/
			
		return $content;
	}
	
    /**
     * This method is used throughout the TYPO3 Backend to show icons for a DB record
     *
     * @param string $table The TCA table name
     * @param array $row The DB record of the TCA table
     * @param string $size "large" "small" or "default", see the constants of the Icon class
     * @return Icon
     */
    public function getIconForRecord($table, array $row, $size = Icon::SIZE_DEFAULT)
    {
        $iconIdentifier = $this->iconFactory->mapRecordTypeToIconIdentifier($table, $row);
        return $this->iconFactory->getIcon($iconIdentifier, $size, 'overlay-deleted');
    }

}