<?php
namespace Litovchenko\AirTable\Field;
use Litovchenko\AirTable\Utility\BaseUtility;

/**
 * @AirTable:<Field>
 * @AirTable\Parameter\Label:<Базовый класс для поля>
 */
class BaseField
{
	const EXPORT = true;
	
	const IMPORT = true;
	
	const TABDEFAULT = 7; // props
	
    /**
     * @return array
     */
    public static function databaseDefinitions($model, $table, $field)
    {
        return [$table=>[
			$field => 'varchar(255) DEFAULT \'\' NOT NULL'
		]];
    }
	
    /**
     * @return string
     */
    public static function parameterClassLabel($class, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @return string
     */
    public static function parameterClassDescription($class, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @return string
     */
    public static function parameterLabel($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }

    /**
     * @return string
     */
    public static function parameterDescription($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @return string
     */
    public static function parameterReadOnly($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @return string
     */
    public static function parameterRequired($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @return string
     */
    public static function parameterDoNotCheck($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @return string
     */
    public static function parameterDoNotSqlAnalyze($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @return string
     */
    public static function parameterDefault($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @return array
     */
    public static function parameterPosition($class, $property, $keyAnnotation, $value)
    {
		$keyRealName = end(explode('\\',$keyAnnotation));
        return [$keyRealName => $value];
    }
	
    /**
     * @return string
     */
    public static function parameterShow($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @modify array $GLOBALS['TCA']
     */
    public static function buildConfiguration($model, $table, $field)
    {
		$databaseDefinitionsAr = get_called_class()::databaseDefinitions($model,$table,$field);
		$label = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\Label');
		if($label == ''){
			$label = '[Название колонки не задано]';
		}
		
		$GLOBALS['TCA'][$table]['columns'][$field]['AirTable.Class'] = get_called_class();
		$GLOBALS['TCA'][$table]['columns'][$field]['AirTable.MySql'] = $databaseDefinitionsAr[$table][$field];
		$GLOBALS['TCA'][$table]['columns'][$field]['label'] = $label;
		//$GLOBALS['TCA'][$table]['columns'][$field]['label'] .= ' '.$databaseDefinitionsAr[$table][$field];
		$GLOBALS['TCA'][$table]['columns'][$field]['label'] .= ' ';
		
		if(BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\LiveSearch') == 1){
			$GLOBALS['TCA'][$table]['ctrl']['searchFields'] .= ','.$field;
		}
		if(BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\Description') != ''){
			$GLOBALS['TCA'][$table]['columns'][$field]['description'] = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\Description');
		}
		if(BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\Required') == 1){
			$GLOBALS['TCA'][$table]['columns'][$field]['config']['eval'] = 'required';
		}
		if(BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\ReadOnly') == 1){
			$GLOBALS['TCA'][$table]['columns'][$field]['config']['readOnly'] = 1;
		}
		if(BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\Placeholder') != ''){
			$GLOBALS['TCA'][$table]['columns'][$field]['config']['placeholder'] = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\Placeholder');
		}
		if(BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\Default') != ''){
			$GLOBALS['TCA'][$table]['columns'][$field]['config']['default'] = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\Default');
		}
	}
	
    /**
     * @return array errors
     */
    public static function buildConfigurationCheck($model, $table, $field)
    {
		// Список ошибок
		$arError = [];
		if(strlen($field)>48) {
			$arError['strlen_48'] = '<li>Длинна название поля превышает 48 символов <code>'.$field.'</code></li>';
		}
		
		// Проверка позиции
		$annotationPosition = BaseUtility::getClassFieldAnnotationValueNew($model, $field, 'AirTable\Field\Parameter\Position');
		$userRTypes = $model::baseRTypes();
		$userTabs = $model::baseTabs();
		
		// Проверяем при параметре "RType"
		if(BaseUtility::hasClassTrait($model,'Litovchenko\AirTable\Field\Traits\RType') == true && !is_array($annotationPosition)){
			$arError['position_rtype'] = '<li>Поле содержит ошибки в настройках параметра 
			<code>"@AirTable\Field\Parameter\Position\\'.htmlspecialchars('<struct>:<tab,num>').'"</code>.
			Параметр не определен при включенном трейте <code>"RType"</code>.</li>';
			
		}elseif(is_array($annotationPosition)){
			foreach($annotationPosition as $kA => $vA){
				$tab = current(explode(',',$vA));
				$num = end(explode(',',$vA));
				//if($kA == '*'){
				//	continue;
				//} else {
					if($kA != '*' && !isset($userRTypes[$kA])){
						$arError['position_struct_'.$kA] = '<li>Поле содержит ошибки в настройках параметра 
						<code>"@AirTable\Field\Parameter\Position\\'.htmlspecialchars('<struct>:<tab,num>').'"</code>.
						Указанная структура <code>"'.$kA.'"</code> <!--(тип записи)--> не определена в модели!</li>';
					}
					if(!isset($userTabs[$tab])){
						$arError['position_tab_'.$tab] = '<li>Поле содержит ошибки в настройках параметра 
						<code>"@AirTable\Field\Parameter\Position\\'.htmlspecialchars('<struct>:<tab,num>').'"</code>.
						Указанная вкладка <code>"'.$tab.'"</code> <!--(таб)--> не определена в модели!</li>';
					}
				//}
			}
		}
		
		return $arError;
	}
	
	// listController func
	public static function listControllerSqlBuilder(&$obj, &$q, $table, $field, $config){
		return $q;
	}
	
	// listController func
	public static function listControllerSqlBuilderOrder(&$obj, $table, $field, $config){
		return true;
	}
	
	// listController func
	public static function listControllerHtmlFilter(&$obj, $table, $field, $config, &$uriBuilder){
		return '<input name="" class="form-control btn-sm" value="" disabled>';
	}
	
	// listController func
	public static function listControllerHtmlTh(&$obj, $table, $field, $config){
		return '<th>TH.'.$field.'</th>';
	}
	
	// listController func
	public static function listControllerHtmlTd(&$obj, $table, $field, $config, $row, &$uriBuilder){
		return '<td>TD.'.$field.'</td>';
	}
	
	// exportController
	public static function exportControllerRow(&$rowData, $class, $table, $field, $config, $row){
		$rowData[$field] = $row[$field];
	}
	
	// importController
	public static function importControllerRow(&$rowData, $class, $table, $field, $config, $row){
		$rowData[$field] = $row[$field];
	}
	
	// Вывод отладочной информации для поля
	public static function fieldErrorDisplay($configuration)
    {
        $infoField = '<strong>Поле содержит ошибки в конфигурации. ';
        $infoField .= 'Пожалуйста, исправте следующие параметры конфигурации для данного поля:</strong><br />';
        $infoField .= '<ul>'.implode('',$configuration['parameters']['message']).'</ul>';
        return '<div class="alert alert-warning">'.$infoField.'</div>';
    }
}
