<?php
namespace Litovchenko\AirTable\Field;
use Litovchenko\AirTable\Field\BaseField;
use Litovchenko\AirTable\Utility\BaseUtility;

/**
 * @AirTable:<Field>
 * @AirTable\Parameter\Label:<Базовый класс для поля связи между таблицами>
 */
class BaseFieldRelelation extends BaseField
{
	const EXPORT = true;
	
	const IMPORT = false;
	
	const TABDEFAULT = 8; // rels
	
    /**
     * @return string
     */
    public static function databaseDefinitions($model, $table, $field)
    {
        return [];
    }
	
    /**
     * @return string
     */
    public static function parameterForeignModel($class, $property, $keyAnnotation, $value)
    {
		if($value == 'current'){
			return $class;
		} elseif($value == 'category'){
			return $class.'Category';
		} else {
			return $value;
		}
    }
	
    /**
     * @return string
     */
    public static function parameterForeignKey($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @return string
     */
    public static function parameterRenderType($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @return string
     */
    public static function parameterParentKey($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @modify array $GLOBALS['TCA']
     */
    public static function buildConfiguration($model, $table, $field)
    {
		parent::buildConfiguration($model, $table, $field);
	}
	
    /**
     * @return array
     */
    public static function buildConfigurationError($model, $table, $field, $arError = [])
    {
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['type'] = 'user';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['userFunc'] = 'Litovchenko\AirTable\Field\BaseField->fieldErrorDisplay';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['parameters']['message'] = $arError;
	}
	
    /**
     * @model
     * @field
     */
    public static function isDoubleRelation($model, $field)
    {
		$fM = self::getForeignModelName($model, $field);
		$fF = self::getForeignFieldName($model, $field);
		$annotationFieldForeign = BaseUtility::getClassFieldAnnotationValueNew($fM,$fF,'AirTable\Field');
		if($annotationFieldForeign != '' && $model != $fM){
			return true;
		}
		return false;
	}
	
    /**
     * @model
     * @field
     */
    public static function getForeignModelName($model, $field)
    {
		return BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\ForeignModel');
	}
	
    /**
     * @model
     * @field
     */
    public static function getForeignTableName($model, $field)
    {
		return BaseUtility::getTableNameFromClass(self::getForeignModelName($model,$field)); // table
	}
	
    /**
     * @model
     * @field
     */
    public static function getForeignFieldName($model, $field)
    {
		return BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\ForeignKey'); // field
	}
	
	// Проверка полей связи
	public static function buildConfigurationCheck($model, $table, $field)
	{
		// Некоторые поля пропускаем!
		// Сделал "@AirTable\Field\Parameter\DoNotCheck:<1>"
		#if($field == 'be_users_id'){
		#	return [];
		#}
		#if($field == 'category_list'){
		#	return [];
		#}
		#if($field == 'category_id'){
		#	return [];
		#}
		#if($field == 'parent_id'){
		#	return [];
		#}
		#if($field == 'tt_content_list'){
		#	return [];
		#}

		// Список ошибок
		$arError = parent::buildConfigurationCheck($model, $table, $field);
		
		// Внешние названия
		$_foreignModelName = self::getForeignModelName($model, $field); // foreign model
		$_foreignTableName = self::getForeignTableName($model, $field); // foreign table
		$_foreignFieldName = self::getForeignFieldName($model, $field); // foreign field
		
		// Класс колонки
		$annotationField = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field');
		$fieldClass = 'Litovchenko\AirTable\Field\\'.$annotationField;
		
		// Далее
		if(!class_exists($_foreignModelName)){
			$arError[0] = '<li>Обязательный параметр <code>"@AirTable\Field\Parameter\ForeignModel:"</code> 
			содержит связь к несуществующей модели <code>"'.$_foreignModelName.'"</code></li>';
		
		} else {
		
			#$namespace_1 = explode('\\',$_foreignModelName);
			#unset($namespace_1[count($namespace_1)-1]);
			#$namespace_1 = implode('\\',$namespace_1);
			
			#$namespace_2 = explode('\\',$model);
			#unset($namespace_2[count($namespace_2)-1]);
			#$namespace_2 = implode('\\',$namespace_2);
			
			#print 'BaseFieldRelelation';
			#print 'Сделать также сопоставление по расширению, по папке';
			#exit();
			
			$compare_1 = md5(BaseUtility::getExtNameFromClassPath($_foreignModelName).
							BaseUtility::getSubDomainNameFromClassPath($_foreignModelName));
			
			$compare_2 = md5(BaseUtility::getExtNameFromClassPath($model).
							BaseUtility::getSubDomainNameFromClassPath($model));
			
			// Если таблицы существуют в рамках одного расширения и одной модели,
			// принудительно ставим необходимым поставить сокрашенные названия полей
			if($compare_1 == $compare_2){
				$_foreignTableNameShort = BaseUtility::getTableNameFromClass($_foreignModelName,1);
				$tableShort = BaseUtility::getTableNameFromClass($model,1);
				
				// Префиксы
				$prefix_1 = explode('_'.$_foreignTableNameShort.'_',$field);
				if(count($prefix_1)>1){
					$prefix_1 = $prefix_1[0].'_';
				} else {
					$prefix_1 = '';
				}
				
				if($field != $prefix_1.$_foreignTableNameShort.$fieldClass::REQPOSTFIXCURRENTFIELD){
					$arError[1] = '<li>Название текущего поля не соответствует рекомендованному названию (сокращенному):
					<code>"'.$prefix_1.$_foreignTableNameShort.$fieldClass::REQPOSTFIXCURRENTFIELD.'"</code></li>';
				}
				if($_foreignFieldName != $prefix_1.$tableShort.$fieldClass::REQPOSTFIXFOREIGNFIELD){
					$arError[2] = '<li>Значение обязательного параметра <code>"@AirTable\Field\Parameter\ForeignKey:"</code>
					не соответствует рекомендованному названию (сокращенному) <code>"'.$prefix_1.$tableShort.$fieldClass::REQPOSTFIXFOREIGNFIELD.'"</code></li>';
				}
			} else {
			
				// Префиксы
				$prefix_1 = explode('_'.$_foreignTableName.'_',$field);
				if(count($prefix_1)>1){
					$prefix_1 = $prefix_1[0].'_';
				} else {
					$prefix_1 = '';
				}
				
				if($field != $prefix_1.$_foreignTableName.$fieldClass::REQPOSTFIXCURRENTFIELD){
					$arError[1] = '<li>Название текущего поля не соответствует рекомендованному названию (полному):
					<code>"'.$prefix_1.$_foreignTableName.$fieldClass::REQPOSTFIXCURRENTFIELD.'"</code></li>';
				}
				if($_foreignFieldName != $prefix_1.$table.$fieldClass::REQPOSTFIXFOREIGNFIELD){
					$arError[2] = '<li>Значение обязательного параметра <code>"@AirTable\Field\Parameter\ForeignKey:"</code>
					не соответствует рекомендованному названию (полному) <code>'.$prefix_1.$table.$fieldClass::REQPOSTFIXFOREIGNFIELD.'"</code></li>';
				}
			}
			

			// Если находим префикс, производим их проверку
			#if($searchPrefixCurrent != '' && substr($searchPrefixCurrent,-1) != '_'){
				#$arError[3] = '<li>Текущее поле '.$field.' содержит префикс без символа <code>"_"</code></li>';
			#
				// Проверяем с реверсом
				// Проверяем правильность создания двухсторонней связи
				if(count($arError) == 0){
					$annotationFieldForeign = BaseUtility::getClassFieldAnnotationValueNew($_foreignModelName,$_foreignFieldName,'AirTable\Field');
					if($annotationFieldForeign != '' && $model != $_foreignModelName){
						// Не реверс
						// Реверс
						// Одинаковые
						#if(!preg_match('/_Revers$/is',$annotationField)){
						#	if($annotationField.'_Revers' != $annotationFieldForeign){
						#		$arError[3] = '<li>Текущее ошибка 1 '.$field.'  <code>"SS1"</code></li>';
						#	}
						#}
						if($annotationField != $annotationFieldForeign.'_Revers' && $annotationField.'_Revers' != $annotationFieldForeign){
							$arError[3] = '<li>Ошибка в установлении двухсторонней связи. Проверте правильность 
							указанных конфигурация для соответствующих колонок. Текущие конфигурации колонок 
							<code>"'.$annotationField.'</code> - <code>'.$model.'"</code> и 
							<code>"'.$annotationFieldForeign.'</code> - <code>'.$_foreignModelName.'"</code></li>';
						}
						#if(preg_match('/_Revers$/is',$annotationField)){
						#	if($annotationField != $annotationFieldForeign.'_Revers'){
						#		$arError[3] = '<li>Текущее ошибка 2 '.$field.'  <code>"SS2"</code></li>';
						#	}
						#}
					}
				}
			#}
		}
		
		#print $this->field."<br />";
		#print $currentPostfix."<br />";
		#print $foreignField."<br />";
		#print $foreignPostfix."<br />";
		
		return $arError;
	}
	
	// listController func
	public static function listControllerSqlBuilder(&$obj, &$q, $table, $field, $config)
	{
		// $q->addSelect($table.'.'.$field);
		$paramFilter_select = BaseUtility::_POSTuc('form1Apply','form1Field_'.$field,'');
		$paramFilter_input = BaseUtility::_POSTuc('form1Apply','form1FieldValue_'.$field,'');

		#switch($config['AirTable.Class']){
		#	case 'Litovchenko\AirTable\Field\Rel_1to1':
		#	case 'Litovchenko\AirTable\Field\Rel_1toM':
		#	case 'Litovchenko\AirTable\Field\Rel_Mto1':
		#	case 'Litovchenko\AirTable\Field\Rel_MtoM':
		#	case 'Litovchenko\AirTable\Field\Rel_1to1_Revers':
		#	case 'Litovchenko\AirTable\Field\Rel_1toM_Revers':
		#	case 'Litovchenko\AirTable\Field\Rel_Mto1_Revers':
		#	case 'Litovchenko\AirTable\Field\Rel_MtoM_Revers':
		#	case 'REL_Morph': // Todo
				switch($paramFilter_select){
					default:
						// 
					break;
					case 'R0': 
					case 'R1': 
					case 'R2': 
					case 'R3': 
					case 'R4': 
					case 'R5': 
					case 'R5+':
						$countReference = preg_replace('/^R/is','',$paramFilter_select);
						if($countReference == '5+'){
							$countReference = preg_replace('/\+$/is','',$countReference);
							$q = $q->has('getDefaultRelation_'.$field.'_Method', '>', intval($countReference));
						} else {
							$q = $q->has('getDefaultRelation_'.$field.'_Method', '=', intval($countReference));
						}
					break;
					case 'REL_INCLUDE_uid':
						switch($config['AirTable.Class']){
							//+
							default:
								$q = $q->whereHas('getDefaultRelation_'.$field.'_Method', function($q) use ($table, $paramFilter_input) {
									$q->whereIn('uid',  explode(',', $paramFilter_input));
								});
							break;
							//+
							case 'Litovchenko\AirTable\Field\Rel_MtoM':
								$q = $q->whereHas('getDefaultRelation_'.$field.'_Method', function($q) use ($table, $paramFilter_input) {
									$q->whereIn('uid_foreign',  explode(',', $paramFilter_input));
								});
							break;
							//+
							case 'Litovchenko\AirTable\Field\Rel_MtoM_Revers':
								$q = $q->whereHas('getDefaultRelation_'.$field.'_Method', function($q) use ($table, $paramFilter_input) {
									$q->whereIn('uid_local',  explode(',', $paramFilter_input));
								});
							break;
						}
					break;
					case 'REL_EXCLUDE_uid':
						switch($config['AirTable.Class']){
							//+
							default:
								$q = $q->whereDoesntHave('getDefaultRelation_'.$field.'_Method', function($q) use ($table, $paramFilter_input) {
									$q->whereIn('uid',  explode(',', $paramFilter_input));
								});
							break;
							//+
							case 'Litovchenko\AirTable\Field\Rel_MtoM':
								$q = $q->whereDoesntHave('getDefaultRelation_'.$field.'_Method', function($q) use ($table, $paramFilter_input) {
									$q->whereIn('uid_foreign',  explode(',', $paramFilter_input));
								});
							break;
							//+
							case 'Litovchenko\AirTable\Field\Rel_MtoM_Revers':
								$q = $q->whereDoesntHave('getDefaultRelation_'.$field.'_Method', function($q) use ($table, $paramFilter_input) {
									$q->whereIn('uid_local',  explode(',', $paramFilter_input));
								});
							break;
						}
					break;
				}
			#break;
		#}
		return $q;
	}
	
	// listController func
	public static function listControllerSqlBuilderWith(&$obj, &$q, $table, $field, $config)
	{
		return $q->with('getDefaultRelation_'.$field.'_Method'); 
	}
	
	// listController func
	public static function listControllerSqlBuilderOrder(&$obj, $table, $field, $config){
		switch($config['AirTable.Class']){
			case 'Litovchenko\AirTable\Field\Rel_1to1_Revers':
			case 'Litovchenko\AirTable\Field\Rel_1toM_Revers':
			case 'Litovchenko\AirTable\Field\Rel_Mto1':
				return true;
			break;
		}
		return false;
	}
	
	// listController func
	public static function listControllerHtmlFilter(&$obj, $table, $field, $config, &$uriBuilder){
		$paramFilter_select = BaseUtility::_POSTuc('form1Apply','form1Field_'.$field,'');
		$paramFilter_input = BaseUtility::_POSTuc('form1Apply','form1FieldValue_'.$field,'');

		/*
		Array
		(
			[route] => /AirTableTxairtabletabcontent/AirTableList0/
			[token] => eaf81a00a1b23613af6669b314391a90f3a0643b
			[paramExt] => air_table
			[paramSubDomain] => Examples6
			[paramClass] => Litovchenko\AirTable\Examples6\Model\ExampleDish
			[tx_airtable_airtabletxairtabletabcontent_airtablelist0] => Array
				(
					[action] => list
					[controller] => List
				)

		)
		*/
		
		// vars
		$class = $GLOBALS['TCA'][$table]['ctrl']['AirTable.Class'];
		$annotationRelationType = BaseUtility::getClassFieldAnnotationValueNew($class,$field,'AirTable\Field');
		$annotationForeignModel = BaseUtility::getClassFieldAnnotationValueNew($class,$field,'AirTable\Field\Parameter\ForeignModel');
		$foreignTable = BaseUtility::getTableNameFromClass($annotationForeignModel);
		
		// icon
		$typeicon_classes = $GLOBALS['TCA'][$foreignTable]['ctrl']['typeicon_classes']['default'];
		$icon = $obj->iconFactory->getIcon($typeicon_classes, \TYPO3\CMS\Core\Imaging\Icon::SIZE_SMALL);
		
		// url
		$getArgs = [
			'paramExt' => BaseUtility::getExtNameFromClassPath($annotationForeignModel),
			'paramSubDomain' => BaseUtility::getSubDomainNameFromClassPath($annotationForeignModel),
			'paramClass' => $annotationForeignModel,
			'recordSelection' => 1, // Сообщаем что это выбор записей
			'recordSelectionFieldname' => 'form1FieldValue_'.$field, // Сообщаем информацию о поле в которое вставлять записи
		];
		$url = $uriBuilder->setArguments($getArgs)->uriFor('list', [], 'List');
		
		$hideOption = '';
		switch($config['AirTable.Class']){
			case 'Litovchenko\AirTable\Field\Rel_1to1':
			case 'Litovchenko\AirTable\Field\Rel_1to1_Revers':
			case 'Litovchenko\AirTable\Field\Rel_1toM_Revers':
			case 'Litovchenko\AirTable\Field\Rel_Mto1':
				$hideOption = 'display: none;';
			break;
		}
		
		return '
			<table width="100%" class="" style="margin-bottom: 0;">
			<tr>
			<td width="30%">
				<select name="form1Field_'.$field.'" class="form-control btn-sm">
					<option disabled>'.$foreignTable.'</option>
					<option value="" '.($paramFilter_select == ''?'selected':'').'>Любое значение</option>
					<option value="REL_INCLUDE_uid" '.($paramFilter_select == 'REL_INCLUDE_uid'?'selected':'').'>Включая список ID (через ",")</option>
					<option value="REL_EXCLUDE_uid" '.($paramFilter_select == 'REL_EXCLUDE_uid'?'selected':'').'>Исключая список ID (через ",")</option>
					<option value="R0" '.($paramFilter_select == 'R0'?'selected':'').'>Кол-во: 0 связей</option>
					<option value="R1" '.($paramFilter_select == 'R1'?'selected':'').'>Кол-во: 1 связь</option>
					<option value="R2" '.($paramFilter_select == 'R2'?'selected':'').' style="'.$hideOption.'">Кол-во: 2 связи</option>
					<option value="R3" '.($paramFilter_select == 'R3'?'selected':'').' style="'.$hideOption.'">Кол-во: 3 связи</option>
					<option value="R4" '.($paramFilter_select == 'R4'?'selected':'').' style="'.$hideOption.'">Кол-во: 4 связи</option>
					<option value="R5" '.($paramFilter_select == 'R5'?'selected':'').' style="'.$hideOption.'">Кол-во: 5 связей</option>
					<option value="R5+" '.($paramFilter_select == 'R5+'?'selected':'').' style="'.$hideOption.'">Кол-во: более 5 связей</option>
					<optgroup label="Для категорий">
						<option value="TODO" '.($paramFilter_select == 'TODO'?'selected':'').' disabled>Стартовый ID, включая подкатегории (1 уровень)</option>
						<option value="TODO" '.($paramFilter_select == 'TODO'?'selected':'').' disabled>Стартовый ID, включая подкатегории (2 уровеня)</option>
						<option value="TODO" '.($paramFilter_select == 'TODO'?'selected':'').' disabled>Стартовый ID, включая подкатегории (3 уровеня)</option>
						<option value="TODO" '.($paramFilter_select == 'TODO'?'selected':'').' disabled>Стартовый ID, включая подкатегории (4 уровеня)</option>
						<option value="TODO" '.($paramFilter_select == 'TODO'?'selected':'').' disabled>Стартовый ID, включая подкатегории (без ограничений)</option>
					</optgroup>
				</select>
			</td>
			<td width="65%" style="padding-left: 5px;">
				<input name="form1FieldValue_'.$field.'" class="form-control btn-sm" value="'.$paramFilter_input.'" placeholder="Выберите значения">
			</td>
			<td width="20" style="padding-left: 5px;" nowrap>
				<a onclick="window.open(\''.$url.'\');" class="btn btn-default btn-sm" title="'.$foreignTable.'">
					'.$icon.'
				</a>
			</td>
			</tr>
			</table>
		';
	}
	
	// listController func
	public static function listControllerHtmlTh(&$obj, $table, $field, $config){
		return \Litovchenko\AirTable\Field\Input::listControllerHtmlTh($obj, $table, $field, $config);
	}
	
	// listController func
	public static function listControllerHtmlTd(&$obj, $table, $field, $config, $row, &$uriBuilder){
		$backendLink = \TYPO3\CMS\Backend\Utility\BackendUtility::getModuleUrl(
			'record_edit', [
				'columnsOnly' => $field,
				'edit['.$table.']['.$row['uid'].']' => 'edit',
				'returnUrl' => \TYPO3\CMS\Core\Utility\GeneralUtility::getIndpEnv('REQUEST_URI')
			]
		);

		$fieldInRow = 'get_default_relation_'.$field.'__method';
		$rowArray = [];
		if(isset($row[$fieldInRow][0])){
			if(!empty($row[$fieldInRow])){
				$rowArray = $row[$fieldInRow];
			}
		} else {
			if(!empty($row[$fieldInRow])){
				$rowArray[0] = $row[$fieldInRow];
			}
		}
		
		// vars
		$class = $GLOBALS['TCA'][$table]['ctrl']['AirTable.Class'];
		$annotationRelationType = BaseUtility::getClassFieldAnnotationValueNew($class,$field,'AirTable\Field');
		$annotationForeignModel = BaseUtility::getClassFieldAnnotationValueNew($class,$field,'AirTable\Field\Parameter\ForeignModel');
		$foreignTable = BaseUtility::getTableNameFromClass($annotationForeignModel);
		
		// tca
		$label = $GLOBALS['TCA'][$foreignTable]['ctrl']['label'];
		$deleted = $GLOBALS['TCA'][$foreignTable]['ctrl']['delete'];
		
		$j = 1;
		$content = '';
		foreach($rowArray as $k => $v){
			// editLink
			$editLink = '';
			if($j == 1){
				$editIcon = $obj->iconFactory->getIcon('actions-system-pagemodule-open', \TYPO3\CMS\Core\Imaging\Icon::SIZE_SMALL);
				$editLink = '<td width="29" nowrap style="vertical-align: top; padding:2px;" rowspan="'.count($rowArray).'">
					<a href="'.$backendLink.'" class="btn btn-default btn-sm" style="width: 25px;">
						'.$editIcon.'
					</a>
				</td>';
			}
			if(preg_match('/_Revers$/is',$config['AirTable.Class'])){
				// &#8601;(вхд.)
				$s = '&#8601; ';
			} else {
				// &#8599;(исхд.)
				$s = '&#8599; ';
			}
			if($v[$deleted] == 1) {
				$icon = $obj->getIconForRecord($foreignTable, $v, \TYPO3\CMS\Core\Imaging\Icon::SIZE_SMALL);
				$content .='
				<tr>
					'.$editLink.'
					<td width="29" nowrap style="vertical-align: top; padding:0; '.HIDE_CHECKBOX.'">
					<div style="position: relative; width: 29px;">
					<label class="btn btn-default btn-sm" style="position: absolute; top: 2px; left: 2px; margin: 0; padding: 0 5px 0 5px; cursor: pointer;">
						<input type="checkbox" class="btn btn-default btn-sm" name="foo_'.$field.'" style="margin-top: 0px;">
					</label>
					</div>
					</td>
					<!--
					<td width="20" nowrap style="'.HIDE_CHECKBOX_TD_NEXT.'">
					<a href="?" class="btn btn-default btn-sm" style="padding-left: 7px; padding-right: 7px;">
						<i class="fa fa-reorder" aria-hidden="true"></i>
					</a>
					</td>
					-->
					<!--<td width="20" nowrap>'.$config['AirTable.Class'].'</td>-->
					<!--<td width="20" nowrap>'.$j.'/'.count($rowArray).'</td>-->
					<td width="20" nowrap><nobr><span style="color_ZZZ: #d45978">'.$icon.' '.$v['uid'].'</span></nobr></td>
					<td nowrap>'.$s.' '.$v[$label].'</td>
				</tr>';
			} else {
				$icon = $obj->iconFactory->getIconForRecord($foreignTable, $v, \TYPO3\CMS\Core\Imaging\Icon::SIZE_SMALL);
				$content .='
				<tr>
					'.$editLink.'
					<td width="29" nowrap style="vertical-align: top; padding:0; '.HIDE_CHECKBOX.'">
					<div style="position: relative; width: 29px;">
					<label class="btn btn-default btn-sm" style="position: absolute; top: 2px; left: 2px; margin: 0; padding: 0 5px 0 5px; cursor: pointer;">
						<input type="checkbox" class="btn btn-default btn-sm" name="foo_'.$field.'" style="margin-top: 0px;">
					</label>
					</div>
					</td>
					<!--
					<td width="20" nowrap style="'.HIDE_CHECKBOX_TD_NEXT.'">
					<a href="?" class="btn btn-default btn-sm" style="padding-left: 7px; padding-right: 7px;">
						<i class="fa fa-reorder" aria-hidden="true"></i>
					</a>
					</td>
					-->
					<!--<td width="20" nowrap>'.$config['AirTable.Class'].'</td>-->
					<!--<td width="20" nowrap>'.$j.'/'.count($rowArray).'</td>-->
					<td width="20" nowrap><nobr><span style="color_ZZZ: #d45978">'.$icon.' '.$v['uid'].'</span></nobr></td>
					<td nowrap>'.$s.' '.$v[$label].'</td>
				</tr>';
			}
			$j++;
		}
		if($content == NULL){
			return '<td style="vertical-align: top; padding: 0;">
				<table class="t3tablerel table table-striped table-bordered table-hover" style="margin-top: -1px; margin-bottom: -1px; border: 0;">
				<tr>
					<td width="29" nowrap style="vertical-align: top; padding:0; '.HIDE_CHECKBOX.'">
					<div style="position: relative; width: 29px;">
					<label class="btn btn-default btn-sm" style="position: absolute; top: 2px; left: 2px; margin: 0; padding: 0 5px 0 5px; cursor: pointer;">
						<input type="checkbox" class="btn btn-default btn-sm" name="foo_'.$field.'" style="margin-top: 0px;">
					</label>
					</div>
					</td>
					<td style="'.HIDE_CHECKBOX_TD_NEXT.'" nowrap><a href="'.$backendLink.'"><code style="'.STYLE_EMPTY_FIELD.'">Установить связь</code></a></td>
				</tr>
				</table>
			</td>';
		}else{
			if(preg_match('/_Revers$/is',$config['AirTable.Class'])){
				$color = 'orange';
			} else {
				$color = ''; // green
			}
			return '<td style="vertical-align: top; padding: 0;">
				<table class="t3tablerel table table-striped table-bordered table-hover" style="margin-top: -1px; margin-bottom: -1px; border: 0; color: '.$color.'">
				<!--
				<thead>
				<tr>
					<td><b>ID</b></td>
					<td><b>Ссылка</b></td>
				</tr>
				</thead>
				-->
				'.$content.'
				</table>
			</td>';
		}
	}
	
	// exportController
	public static function exportControllerRow(&$rowData, $class, $table, $field, $config, $row){
		$annotationForeignModel = BaseUtility::getClassFieldAnnotationValueNew($class,$field,'AirTable\Field\Parameter\ForeignModel');
		$foreignTable = BaseUtility::getTableNameFromClass($annotationForeignModel);
		$label = $GLOBALS['TCA'][$foreignTable]['ctrl']['label'];
		$disabled = $GLOBALS['TCA'][$foreignTable]['ctrl']['enablecolumns']['disabled'];
		$deleted = $GLOBALS['TCA'][$foreignTable]['ctrl']['delete'];
		$sorting = $GLOBALS['TCA'][$foreignTable]['ctrl']['sortby'];

		$fieldInRow = 'get_default_relation_'.$field.'__method';
		$rowArray = [];
		if(isset($row[$fieldInRow][0])){
			if(!empty($row[$fieldInRow])){
				$rowArray = $row[$fieldInRow];
			}
		} else {
			if(!empty($row[$fieldInRow])){
				$rowArray[0] = $row[$fieldInRow];
			}
		}
		
		foreach($rowArray as $k => $v){
			$rowData[$field][] = 'A'.$v['uid'];
			if($label != ''){
				$rowData[$field.'.'.$label][] = $v[$label];
			}
			#if($disabled != ''){
			#	$rowData[$field.'.'.$disabled][] = $v[$disabled];
			#}
			if($deleted != ''){
				$rowData[$field.'.'.$deleted][] = $v[$deleted];
			}
			#if($sorting != ''){
			#	$rowData[$field.'.'.$sorting][] = $v[$sorting];
			#}
		}
		
		$rowData[$field] = implode("\n",$rowData[$field]);
		if($label != ''){
			$rowData[$field.'.'.$label] = implode("\n",$rowData[$field.'.'.$label]);
		}
		#if($disabled != ''){
		#	$rowData[$field.'.'.$disabled] = implode("\n",$rowData[$field.'.'.$disabled]);
		#}
		if($deleted != ''){
			$rowData[$field.'.'.$deleted] = implode("\n",$rowData[$field.'.'.$deleted]);
		}
		#if($sorting != ''){
		#	$rowData[$field.'.'.$sorting] = implode("\n",$rowData[$field.'.'.$sorting]);
		#}
	}
	
	// importController
	public static function importControllerRow(&$rowData, $class, $table, $field, $config, $row){
		
		#default: 
		#	$modelName::linkCreate($recordId, $kData, $vArex[0]); // true || false // (id-текущей, поле связи, id-связи)
		#break;
		#case 'D': 
		#	$modelName::linkRemove($recordId, $kData, trim($vArex,'D')); // true || false // (id-текущей, поле связи, id-связи) 
		#break;
		#case 'T':  
		#	$modelName::linkRemove($recordId, $kData); // true || false // (id-текущей, поле связи) 
		#break;
		
		if(!empty($row[$field])){
			#$idListAttach = [];
			#$idListDetach = [];
			$fieldInRow = $field.'_Method';
			$relList = explode(chr(10),$row[$field]);
			foreach($relList as $k => $v){
				if(preg_match('/^T(.*)/is',$v,$match)){
					$class::linkRemove($field.'_Method',$row['uid']);
				
				}elseif(preg_match('/^A(.*)/is',$v,$match)){
					$class::linkCreate($field.'_Method',$row['uid'],$match[1]);
				
				}elseif(preg_match('/^D(.*)/is',$v,$match)){
					$class::linkRemove($field.'_Method',$row['uid'],$match[1]);
				}
			}
		}
		// $rowData[$field] = 1;
		// $rowData[$field] = '';
	}
	
	// Автоматизированная выборка связей 
	// public function linkApproach($field){}
	
	// Создание связей
	// public function linkCreate($field, $idTwo = null, $data = []){}
	
	// Удаление связей
	// public function linkRemove($field, $idTwo = null, $data = []){}
	
}