<?php
namespace Litovchenko\AirTable\Field;
use Litovchenko\AirTable\Field\BaseField;
use Litovchenko\AirTable\Utility\BaseUtility;

/**
 * @AirTable:<Field>
 * @AirTable\Parameter\Label:<Строка (до 255 символов)>
 */
class Input extends BaseField
{
    /**
     * @return string
     */
    public static function databaseDefinitions($model, $table, $field)
    {
        return [$table=>[
			$field => 'varchar(255) DEFAULT \'\' NOT NULL'
		]];
    }
	
    /**
     * @return string
     */
    public static function parameterPlaceholder($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @return string
     */
    public static function parameterLiveSearch($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @return string
     */
    public static function parameterMax($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @return string
     */
    public static function parameterSize($class, $property, $keyAnnotation, $value)
    {
        return $value;
    }
	
    /**
     * @modify array
     */
    public static function buildConfiguration($model, $table, $field)
    {
		parent::buildConfiguration($model,$table,$field);
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['type'] = 'input';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['eval'] .= ',trim';
		
		// Значение по умолчанию
		$default = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\Default');
		if(!empty($default)){
			$GLOBALS['TCA'][$table]['columns'][$field]['config']['default'] = $default;
		}
		
		// Max
		$max = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\Max');
		if(!empty($max)){
			$GLOBALS['TCA'][$table]['columns'][$field]['config']['max'] = $max;
		}
		
		// Size
		$size = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\Size');
		if(!empty($size)){
			$GLOBALS['TCA'][$table]['columns'][$field]['config']['size'] = $size;
		}
	}
	
	// listController func
	public static function listControllerSqlBuilder(&$obj, &$q, $table, $field, $config){
		// $q->addSelect($table.'.'.$field);
		$paramFilter_select = BaseUtility::_POSTuc('form1Apply','form1Field_'.$field,'');
		$paramFilter_input = BaseUtility::_POSTuc('form1Apply','form1FieldValue_'.$field,'');
		$paramFilter_input = trim($paramFilter_input);
		switch($paramFilter_select){
			// case 'any': break;
			case 'EMPTY': 		
				return $q->where($table.'.'.$field,'=',''); 
			break;
			case 'NOT EMPTY': 	
				return $q->where($table.'.'.$field,'<>',''); 
			break;
			case '=': 			
				return $q->where($table.'.'.$field,'=',$paramFilter_input); 
			break;
			case '!=': 			
				return $q->where($table.'.'.$field,'<>',$paramFilter_input); 
			break;
			case 'LIKE': 		
				return $q->where($table.'.'.$field,'like',$paramFilter_input); 
			break;
			case 'NOT LIKE':	
				return $q->where($table.'.'.$field,'not like',$paramFilter_input); 
			break;
			case 'REGEXP': 		
			return $q->where($table.'.'.$field,'regexp',$paramFilter_input); 
			break;
			case 'NOT REGEXP': 	
				return $q->where($table.'.'.$field,'not regexp',$paramFilter_input); 
			break;
		}
		return $q;
	}
	
	// listController func
	public static function listControllerSqlBuilderOrder(){
		return true;
	}
	
	// listController func
	public static function listControllerHtmlFilter(&$obj, $table, $field, $config, &$uriBuilder){
		$paramFilter_select = BaseUtility::_POSTuc('form1Apply','form1Field_'.$field,'');
		$paramFilter_input = BaseUtility::_POSTuc('form1Apply','form1FieldValue_'.$field,'');
		$paramFilter_input = htmlspecialchars($paramFilter_input);
		$paramFilter_input = stripslashes($paramFilter_input);
		return '
			<table width="100%" class="" style="margin-bottom: 0;">
			<tr>
			<td width="30%">
				<select name="form1Field_'.$field.'" class="form-control btn-sm">
					<option value="" '.($paramFilter_select == ''?'selected':'').'>Любое значение</option>
					<option value="EMPTY" '.($paramFilter_select == 'EMPTY'?'selected':'').'>Не заполнено (пустое)</option>
					<option value="NOT EMPTY" '.($paramFilter_select == 'NOT EMPTY'?'selected':'').'>Заполнено (не пустое)</option>
					<option value="=" '.($paramFilter_select == '='?'selected':'').'>=</option>
					<option value="!=" '.($paramFilter_select == '!='?'selected':'').'>!=</option>
					<option value="LIKE" '.($paramFilter_select == 'LIKE'?'selected':'').'>LIKE</option>
					<option value="NOT LIKE" '.($paramFilter_select == 'NOT LIKE'?'selected':'').'>NOT LIKE</option>
					<option value="REGEXP" '.($paramFilter_select == 'REGEXP'?'selected':'').'>REGEXP</option>
					<option value="NOT REGEXP" '.($paramFilter_select == 'NOT REGEXP'?'selected':'').'>NOT REGEXP</option>
				</select>
			</td>
			<td width="70%" style="padding-left: 5px;">
				<input name="form1FieldValue_'.$field.'" class="form-control btn-sm" value="'.$paramFilter_input.'" placeholder="Введите значение">
			</td>
			</tr>
			</table>
		';
	}
	
	// listController func
	public static function listControllerHtmlTh(&$obj, $table, $field, $config){
		$debugStr = '';
		if($GLOBALS['TYPO3_CONF_VARS']['BE']['debug'] == true){
			$debugStr = '<code style="color: #b4b2b2;">'.BaseUtility::getClassNameWithoutNamespace($config['AirTable.Class']).'.'.$field.'</code>';
		}
		$checkedAndDisabled = '';
		if($field == 'uid' || $field == 'pid' || $field == 'RType'){
			$checkedAndDisabled = ' checked disabled ';
		}
		// Ранее было padding-left: 32px;
		return '<th nowrap style="padding: 2px; border-top: none;">
			<div style="position: relative;">
			<label class="btn btn-default btn-sm" style="width: 100%; margin: 0; padding: 0 5px 0 5px; text-align: left; cursor: pointer;">
				<input type="checkbox" class="btn btn-default btn-sm" style="margin-top: 0px;" name="field_uid" value="'.$field.'" onClick="toggleFields(this,\'field_uid\');" '.$checkedAndDisabled.' disabled_ZZZ>
				<b>'.$GLOBALS['LANG']->sL($config['label']).'</b>
			</label><br />
			'.$debugStr.'
			</div>
		</th>';
		
		// Старый вариант (немного другое оформление)
		/*
		return '<th nowrap style="border-top: none;">
			<div style="position: relative; margin-left: 26px;">
			<label class="btn btn-default btn-sm" style="position: absolute; min-width: 24px; top: -4px; left: -30px; margin: 0; padding: 0 5px 0 5px; cursor: pointer;">
				<input type="checkbox" class="btn btn-default btn-sm" style="margin-top: 0px;" name="field_uid" value="'.$field.'" onClick="toggleFields(this,\'field_uid\');" '.$checkedAndDisabled.' disabled_ZZZ>
			</label>
			'.$GLOBALS['LANG']->sL($config['label']).'<br />
			'.$debugStr.'
			</div>
		</th>';
		*/
		
		// return '<th nowrap>'.$config['label'].'</th>';
	}
	
	// listController func
	public static function listControllerHtmlTd(&$obj, $table, $field, $config, $row, &$uriBuilder){
		$backendLink = \TYPO3\CMS\Backend\Utility\BackendUtility::getModuleUrl(
			'record_edit', [
				'columnsOnly' => $field,
				'edit['.$table.']['.$row['uid'].']' => 'edit',
				'returnUrl' => \TYPO3\CMS\Core\Utility\GeneralUtility::getIndpEnv('REQUEST_URI')
			]
		);
		if(trim($row[$field]) == ''){
			$rowValue = '<code style="'.STYLE_EMPTY_FIELD.'">Заполнить</code>';
		} else {
			$rowValue = htmlspecialchars($row[$field]);
		}
		#if($field == 'uid'){
		#	return '<td nowrap style="vertical-align: top;"><a href="'.$backendLink.'">'.$rowValue.'</a></td>';
		#} else {
			return '<td nowrap style="vertical-align: top;"><a href="'.$backendLink.'">'.$rowValue.'</a></td>';
		#}
	}
}
