<?php
namespace Litovchenko\AirTable\Field;
use Litovchenko\AirTable\Field\Input;
use Litovchenko\AirTable\Utility\BaseUtility;

/**
 * @AirTable:<Field>
 * @AirTable\Parameter\Label:<Число целое>
 */
class InputNumber extends Input
{
    /**
     * @return string
     */
    public static function databaseDefinitions($model, $table, $field)
    {
        return [$table=>[
			$field => 'tinyint(4) unsigned DEFAULT \'0\' NOT NULL'
		]];
    }

    /**
     * @modify array
     */
    public static function buildConfiguration($model, $table, $field)
    {
		parent::buildConfiguration($model,$table,$field);
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['type'] = 'input';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['eval'] .= ',num';
    }
	
	// listController func
	public static function listControllerSqlBuilder(&$obj, &$q, $table, $field, $config){
		// $q->addSelect($table.'.'.$field);
		$paramFilter_select = BaseUtility::_POSTuc('form1Apply','form1Field_'.$field,'');
		$paramFilter_input = BaseUtility::_POSTuc('form1Apply','form1FieldValue_'.$field,'');
		$paramFilter_input = trim($paramFilter_input);
		switch($paramFilter_select){
			// case 'any': break;
			case 'EMPTY': 
				return $q->where($table.'.'.$field,'=',0); 
			break;
			case 'NOT EMPTY': 
				return $q->where($table.'.'.$field,'<>',0); 
			break;
			case '=': 
				return $q->where($table.'.'.$field,'=',$paramFilter_input); 
			break;
			case '!=': 
				return $q->where($table.'.'.$field,'<>',$paramFilter_input); 
			break;
			case '<': 
				return $q->where($table.'.'.$field,'<',$paramFilter_input); 
			break;
			case '<=': 
				return $q->where($table.'.'.$field,'<=',$paramFilter_input); 
			break;
			case '>': 
				return $q->where($table.'.'.$field,'>',$paramFilter_input); 
			break;
			case '>=': 
				return $q->where($table.'.'.$field,'>=',$paramFilter_input); 
			break;
			case 'IN': 
				$array = explode(',',$paramFilter_input);
				return $q->whereIn($table.'.'.$field,$array); 
			break;
			case 'NOT IN': 
				$array = explode(',',$paramFilter_input);
				return $q->whereNotIn($table.'.'.$field,$array); 
			break;
			case 'BETWEEN': 
				$array = explode(',', $paramFilter_input);
				// $array[0] start
				// $array[1] end
				return $q->whereBetween($table.'.'.$field,[$array[0],$array[1]]); 
			break;
			case 'NOT BETWEEN': 
				$array = explode(',', $paramFilter_input);
				// $array[0] start
				// $array[1] end
				return $q->whereNotBetween($table.'.'.$field,[$array[0],$array[1]]); 
			break;
		}
		return $q;
	}
	
	// listController func
	public static function listControllerHtmlFilter(&$obj, $table, $field, $config, &$uriBuilder){
		$paramFilter_select = BaseUtility::_POSTuc('form1Apply','form1Field_'.$field,'');
		$paramFilter_input = BaseUtility::_POSTuc('form1Apply','form1FieldValue_'.$field,'');
		$paramFilter_input = htmlspecialchars($paramFilter_input);
		$paramFilter_input = stripslashes($paramFilter_input);
		return '
			<table width="100%" class="" style="margin-bottom: 0;">
			<tr>
			<td width="30%">
				<select name="form1Field_'.$field.'" class="form-control btn-sm">
					<option value="" '.($paramFilter_select == ''?'selected':'').'>Любое значение</option>
					<option value="EMPTY" '.($paramFilter_select == 'EMPTY'?'selected':'').'>Не заполнено (ноль)</option>
					<option value="NOT EMPTY" '.($paramFilter_select == 'NOT EMPTY'?'selected':'').'>Заполнено (не ноль)</option>
					<option value="=" '.($paramFilter_select == '='?'selected':'').'>=</option>
					<option value="!=" '.($paramFilter_select == '!='?'selected':'').'>!=</option>
					<option value="<" '.($paramFilter_select == '<'?'selected':'').'>&lt;</option>
					<option value="<=" '.($paramFilter_select == '<='?'selected':'').'>&lt;=</option>
					<option value=">" '.($paramFilter_select == '>'?'selected':'').'>&gt;</option>
					<option value=">=" '.($paramFilter_select == '>='?'selected':'').'>&gt;=</option>
					<option value="IN" '.($paramFilter_select == 'IN'?'selected':'').'>IN (значения через ",")</option>
					<option value="NOT IN" '.($paramFilter_select == 'NOT IN'?'selected':'').'>NOT IN (значения через ",")</option>
					<option value="BETWEEN" '.($paramFilter_select == 'BETWEEN'?'selected':'').'>BETWEEN (два значения через ",")</option>
					<option value="NOT BETWEEN" '.($paramFilter_select == 'NOT BETWEEN'?'selected':'').'>NOT BETWEEN (два значения через ",")</option>
				</select>
			</td>
			<td width="70%" style="padding-left: 5px;">
				<input name="form1FieldValue_'.$field.'" class="form-control btn-sm" value="'.$paramFilter_input.'" placeholder="Введите значение">
			</td>
			</tr>
			</table>
		';
	}
}