<?php
namespace Litovchenko\AirTable\Field;
use Litovchenko\AirTable\Field\BaseFieldRelelation;
use Litovchenko\AirTable\Utility\BaseUtility;

/**
 * @AirTable:<Field>
 * @AirTable\Parameter\Label:<M:1 Связь многие к одному>
 * @AirTable\Parameter\Description:<Информация о связи пишется в колонку текущей таблицы. Доступно три варианта конфигурации: default, db, tree>
 */
class Rel_Mto1 extends BaseFieldRelelation
{
	const IMPORT = true;
	
    const REQPOSTFIXCURRENTFIELD = '_id';

    const REQPOSTFIXFOREIGNFIELD = '_many';
	
    /**
     * @var string
     */
    protected $RenderType = 'select'; // select || db || tree

    /**
     * @var string
     */
    protected $ParentKey = 'pid';
	
    /**
     * @return string
     */
    public static function databaseDefinitions($model, $table, $field)
    {
		$sql = []; // Имеем в виду, что может быть связь "На себя"
		$sql[$table][$field] = 'int(11) DEFAULT \'0\' NOT NULL';
		// $sql[parent::getForeignTableName($model,$field)][parent::getForeignFieldName($model,$field)] = 'int(11) DEFAULT \'0\' NOT NULL';
		return $sql;
    }
	
    /**
     * @modify array
     */
    public static function buildConfiguration($model, $table, $field)
    {
		parent::buildConfiguration($model, $table, $field);
		$GLOBALS['TCA'][$table]['columns'][$field]['label'] = $GLOBALS['TCA'][$table]['columns'][$field]['label'].' M:1';
		if(self::isDoubleRelation($model, $field)){
			$GLOBALS['TCA'][$table]['columns'][$field]['label'] .= '↔';
		}

		// Fix Typo3 bug (про создании записей для "renderType->selectTree" не работает defVals)
		if(isset($GLOBALS['_GET']['defVals'][$table][$field]) && $field == 'parent_id'){
			$renderType = 'select';
		
		// Fix Typo3 bug (про создании записей для "renderType->selectTree" не работает defVals)
		}elseif(isset($GLOBALS['_GET']['defVals'][$table][$field]) && $field == 'category_id'){
			$renderType = 'select';
		
		// Без бага
		} else {
			$renderType = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\RenderType');
		}
		
		switch($renderType){
			default:
			case 'select':
				self::buildConfigurationRelationSelect($model, $table, $field);
			break;
			case 'db':
				self::buildConfigurationRelationDb($model, $table, $field);
			break;
			case 'tree':
				self::buildConfigurationRelationTree($model, $table, $field);
			break;
		}
	}
	
    /**
     * @return array
     */
    public static function buildConfigurationRelationSelect($model, $table, $field)
    {
		$_foreignModelName = self::getForeignModelName($model, $field); // foreign model
		$_foreignTableName = self::getForeignTableName($model, $field); // foreign table
		if(BaseUtility::hasClassTrait($_foreignModelName,'Litovchenko\AirTable\Field\Traits\RType') == true) {
			if(BaseUtility::hasClassTrait($_foreignModelName,'Litovchenko\AirTable\Field\Traits\Title') == true){
				$sort = ' ORDER BY RType ASC, title ASC ';
			} else {
				$sort = ' ORDER BY RType ASC, uid ASC ';
			}
		} elseif(BaseUtility::hasClassTrait($_foreignModelName,'Litovchenko\AirTable\Field\Traits\Title') == true) {
			$sort = ' ORDER BY title ASC ';
		} else {
			$sort = ' ORDER BY uid ASC ';
		}
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['type'] = 'select';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['renderType'] = 'selectSingle';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['items'] = [
			0 => [
				0 => '-- Выберите значение --',
				1 => 0
			]
		];
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['minitems'] = 0;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['maxitems'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['default'] = 0;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['size'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['foreign_table'] = $_foreignTableName; // -> Устанавливать автоматически
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['foreign_table_where'] = $sort; // -> Устанавливать автоматически
    }
	
    /**
     * @return array
     */
    public static function buildConfigurationRelationDb($model, $table, $field)
    {
		$_foreignModelName = self::getForeignModelName($model, $field); // foreign model
		$_foreignTableName = self::getForeignTableName($model, $field); // foreign table
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['type'] = 'group';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['internal_type'] = 'db';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['allowed'] = $_foreignTableName; // -> Устанавливать автоматически
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['minitems'] = 0;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['maxitems'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['default'] = 0;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['size'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['hideSuggest'] = 1;
		#'suggestOptions' => [
		#	'default' => [
		#		'pidList' => 0,
		#		'pidDepth' => 0,
		#		'maxItemsInResultList' => 100,
		#		'searchWholePhrase' => 1,
		#	]
		#],
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['fieldWizard']['recordsOverview']['disabled'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['fieldControl'] = [
			'addRecord' => [
				'disabled' => 0,
				'renderType' => 'addRecord',
				'options' => [
					#'pid' => 0, // -> Устанавливать автоматически
					'table' => $_foreignTableName, // -> Устанавливать автоматически
				]
			],
			'editPopup' => [
				'disabled' => 1,
			],
			'listModule' => [
				'disabled' => 1,
				],
			'elementBrowser' => [
				'disabled' => 1
			]
		];
    }
	
    /**
     * @return array
     */
    public static function buildConfigurationRelationTree($model, $table, $field)
    {
		$_foreignModelName = self::getForeignModelName($model, $field); // foreign model
		$_foreignTableName = self::getForeignTableName($model, $field); // foreign table
		$parentKey = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\ParentKey');
		if(BaseUtility::hasClassTrait($_foreignModelName,'Litovchenko\AirTable\Field\Traits\RType') == true) {
			if(BaseUtility::hasClassTrait($_foreignModelName,'Litovchenko\AirTable\Field\Traits\Title') == true){
				$sort = ' ORDER BY RType ASC, title ASC ';
			} else {
				$sort = ' ORDER BY RType ASC, uid ASC ';
			}
		} elseif(BaseUtility::hasClassTrait($_foreignModelName,'Litovchenko\AirTable\Field\Traits\Title') == true) {
			$sort = ' ORDER BY title ASC ';
		} else {
			$sort = ' ORDER BY uid ASC ';
		}
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['type'] = 'select';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['renderType'] = 'selectTree';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['enableMultiSelectFilterTextfield'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['minitems'] = 0;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['maxitems'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['default'] = 0;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['foreign_table'] = $_foreignTableName; // -> Устанавливать автоматически
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['foreign_table_where'] = $sort; // -> Устанавливать автоматически
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['treeConfig'] = [
			'parentField' => $parentKey!=''?$parentKey:'parent_id', // -> Устанавливать автоматически
			'appearance' => [
				'expandAll' => 0,
				'showHeader' => 1
			]
		];
    }
	
	// Автоматизированная выборка связей 
	public function linkApproach($obj, $class, $field){
		$annotationForeignModel = BaseUtility::getClassFieldAnnotationValueNew($class,$field,'AirTable\Field\Parameter\ForeignModel');
		$annotationForeignKey = BaseUtility::getClassFieldAnnotationValueNew($class,$field,'AirTable\Field\Parameter\ForeignKey');
		return $obj->belongsTo($annotationForeignModel, $field, 'uid');
	}
	
	// Создание связей
	public function linkCreate($obj, $class, $field, $idTwo = null, $data = []){
		$obj->{$field}()->associate(intval($idTwo)); // belongsTo
		return $obj->save();
	}
	
	// Удаление связей
	public function linkRemove($obj, $class, $field, $idTwo = null, $data = []){
		$obj->{$field}()->dissociate(); // belongsTo
		return $obj->save();
	}

}