<?php
namespace Litovchenko\AirTable\Field;
use Litovchenko\AirTable\Field\BaseFieldRelelation;
use Litovchenko\AirTable\Utility\BaseUtility;

/**
 * @AirTable:<Field>
 * @AirTable\Parameter\Label:<M:M Связь многие ко многим>
 * @AirTable\Parameter\Description:<Информация о связи пишется в промежуточную таблицу "sys_mm". Доступно три варианта конфигурации: default, db, tree>
 */
class Rel_MtoM extends BaseFieldRelelation
{
	const IMPORT = true;
	
    const REQPOSTFIXCURRENTFIELD = '_many';

    const REQPOSTFIXFOREIGNFIELD = '_many';
	
    /**
     * @var string
     */
    protected $RenderType = 'select'; // select || db || tree
	
    /**
     * @var string
     */
    protected $ParentKey = 'pid';

    /**
     * @return string
     */
    public static function databaseDefinitions($model, $table, $field)
    {
		$sql = []; // Имеем в виду, что может быть связь "На себя"
		$sql[$table][$field] = 'int(11) DEFAULT \'0\' NOT NULL';
		// $sql[parent::getForeignTableName($model,$field)][parent::getForeignFieldName($model,$field)] = 'int(11) DEFAULT \'0\' NOT NULL';
		return $sql;
    }
	
    /**
     * @modify array
     */
    public static function buildConfiguration($model, $table, $field)
    {
		parent::buildConfiguration($model, $table, $field);
		$GLOBALS['TCA'][$table]['columns'][$field]['label'] = $GLOBALS['TCA'][$table]['columns'][$field]['label'].' M:M';
		if(self::isDoubleRelation($model, $field)){
			$GLOBALS['TCA'][$table]['columns'][$field]['label'] .= '↔';
		}
		
		// Fix Typo3 bug (про создании записей для "renderType->selectTree" не работает defVals)
		if(isset($GLOBALS['_GET']['defVals'][$table][$field]) && $field == 'category_many'){
			$renderType = 'select';
		
		// Без бага
		} else {
			$renderType = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\RenderType');
		}
		
		switch($renderType){
			default:
			case 'select':
				self::buildConfigurationRelationSelect($model, $table, $field);
			break;
			case 'db':
				self::buildConfigurationRelationDb($model, $table, $field);
			break;
			case 'tree':
				self::buildConfigurationRelationTree($model, $table, $field);
			break;
		}
	}
	
    /**
     * @modify array
     */
    public static function postBuildConfiguration($model, $table, $field)
    {
		// Добавляем информацию в таблицу "sys_mm"
		// $tableLabel = BaseUtility::getClassAnnotationValueNew($model,'AirTable\Parameter\Label');
		// $GLOBALS['TCA']['sys_mm']['columns']['tablename']['config']['items'][$model] = [0=>$tableLabel,1=>$model];
	}
	
    /**
     * @return array
     */
    public static function buildConfigurationRelationSelect($model, $table, $field)
    {
		$_foreignModelName = self::getForeignModelName($model, $field); // foreign model
		$_foreignTableName = self::getForeignTableName($model, $field); // foreign table
		if(BaseUtility::hasClassTrait($_foreignModelName,'Litovchenko\AirTable\Field\Traits\RType') == true) {
			if(BaseUtility::hasClassTrait($_foreignModelName,'Litovchenko\AirTable\Field\Traits\Title') == 1){
				$sort = ' ORDER BY RType ASC, title ASC ';
			} else {
				$sort = ' ORDER BY RType ASC, uid ASC ';
			}
		} elseif(BaseUtility::hasClassTrait($_foreignModelName,'Litovchenko\AirTable\Field\Traits\Title') == true) {
			$sort = ' ORDER BY title ASC ';
		} else {
			$sort = ' ORDER BY uid ASC ';
		}
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['type'] = 'select';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['renderType'] = 'selectMultipleSideBySide';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['enableMultiSelectFilterTextfield'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['minitems'] = 0;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['maxitems'] = 100;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['size'] = 10;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['foreign_table'] = $_foreignTableName; // -> Устанавливать автоматически
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['foreign_table_where'] = $sort; // -> Устанавливать автоматически
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['MM'] = 'sys_mm';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['MM_hasUidField'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['MM_match_fields'] = [
			'tablename' => $model, // -> Устанавливать автоматически
			'fieldname' => $field, // -> Устанавливать автоматически
			// 'indent' => '' // произвольные данные
		];
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['fieldControl'] = [
			'addRecord' => [
				'disabled' => 0,
				'renderType' => 'addRecord',
				'options' => [
					#'pid' => 0, // -> Устанавливать автоматически
					'table' => $_foreignTableName, // -> Устанавливать автоматически
				]
			],
			'editPopup' => [
				'disabled' => 1,
			],
			'listModule' => [
				'disabled' => 1,
			],
			'elementBrowser' => [
				'disabled' => 1
			]
		];
    }
	
    /**
     * @return array
     */
    public static function buildConfigurationRelationDb($model, $table, $field)
    {
		$_foreignModelName = self::getForeignModelName($model, $field); // foreign model
		$_foreignTableName = self::getForeignTableName($model, $field); // foreign table
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['type'] = 'group';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['internal_type'] = 'db';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['allowed'] = $_foreignTableName; // -> Устанавливать автоматически
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['MM'] = 'sys_mm';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['MM_hasUidField'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['MM_match_fields'] = [
			'tablename' => $model, // -> Устанавливать автоматически
			'fieldname' => $field, // -> Устанавливать автоматически
			// 'indent' => '' // произвольные данные
		];
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['minitems'] = 0;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['maxitems'] = 100;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['size'] = 10;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['hideSuggest'] = 1;
		#'suggestOptions' => [
		#	'default' => [
		#		'pidList' => 0,
		#		'pidDepth' => 0,
		#		'maxItemsInResultList' => 100,
		#		'searchWholePhrase' => 1,
		#	]
		#],
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['fieldWizard']['recordsOverview']['disabled'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['fieldControl'] = [
			'addRecord' => [
				'disabled' => 0,
				'renderType' => 'addRecord',
				'options' => [
					#'pid' => 0, // -> Устанавливать автоматически
					'table' => $_foreignTableName, // -> Устанавливать автоматически
				]
			],
			'editPopup' => [
				'disabled' => 1,
			],
			'listModule' => [
				'disabled' => 1,
			],
			'elementBrowser' => [
				'disabled' => 1
			]
		];
    }
	
    /**
     * @return array
     */
    public static function buildConfigurationRelationTree($model, $table, $field)
    {
		$_foreignModelName = self::getForeignModelName($model, $field); // foreign model
		$_foreignTableName = self::getForeignTableName($model, $field); // foreign table
		$parentKey = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\ParentKey');
		if(BaseUtility::hasClassTrait($_foreignModelName,'Litovchenko\AirTable\Field\Traits\RType') == true) {
			if(BaseUtility::hasClassTrait($_foreignModelName,'Litovchenko\AirTable\Field\Traits\Title') == 1){
				$sort = ' ORDER BY RType ASC, title ASC ';
			} else {
				$sort = ' ORDER BY RType ASC, uid ASC ';
			}
		} elseif(BaseUtility::hasClassTrait($_foreignModelName,'Litovchenko\AirTable\Field\Traits\Title') == true) {
			$sort = ' ORDER BY title ASC ';
		} else {
			$sort = ' ORDER BY uid ASC ';
		}
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['type'] = 'select';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['renderType'] = 'selectTree';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['enableMultiSelectFilterTextfield'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['minitems'] = 0;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['maxitems'] = 100;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['foreign_table'] = $_foreignTableName; // -> Устанавливать автоматически
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['foreign_table_where'] = $sort; // -> Устанавливать автоматически
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['MM'] = 'sys_mm';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['MM_hasUidField'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['MM_match_fields'] = [
			'tablename' => $model, // -> Устанавливать автоматически
			'fieldname' => $field, // -> Устанавливать автоматически
			// 'indent' => '' // произвольные данные
		];
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['treeConfig'] = [
			'parentField' =>$parentKey!=''?$parentKey:'parent_id', // -> Устанавливать автоматически
			'appearance' => [
				'expandAll' => 0,
				'showHeader' => 1
			]
		];
    }
	
	// Автоматизированная выборка связей 
	public function linkApproach($obj, $class, $field){
		$annotationForeignModel = BaseUtility::getClassFieldAnnotationValueNew($class,$field,'AirTable\Field\Parameter\ForeignModel');
		$annotationForeignKey = BaseUtility::getClassFieldAnnotationValueNew($class,$field,'AirTable\Field\Parameter\ForeignKey');
		return $obj->belongsToMany($annotationForeignModel, 'sys_mm', 'uid_local', 'uid_foreign')
			->where('sys_mm.tablename', '=', $class) // BaseUtility::getTableNameFromClass()
				->where('sys_mm.fieldname', '=', $field)
					->orderBy('sys_mm.sorting', 'asc');
	}
	
	// Создание связей
	public function linkCreate($obj, $class, $field, $idTwo = null, $data = []){
		// belongsToMany
		$arSync = [];
		$indentArray = [
			'tablename' => get_class($obj), 
			'fieldname' => preg_replace('/_Method$/is','',$field)
		];
		#if(is_array($idTwo)){
		#	foreach($idTwo as $k => $id){
		#		$arSync[$id] = $indentArray;
		#		$arSync[$id]['sorting'] = $k;
		#	}
		#} else {
			$arSync[$idTwo] = $indentArray;
			$arSync[$idTwo]['sorting'] = 1;
		#}
		$obj->{$field}()->syncWithoutDetaching($arSync);
		return $obj->save();
	}
	
	// Удаление связей
	public function linkRemove($obj, $class, $field, $idTwo = null, $data = []){
		// belongsToMany
		$obj->{$field}()
			->wherePivot('tablename', '=', get_class($obj))
				->wherePivot('fieldname', '=', preg_replace('/_Method$/is','',$field))
					->detach($idTwo); // Если = "0" - удаляеться все связи
		return $obj->save();
	}

}