<?php
namespace Litovchenko\AirTable\Field;
use Litovchenko\AirTable\Field\BaseFieldRelelation;
use Litovchenko\AirTable\Utility\BaseUtility;

/**
 * @AirTable:<Field>
 * @AirTable\Parameter\Label:<Полиморфная связь 1:1>
 * @AirTable\Parameter\Description:<Информация о связи пишется в 4 колонки внешней таблицы>
 */
class Rel_Poly_1to1 extends BaseFieldRelelation
{
    const REQPOSTFIXCURRENTFIELD = '_one';

    const REQPOSTFIXFOREIGNFIELD = '_one';
	
    /**
     * @return string
     */
    public static function databaseDefinitions($model, $table, $field)
    {
		$sql = []; // Имеем в виду, что может быть связь "На себя"
		$sql[$table][$field] = 'int(11) DEFAULT \'0\' NOT NULL';
		$sql[parent::getForeignTableName($model,$field)]['foreign_table'] = 'varchar(255) DEFAULT \'\' NOT NULL';
		$sql[parent::getForeignTableName($model,$field)]['foreign_field'] = 'varchar(255) DEFAULT \'\' NOT NULL';
		$sql[parent::getForeignTableName($model,$field)]['foreign_uid'] = 'int(11) DEFAULT \'0\' NOT NULL';
		$sql[parent::getForeignTableName($model,$field)]['foreign_sortby'] = 'int(11) DEFAULT \'0\' NOT NULL';
		// $sql[parent::getForeignTableName($model,$field)][parent::getForeignFieldName($model,$field)] = 'int(11) DEFAULT \'0\' NOT NULL';
		return $sql;
    }
	
    /**
     * @modify array
     */
    public static function buildConfiguration($model, $table, $field)
    {
		parent::buildConfiguration($model, $table, $field);
		self::buildConfigurationRelation($model, $table, $field);
		
		$GLOBALS['TCA'][$table]['columns'][$field]['label'] = $GLOBALS['TCA'][$table]['columns'][$field]['label'].' POLY-1:1';
	}

    /**
     * @return array
     */
    public static function buildConfigurationRelation($model, $table, $field)
    {
		if(BaseUtility::hasClassTrait(parent::getForeignModelName($model,$field),'Litovchenko\AirTable\Field\Traits\Sorting') == true
			or parent::getForeignTableName($model,$field) == 'tt_content' // Специально для таблицы "tt_content" - устанавливаем сортировку по полю "sorting"
		){
			$sort = 'sorting';
		} else {
			$sort = '';
		}
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['type'] = 'inline';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['minitems'] = 0;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['maxitems'] = 1;
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['behaviour'] = [
			'enableCascadingDelete' => false, // Запрещяем каскадное удаление
		];
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['appearance'] = [
			#'collapseAll' => 1,
			'expandSingle' => 0,
			'newRecordLinkAddTitle' => 1,
			'levelLinksPosition' => 'top',
			'showPossibleLocalizationRecords' => 0,
			'showRemovedLocalizationRecords' => 0,
			'showAllLocalizationLink' => 0,
			'showSynchronizationLink' => 0,
			'useSortable' => 1,
			'enabledControls' => [
				'info' => 1,
				'new' => 1,
				'dragdrop' => 1,
				'sort' => 1,
				'hide' => 1,
				'delete' => 1,
				'localize' => 0
			]
		];
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['foreign_table'] = parent::getForeignTableName($model,$field); // -> Устанавливать автоматически
		
		// Лишнее поле - с ним не получилось записывать не таблицу, а модель
		// $GLOBALS['TCA'][$table]['columns'][$field]['config']['foreign_table_field'] = 'foreign_table'; // -> Поле куда пишется название таблицы
		
		// -> Сопоставление записи
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['foreign_match_fields'] = [
			'foreign_table' => $model,
			'foreign_field' => $field
		];
		
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['foreign_field'] = 'foreign_uid'; // -> Поле куда пишется ID
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['foreign_sortby'] = $sort; // -> Поле определения сортировки
    }
	
    /**
     * @modify array
     */
    public static function postBuildConfiguration($model, $table, $field)
    {
		// Добавляем информацию во внешнуюю таблицу
		if(BaseUtility::hasClassTrait(parent::getForeignModelName($model,$field),'Litovchenko\AirTable\Field\Traits\RelPolyRevers') == true){
			#$tableLabel = BaseUtility::getClassAnnotationValueNew($model,'AirTable\Parameter\Label');
			#$GLOBALS['TCA'][parent::getForeignTableName($model,$field)]['columns']['foreign_table']['config']['items'][$model] = [
			#	0=>$tableLabel,
			#	1=>$model
			#];
		}
	}
	
	// Автоматизированная выборка связей 
	public function linkApproach($obj, $class, $field){
		$annotationForeignModel = BaseUtility::getClassFieldAnnotationValueNew($class,$field,'AirTable\Field\Parameter\ForeignModel');
		$annotationForeignKey = BaseUtility::getClassFieldAnnotationValueNew($class,$field,'AirTable\Field\Parameter\ForeignKey');
		// morphOne($related, $name, $type = null, $id = null, $localKey = null)
		return $obj->morphOne(
			$annotationForeignModel,
			'getDefaultRelation_'.$field.'_Method',
			'foreign_table',
			'foreign_uid',
			'uid'
		)->where('foreign_field', '=', $field);
	}
	
	// Создание связей
	public function linkCreate($obj, $class, $field, $idTwo = null, $data = []){} //Todo
	
	// Удаление связей
	public function linkRemove($obj, $class, $field, $idTwo = null, $data = []){} //Todo

}