<?php
namespace Litovchenko\AirTable\Field;
use Litovchenko\AirTable\Field\BaseField;
use Litovchenko\AirTable\Utility\BaseUtility;

/**
 * @AirTable:<Field>
 * @AirTable\Parameter\Label:<Логический переключатель Int (между набором представленных вариантов)>
 * @AirTable\Parameter\Description:<По умолчанию "пустая строка">
 */
class SwitchInt extends BaseField
{
    /**
     * @return string
     */
    public static function databaseDefinitions($model, $table, $field)
    {
        return [$table=>[
			$field => 'smallint UNSIGNED DEFAULT \'0\' NOT NULL'
		]];
    }
	
    /**
     * @return array
     */
    public static function parameterItems($class, $property, $keyAnnotation, $value)
    {
		$keyRealName = end(explode('\\',$keyAnnotation));
        return [$keyRealName => [
				0=>($value),
				1=>trim($keyRealName)
			]
		];
    }

    /**
     * @modify array
     */
    public static function buildConfiguration($model, $table, $field)
    {
		parent::buildConfiguration($model,$table,$field);
		$Items = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\Items');
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['type'] = 'radio';
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['items'] = [];
		$GLOBALS['TCA'][$table]['columns'][$field]['config']['items'] = $Items;
		
		// Значение по умолчанию
		$default = BaseUtility::getClassFieldAnnotationValueNew($model,$field,'AirTable\Field\Parameter\Default');
		if(!empty($default)){
			$GLOBALS['TCA'][$table]['columns'][$field]['config']['default'] = $default;
		} else {
			$GLOBALS['TCA'][$table]['columns'][$field]['config']['default'] = 0;
		}
    }
	
	// listController func
	public static function listControllerSqlBuilder(&$obj, &$q, $table, $field, $config){
		// $q->addSelect($table.'.'.$field);
		$paramFilter = BaseUtility::_POSTuc('form1Apply','form1Field_'.$field,'');
		
		// Включая (логика OR):
		$q = $q->where(function($q) use ($table,$field,$paramFilter){
			foreach($paramFilter as $k => $v){
				$temp = explode('|',$v);
				if($v == 'INCLUDE_EMPTY'){
					$q->orWhere($table.'.'.$field,'=','');
				}
				if($temp[0] == 'INCLUDE'){
					$q->orWhere($table.'.'.$field,'=',$temp[1]);
				}
			}
		});
		
		// Исключая (логика AND):
		$q = $q->where(function($q) use ($table,$field,$paramFilter){
			foreach($paramFilter as $k => $v){
				$temp = explode('|',$v);
				if($v == 'EXCLUDE_EMPTY'){
					$q->where($table.'.'.$field,'!=','');
					}
				if($temp[0] == 'EXCLUDE'){
					$q->where($table.'.'.$field,'!=',$temp[1]);
				}
			}
		});
		
		return $q;
	}
	
	// listController func
	public static function listControllerSqlBuilderOrder(){
		return true;
	}
	
	// listController func
	public static function listControllerHtmlFilter(&$obj, $table, $field, $config, &$uriBuilder){
		$paramFilter = BaseUtility::_POSTuc('form1Apply','form1Field_'.$field,'');
		
		/*
		$checkbox = '';
		$items = $config['config']['items'];
		foreach($items as $k => $v){
			if(trim($v[1]) != ''){
				$checkbox .= '
				<div style="margin-left: 15px;">
				<label class="btn btn-default btn-sm" style="margin: 0; margin-bottom: 5px; padding: 0 7px 0 7px; width: 100%; text-align: left; cursor: pointer;">
					<input type="checkbox" name="tab1['.$field.']" class="btn btn-default btn-sm" style="margin-top: 0px;" value="= '.$v[1].'" '.($paramFilter == '= '.$v[1]?'checked':'').'> 
					= '.$v[0].'
				</label>
				</div>
				';
			}
		}
		*/
		
		$items = $config['config']['items'];
		$select = '<div style="">';
		$select .= '<select name="form1Field_'.$field.'[]" class="form-control btn-sm" size="'.((count($items)*2)+5).'" multiple>';
		
		$select .= '<optgroup label="Включая (логика OR):">';
		$select .= '<option value="INCLUDE_EMPTY" '.(in_array('INCLUDE_EMPTY',$paramFilter)?'selected':'').'>Значение не выбрано</option>';
		foreach($items as $k => $v){
			if(trim($v[1]) != ''){
				$select .= '<option value="INCLUDE|'.$v[1].'" '.(in_array('INCLUDE|'.$v[1],$paramFilter)?'selected':'').'><!--['.$v[1].']--> '.$GLOBALS['LANG']->sL($v[0]).'</option>';
			}
		}
		$select .= '</optgroup>';
		$select .= '<optgroup label="Исключая (логика AND):">';
		$select .= '<option value="EXCLUDE_EMPTY" '.(in_array('EXCLUDE_EMPTY',$paramFilter)?'selected':'').'>Значение не выбрано</option>';
		foreach($items as $k => $v){
			if(trim($v[1]) != ''){
				$select .= '<option value="EXCLUDE|'.$v[1].'" '.(in_array('EXCLUDE|'.$v[1],$paramFilter)?'selected':'').'><!--['.$v[1].']--> '.$GLOBALS['LANG']->sL($v[0]).'</option>';
			}
		}
		$select .= '</optgroup>';
		$select .= '</select>';
		$select .= '</div>';

		return $select;
		/* return '
			<!--
			<label class="btn btn-default btn-sm" style="margin: 0; margin-bottom: 5px; padding: 0 7px 0 7px; width: 100%; text-align: left; cursor: pointer;">
				<input type="radio" name="tab1['.$field.']" class="btn btn-default btn-sm" style="margin-top: 0px;" value="" '.($paramFilter == ''?'checked':'').'> 
				Любое значение
			</label>
			<label class="btn btn-default btn-sm" style="margin: 0; margin-bottom: 5px; padding: 0 7px 0 7px; width: 100%; text-align: left; cursor: pointer;">
				<input type="radio" name="tab1['.$field.']" class="btn btn-default btn-sm" style="margin-top: 0px;" value="EMPTY" '.($paramFilter == 'EMPTY'?'checked':'').'> 
				Значение не выбрано (пустое поле)
			</label>
			<label class="btn btn-default btn-sm" style="margin: 0; margin-bottom: 5px; padding: 0 7px 0 7px; width: 100%; text-align: left; cursor: pointer;">
				<input type="radio" name="tab1['.$field.']" class="btn btn-default btn-sm" style="margin-top: 0px;" value="NOT EMPTY" '.($paramFilter == 'NOT EMPTY'?'checked':'').'> 
				Значение выбрано (не пустое поле)
			</label>
			<label class="btn btn-default btn-sm" style="margin: 0; margin-bottom: 5px; padding: 0 7px 0 7px; width: 100%; text-align: left; cursor: pointer;">
				<input type="radio" name="tab1['.$field.']" class="btn btn-default btn-sm" style="margin-top: 0px;" value="VALUE LIST" '.($paramFilter == 'VALUE LIST'?'checked':'').'> 
				Выбрать со следующими значениями:
			</label>
			<label class="btn btn-default btn-sm" style="margin: 0; margin-bottom: 5px; padding: 0 7px 0 7px; width: 100%; text-align: left; cursor: pointer;">
				<input type="radio" name="tab1['.$field.']" class="btn btn-default btn-sm" style="margin-top: 0px;" value="NOT VALUE LIST" '.($paramFilter == 'NOT VALUE LIST'?'checked':'').'> 
				Ни одно из существующих значений
			</label>
			-->
		';
		*/
	}
	
	// listController func
	public static function listControllerHtmlTh(&$obj, $table, $field, $config){
		return \Litovchenko\AirTable\Field\Input::listControllerHtmlTh($obj, $table, $field, $config);
	}
	
	// listController func
	public static function listControllerHtmlTd(&$obj, $table, $field, $config, $row, &$uriBuilder){
		$backendLink = \TYPO3\CMS\Backend\Utility\BackendUtility::getModuleUrl(
			'record_edit', [
				'columnsOnly' => $field,
				'edit['.$table.']['.$row['uid'].']' => 'edit',
				'returnUrl' => \TYPO3\CMS\Core\Utility\GeneralUtility::getIndpEnv('REQUEST_URI')
			]
		);
		if(trim($row[$field]) != ''){
			$temp = BaseUtility::getTcaFieldItem($table,$field,$row[$field]);
			$item_n = $temp[0];
			$item_v = $temp[1];
			if($item_n == '' && $item_v == ''){
				return '<td nowrap style="vertical-align: top;"><a href="'.$backendLink.'"><code>[? '.$row[$field].']</code></a></td>';
			} else {
				$debugStr = '';
				if($GLOBALS['TYPO3_CONF_VARS']['BE']['debug'] == true){
					$debugStr = '<code>['.$item_v.']</code> ';
				}
				return '<td nowrap style="vertical-align: top;"><a href="'.$backendLink.'">'.$debugStr.$GLOBALS['LANG']->sL($item_n).'</a></td>';
			}
		} else {
			return '<td nowrap style="vertical-align: top;"><a href="'.$backendLink.'"><code style="'.STYLE_EMPTY_FIELD.'">Значение не выбрано</code></a></td>';
		}
	}

}
